
import gurobi.GRB;
import gurobi.GRBException;
import gurobi.GRBVar;

import java.util.ArrayList;

public class FixedVariables {
	public double[][]y_i;
	public double[][][]y;
	public double[][]y_o;
	public double[][]y_d;
	public double[][][]x_Bo;
	public double[][][]x;
	public double[][][][]x_B;
	public double[]dummy;
	public double[]t;
	public double[]d;
	public double[][][][]l;
	public double[][][]z;
	Problem problem; 
	
	public FixedVariables(Problem problem){
		this.y_i 	= new double[problem.vessels.size()][problem.positions.size()];
		this.y 		= new double[problem.vessels.size()][problem.positions.size()][problem.positions.size()];
		this.y_o 	= new double[problem.vessels.size()][problem.positions.size()];
		this.y_d 	= new double[problem.vessels.size()][problem.positions.size()];
		this.x_Bo 	= new double[problem.vessels.size()][problem.positions.size()][problem.speedProfiles.size()];
		this.x		= new double[problem.vessels.size()][problem.positions.size()][problem.speedProfiles.size()];
		this.x_B	= new double[problem.vessels.size()][problem.positions.size()][problem.positions.size()][problem.speedProfiles.size()];
		this.dummy	= new double[problem.positions.size()];
		this.t		= new double[problem.positions.size()];
		this.d		= new double[problem.positions.size()];
		this.l		= new double[problem.positions.size()][problem.balanceCategories.size()][problem.deckCategories.size()][problem.deckCategories.size()];
		this.z		= new double[problem.balanceCategories.size()][problem.deckCategories.size()][problem.months.size()];
		this.problem = problem;
	}
	
		
	public void reset(){
		for (int v = 0; v < problem.vessels.size(); v++ ){
				for (int i = 0; i < problem.positions.size(); i++){
					y_i[v][i] = -1;
					y_o[v][i] = -1;
					y_d[v][i] = -1;
					for (int s = 0; s < problem.speedProfiles.size(); s++){
						x_Bo[v][i][s] 	= -1;
						x[v][i][s] 		= -1;
					}
					for (int j = 0; j < problem.positions.size(); j++){
						y[v][i][j] = -1;
						for (int s = 0; s < problem.speedProfiles.size(); s++){
							x_B[v][i][j][s] = -1;
						}
					}
					dummy[i] 	= -1;
					t[i] 		= -1;
					d[i]		= -1;
					for (int b = 0; b < problem.balanceCategories.size(); b++){
			      		for (int p = 0; p < problem.deckCategories.size(); p++){
			      			for (int k = 0; k < problem.deckCategories.size(); k++){
			      					l[i][b][p][k] = -1;
			      			}
			      			for (int m = 0; m < problem.months.size(); m++){
			      				z[b][p][m] = -1;
			      			}
			      		}
					}	
				}		
			}
	}
	
	public void addPrimary(int counter, GRBVar[][] y_iPV, GRBVar[][] y_oPV, GRBVar[][][] yPV){
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (isPositionInPrimaryPeriod(counter, problem.positions.get(i), problem.months)){
					try {
						if(y_iPV[v][i] != null){
						y_i[v][i]= y_iPV[v][i].get(GRB.DoubleAttr.X);
						}
						if(y_oPV[v][i] != null){
						y_o[v][i]= y_oPV[v][i].get(GRB.DoubleAttr.X);
						}
					}catch (GRBException e){
						e.printStackTrace();
					}
				}	
				for (int j = 0; j < problem.positions.size(); j++){
					try {
						if (yPV[v][i][j] != null && isPositionInPrimaryPeriod(counter, problem.positions.get(j), problem.months)){
								y[v][i][j] = yPV[v][i][j].get(GRB.DoubleAttr.X);
						}
					} catch (GRBException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}	
			}
		}
	}
	


	/*
	* ------------------------- Method to evaluate if position is in primary -------------------------
	*/
	public boolean isPositionInPrimaryPeriod(int counter, Position i, ArrayList<Month> months){
		if (i.endTime <= months.get(counter).end && i.endTime >= months.get(counter).start){
			return true;
		}
		else{
			return false;
		}
	}
}