import gurobi.*;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

/*
 * This class contains methods to print solutions to the screen.
 */

public class SolutionPrinter {
	public GRBModel model;
	
	
	public static void printSolution(GRBModel model, Problem problem){

		try{
		
    /*
     * ------------------ Print solution -----------------------
     */
    
    System.out.println("\nTOTAL COSTS: " + model.get(GRB.DoubleAttr.ObjVal));
    System.out.println("SOLUTION:");
    
    
    /*
     * ------------------ Preliminary printouts for solution analysis -----------------------
		 */
    
    System.out.println("-------------------------------------");
    System.out.println("Vessel sailings: ");
    System.out.println("");

    for (int v = 0; v < problem.vessels.size(); v++) {
    	
    	for (int i = 0; i < problem.positions.size(); i++){
    		if (problem.vesselCanSailInitialPosition(problem.vessels.get(v), problem.positions.get(i))){
    			if (model.getVarByName("y_o[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X) > 0.5){
    				System.out.println("Vessel " + problem.vessels.get(v).code + " sails from origin to " + problem.positions.get(i).id);
    			}
    		}
    	}
    	
    	for (int i = 0; i < problem.positions.size(); i++){
    		for (int j = 0; j < problem.positions.size(); j++){
      			if ((problem.vesselCanSailPositionToPosition(problem.vessels.get(v), problem.positions.get(i), problem.positions.get(j))) && model.getVarByName("y[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id + "]").get(GRB.DoubleAttr.X) > 0.5){
      				System.out.println("Vessel " + problem.vessels.get(v).code + " sails from position " + problem.positions.get(i).id + " to position " + problem.positions.get(j).id);
      		}
    		}
    	}
    	
    	for (int i = 0; i < problem.positions.size(); i++){
    			if ((problem.vesselCanSailPosition(problem.vessels.get(v), problem.positions.get(i))) && model.getVarByName("y_d[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X) > 0.5){
    				System.out.println("Vessel " + problem.vessels.get(v).code + " sails from position " + problem.positions.get(i).id + " to its destination.");
    		}
    	}
    }
    
    System.out.println("-------------------------------------");
    System.out.println("Voyage delays:");
    System.out.println("");

    for (int i = 0; i < problem.positions.size(); i++){
    	if (model.getVarByName("d[" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X) > 0){
    		System.out.println("The sailing of voyage " + problem.positions.get(i).id + " is " + model.getVarByName("d[" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X) + " days delayed." );
    	}
    }
    
    /*
    System.out.println("-------------------------------------");
    System.out.println("Voyage start times:");
    System.out.println("");
    
    for (int i = 0; i < problem.positions.size(); i++){
    	System.out.println("The sailing of voyage " + problem.positions.get(i).id + " starts at day " + model.getVarByName("t[" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X));
    }
    */
    
    System.out.println("-------------------------------------");
    System.out.println("Voyage loads:");
    System.out.println("");
    
    for (int i = 0; i < problem.positions.size(); i++){
    	for (int b = 0; b < problem.balanceCategories.size(); b++){
    		for (int p = 0; p < problem.deckCategories.size(); p++){
    			for (int k = 0; k < problem.deckCategories.size(); k++){
    				if (model.getVarByName("l[" + problem.positions.get(i).id + "][" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.deckCategories.get(k).code + "]") != null && model.getVarByName("l[" + problem.positions.get(i).id + "][" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.deckCategories.get(k).code + "]").get(GRB.DoubleAttr.X) > 0){
          		System.out.println("[" + problem.positions.get(i).id + "][" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.deckCategories.get(k).code + "] = " + model.getVarByName("l[" + problem.positions.get(i).id + "][" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.deckCategories.get(k).code + "]").get(GRB.DoubleAttr.X));
    				}
    			}
    		}
    	}
    }
    
    System.out.println("-------------------------------------");
    System.out.println("Voyages covered by chartering vessels:");
    System.out.println("");
    
    for (int i = 0; i < problem.positions.size(); i++){
    	if (model.getVarByName("dummy[" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X) > 0){
    		System.out.println("[" + problem.positions.get(i).id + "] = " + model.getVarByName("dummy[" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X));
    	}
    }
    
    System.out.println("-------------------------------------");
    System.out.println("Demand covered by space chartering:");
    System.out.println("");
    
    for (int b = 0; b < problem.balanceCategories.size(); b++){
    	for (int p = 0; p < problem.deckCategories.size(); p++){
    		for (int m = 0; m < problem.months.size(); m++){
    			if (model.getVarByName("z[" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.months.get(m).code + "]") != null && model.getVarByName("z[" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.months.get(m).code + "]").get(GRB.DoubleAttr.X) > 0){
        		System.out.println("[" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.months.get(m).code + "] = " + model.getVarByName("z[" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.months.get(m).code + "]").get(GRB.DoubleAttr.X));
    			}
    		}
    	}
    }
    
    /*
    System.out.println("-------------------------------------");
    System.out.println("Vessel speed profiles on voyage servings:");
    System.out.println("");
    
    
    for (int v = 0; v < problem.vessels.size(); v++) {
    	for (int i = 0; i < problem.positions.size(); i++){
    		for (int s = 0; s < problem.speedProfiles.size(); s++){
    			if (model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").get(GRB.DoubleAttr.X) > 0){
      			System.out.println("Vessel " + problem.vessels.get(v).code + " sails with speed weight " + problem.speedProfiles.get(s).code + " = "+ model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").get(GRB.DoubleAttr.X) +  " on " + problem.positions.get(i).id);
    			}
    		}
    	}
    }
    */
    System.out.println("-------------------------------------");


		}catch(GRBException e) {
      System.out.println("Error code: " + e.getErrorCode() + ". " +
          e.getMessage());
    }
	}
	
	
	

	/*
	 * ------------------------- Method to evaluate if a vessel can sail a voyage ------------------------------------------------
	 */

	public static boolean vesselCanSailPosition(Vessel v, Position i, Problem problem){
		if (i.endTime + GlobalConstants.maxDaysDelayed >= v.dateOffset){
			return true;
		}
		else{
			return false;
		}
	}
	
	
	/*
	 * ------------------------- Method to evaluate if a vessel can sail a voyage as an initial position --------------------------
	 */

	public static boolean vesselCanSailInitialPosition(Vessel v, Position i, Problem problem){
		if (i.endTime + GlobalConstants.maxDaysDelayed >= v.dateOffset 
				/*+ SailingBallastStart.getShortestSailingBallastStartDuration(v.code, i.route, problem.sailingBallastStarts)*/){
			return true;
		}
		else{
			return false;
		}
	}
	
	
	/*
	 * ------------------------- Method to evaluate if a vessel can sail from a voyage to another voyage -------------------------
	 */

	public static boolean vesselCanSailPositionToPosition(Vessel v, Position i, Position j, Problem problem){
		
		if (
				i.endTime + GlobalConstants.maxDaysDelayed >= v.dateOffset
				&&
				i != j
				&&
				i.startTime + Sailing.getShortestSailingDuration(v.code, i.route, problem.sailings) + SailingBallast.getShortestSailingBallastDuration(v.code, i.route, j.route, problem.sailingBallasts) <= j.endTime + GlobalConstants.maxDaysDelayed
				&&
				j.endTime + GlobalConstants.maxDaysDelayed >= v.dateOffset
				){
			return true;
		}
		else{
			return false;
		}
	}
}
