import java.util.ArrayList;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;


public class SailingBallast {
	public String code;
  public String routeFrom;
  public String routeTo;
  public String vessel;
  public String speedProfileCode;
  public double duration;
  public double costs;

  public SailingBallast(String aa, String bb, String cc, String dd, String ee, double ff, double gg) {
    code = aa;
    routeFrom = bb;
    routeTo = cc;
    vessel = dd;
    speedProfileCode = ee;
    duration = ff;
    costs = gg;
  }
  
  
  /*
   * ------------------------ Method to retrieve ballast sailing duration of a vessel at a given speed for a voyage -----------------------------
   */
  
  public static double getSailingDuration(String vessel, String routeFrom, String routeTo, String speedProfileCode, ArrayList<SailingBallast> sailingBallasts){
  	double duration = -1.00;
  	
  	for (int n = 0; n < sailingBallasts.size(); n++){
  		if (vessel.equals(sailingBallasts.get(n).vessel) && routeFrom.equals(sailingBallasts.get(n).routeFrom) && routeTo.equals(sailingBallasts.get(n).routeTo) && speedProfileCode.equals(sailingBallasts.get(n).speedProfileCode)){
  			duration = sailingBallasts.get(n).duration;
  		}
  	}
  	
  	if (duration < 0){
  		System.out.println("ERROR: BALLAST SAILING DURATION NOT FOUND");
  	}
  	
  	return duration;
  }
  
  
  
  /*
   * ----------------------------------- Method to retrieve shortest ballast sailing for a vessel to a voyage -----------------------------
   */
  
  public static double getShortestSailingBallastDuration(String vessel, String routeFrom, String routeTo, ArrayList<SailingBallast> sailingBallasts){
  	double duration = -1.00;
  	
  	for (int n = 0; n < sailingBallasts.size(); n++){
  		if (vessel.equals(sailingBallasts.get(n).vessel) && routeFrom.equals(sailingBallasts.get(n).routeFrom) && routeTo.equals(sailingBallasts.get(n).routeTo) && duration < 0){
  			duration = sailingBallasts.get(n).duration;
  		}
  		else if (vessel.equals(sailingBallasts.get(n).vessel) && routeFrom.equals(sailingBallasts.get(n).routeFrom) && routeTo.equals(sailingBallasts.get(n).routeTo) && sailingBallasts.get(n).duration < duration){
  			duration = sailingBallasts.get(n).duration;
  		}
  	}
  	
  	return duration;
  }
  
  
  /*
   * ------------------------- Method to retrieve cost of ballast sailing between positions ----------------------
   */
  
  public static double getCosts(String vessel, String routeFrom, String routeTo, String speedProfileCode, ArrayList<SailingBallast> sailingBallasts){
  	double costs = -1.00;
  	
  	for (int n = 0; n < sailingBallasts.size(); n++){
  		if (vessel.equals(sailingBallasts.get(n).vessel) && routeFrom.equals(sailingBallasts.get(n).routeFrom) && routeTo.equals(sailingBallasts.get(n).routeTo) && speedProfileCode.equals(sailingBallasts.get(n).speedProfileCode)){
  			costs = sailingBallasts.get(n).costs;
  		}
  	}
  	
  	/*
  	if (costs < 0){
  		System.out.println("ERROR: COST OF BALLAST SAILING NOT FOUND: [" + vessel + "][" + routeTo + "][" + routeFrom + "]");
  	}
  	*/
  	return costs;
  }
  
  
  
  
  public static double getHighestCosts(String vessel, String routeFrom, String routeTo, ArrayList<SailingBallast> sailingBallasts){
  	double costs = 9999999999999999999.9;
  	int length = sailingBallasts.size();
  	for (int n = 0; n < length; n++){
  		if (vessel == sailingBallasts.get(n).vessel && routeFrom == sailingBallasts.get(n).routeFrom && routeTo == sailingBallasts.get(n).routeTo && sailingBallasts.get(n).costs < costs){
  			costs = sailingBallasts.get(n).costs;
  		}
  	}
  	return costs;
  }

	public static ArrayList<SailingBallast> importSailingBallasts(HSSFWorkbook workbook){
  	HSSFSheet sailingBallastSheet = workbook.getSheet("SailingBallast");
  	int numberOfRowsInSailingBallastSheet = sailingBallastSheet.getLastRowNum();
		
		ArrayList<SailingBallast> sailingBallasts = new ArrayList<SailingBallast>();
				
		for (int rowNum = 1; rowNum <= numberOfRowsInSailingBallastSheet; rowNum++){
			Row row = sailingBallastSheet.getRow(rowNum);
			Cell c = row.getCell(0);
			
			if (c != null){
			Cell sailingBallastCode = row.getCell(0);
			sailingBallastCode.setCellType(Cell.CELL_TYPE_STRING);
		  Cell sailingBallastRouteCodeFrom = row.getCell(1);
		  sailingBallastRouteCodeFrom.setCellType(Cell.CELL_TYPE_STRING);
		  Cell sailingBallastRouteCodeTo = row.getCell(2);
		  sailingBallastRouteCodeTo.setCellType(Cell.CELL_TYPE_STRING);
		  Cell sailingBallastVesselCode = row.getCell(3);
		  sailingBallastVesselCode.setCellType(Cell.CELL_TYPE_STRING);
		  Cell sailingBallastSpeedProfileCode = row.getCell(4);	
		  sailingBallastSpeedProfileCode.setCellType(Cell.CELL_TYPE_STRING);
		  Cell sailingBallastDuration = row.getCell(5);
		  Cell sailingBallastCosts = row.getCell(6);

		  sailingBallasts.add(new SailingBallast(
					(String)sailingBallastCode.getStringCellValue(),
					(String)sailingBallastRouteCodeFrom.getStringCellValue(),
					(String)sailingBallastRouteCodeTo.getStringCellValue(),
					(String)sailingBallastVesselCode.getStringCellValue(),
					(String)sailingBallastSpeedProfileCode.getStringCellValue(),
					(double)sailingBallastDuration.getNumericCellValue()/100,
					(double)sailingBallastCosts.getNumericCellValue()/100));
				}
		 	}
		return sailingBallasts;
  }
  
}
