import java.util.ArrayList;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;


public class Demand {
	public String balanceCategoryCode;
	public String monthCode;
	public String deckCategoryCode;
	public double sqm;
	public double spaceCharterCost;
	public double spaceCharterSQMCap;

  public Demand(String aa, String bb, String cc, double dd, double ee, double ff) {
  	balanceCategoryCode = aa;
  	monthCode = bb;
  	deckCategoryCode = cc;
  	sqm = dd;
  	spaceCharterCost = ee;
  	spaceCharterSQMCap = ff;
  }
  
  
  /*
   * ----------------- Method to retrieve the space charter cap of a given deck category in a balance category in a month --------------------
   */
  
  public static double getSpaceCharterCap(String balanceCategoryCode, String monthCode, String deckCategoryCode, ArrayList<Demand> demands){
  	double cap = 0;
  	
  	for (int n = 0; n < demands.size(); n++){
  		if (demands.get(n).balanceCategoryCode.equals(balanceCategoryCode) && demands.get(n).deckCategoryCode.equals(deckCategoryCode) && demands.get(n).monthCode.equals(monthCode)){
  			cap = demands.get(n).spaceCharterSQMCap;
  		}
  	}
  	return cap;
  }
  
  
  
  public static double getSpaceCharterCost(String balanceCategoryCode, String monthCode, String deckCategoryCode, ArrayList<Demand> demands){
  	double cost = 0;
  	for (int d = 0; d < demands.size(); d++){
  		if (demands.get(d).balanceCategoryCode == balanceCategoryCode && demands.get(d).deckCategoryCode == deckCategoryCode && demands.get(d).monthCode == monthCode){
  			cost = demands.get(d).spaceCharterCost;
  		}
  	}
  	return cost;
  }
  
  
  
  public static boolean isDeckCategoryInBalanceCategory(String deckCategoryCode, String balanceCategoryCode, ArrayList<Demand> demands){
  	for (int d = 0; d < demands.size(); d++){
  		if (demands.get(d).balanceCategoryCode == balanceCategoryCode && demands.get(d).deckCategoryCode == deckCategoryCode){
  			return true;
  		}
  	}
  	return false;
  }
  
  
  
  /*
   * --------------------------- Method to retrieve demand in a balance category in a month of a deck category -----------------------
   */
  
  public static double getSQMDemand(String balanceCategoryCode, String monthCode, String deckCategoryCode, ArrayList<Demand> demands){
  	double demand = 0;
  	
  	for (int d = 0; d < demands.size(); d++){
  		if (demands.get(d).balanceCategoryCode.equals(balanceCategoryCode) && demands.get(d).deckCategoryCode.equals(deckCategoryCode) && demands.get(d).monthCode.equals(monthCode)){
  			demand = demands.get(d).sqm;
  		}
  	}
  	
  	if (demand < 0){
  		System.out.println("ERROR: DEMAND NOT FOUND");
  	}
  	return demand;
  }
  
  
   
	public static ArrayList<Demand> importDemands(HSSFWorkbook workbook){
  	HSSFSheet demandSheet = workbook.getSheet("Demand");
  	int numberOfDemandRows = demandSheet.getLastRowNum();
		
		ArrayList<Demand> demands = new ArrayList<Demand>();
				
		for (int rowNum = 1; rowNum <= numberOfDemandRows; rowNum++){
			Row row = demandSheet.getRow(rowNum);
			
			Cell c = row.getCell(0);
			if (c != null){
			Cell balanceCategoryCode = row.getCell(1);
			balanceCategoryCode.setCellType(Cell.CELL_TYPE_STRING);
			Cell monthCode = row.getCell(2);
			monthCode.setCellType(Cell.CELL_TYPE_STRING);
			Cell deckCategoryCode = row.getCell(3);
			deckCategoryCode.setCellType(Cell.CELL_TYPE_STRING);
			Cell sqm = row.getCell(4);
			Cell spaceCharterCost = row.getCell(5);
			Cell spaceCharterSQMCap = row.getCell(6);

			demands.add(new Demand(
					(String)balanceCategoryCode.getStringCellValue(),
					(String)monthCode.getStringCellValue(),
					(String)deckCategoryCode.getStringCellValue(),
					(double)sqm.getNumericCellValue()/100,
					(double)spaceCharterCost.getNumericCellValue()/100,
					(double)spaceCharterSQMCap.getNumericCellValue()/100));
				}
		 	}
		return demands;
  }
}