import java.util.ArrayList;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;


public class VesselSetup {
	public String vesselCode;
	public String standardRouteCode;
	public String deckCategoryCode;
	public double capacity;

  public VesselSetup(String aa, String bb, String cc, double dd) {
  	vesselCode = aa;
  	standardRouteCode = bb;
  	deckCategoryCode = cc;
  	capacity = dd;
  }
  
  
  
  /*
   * --------------------------- Method to get the maximum capacity of any vessel in a given deck category ---------------------------
   */
  
  public static double getMaxCapacity(ArrayList<VesselSetup> vesselSetups, String deckCategoryCode){
  	double capacity = 0;	
  	
  	for (int n = 0; n < vesselSetups.size(); n++){
  		if (vesselSetups.get(n).deckCategoryCode.equals(deckCategoryCode) && vesselSetups.get(n).capacity > capacity){
  			capacity = vesselSetups.get(n).capacity;
  		}
  	}	

  	return capacity;
  }
  
  
  
  /*
   * --------------------- Method to retrieve capacity of a vessel at a given route for a deck category ----------------------
   */
  
  public static double getCapacity(ArrayList<VesselSetup> vesselSetups, String vesselCode, String standardRouteCode, String deckCategoryCode){
  	double capacity = 0;	
  	
  	for (int v = 0; v < vesselSetups.size(); v++){
  		if (vesselSetups.get(v).vesselCode.equals(vesselCode) && vesselSetups.get(v).standardRouteCode.equals(standardRouteCode) && vesselSetups.get(v).deckCategoryCode.equals(deckCategoryCode)){
  			capacity = vesselSetups.get(v).capacity;
  		}
  	}	

  	
  	return capacity;
  }
   
  
  
  /*
   * ---------------------------- Method that reads the vessel setup data from Excel file --------------------------
   */
  
	public static ArrayList<VesselSetup> importVesselSetups(HSSFWorkbook workbook){
  	HSSFSheet vesselSetupSheet = workbook.getSheet("VesselSetup");
  	int numberOfVesselSetups = vesselSetupSheet.getLastRowNum();
		
		ArrayList<VesselSetup> vesselSetups = new ArrayList<VesselSetup>();
				
		for (int rowNum = 1; rowNum <= numberOfVesselSetups; rowNum++){
			Row row = vesselSetupSheet.getRow(rowNum);
			
			Cell c = row.getCell(1);
			if (c != null){
			Cell vesselCode = row.getCell(1);
			vesselCode.setCellType(Cell.CELL_TYPE_STRING);
			Cell standardRouteCode = row.getCell(2);
			standardRouteCode.setCellType(Cell.CELL_TYPE_STRING);
			Cell deckCategoryCode = row.getCell(3);
			deckCategoryCode.setCellType(Cell.CELL_TYPE_STRING);
			Cell capacity = row.getCell(4);

			vesselSetups.add(new VesselSetup(
					(String)vesselCode.getStringCellValue(),
					(String)standardRouteCode.getStringCellValue(),
					(String)deckCategoryCode.getStringCellValue(),
					(double)capacity.getNumericCellValue()/100));
				}
		 	}
		return vesselSetups;
  }
}