
import gurobi.GRB;
import gurobi.GRBException;
import gurobi.GRBVar;

import java.util.ArrayList;

import gurobi.GRB.DoubleAttr;
import gurobi.GRBEnv;
import gurobi.GRBLinExpr;
import gurobi.GRBModel;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Date;


//import org.apache.poi.hslf.model.Sheet;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFDataFormat;
import org.apache.poi.hssf.usermodel.HSSFDataFormat;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;

public class SimulationStatistics {
	private ArrayList<Double> objectiveValue; 
	private int numberOfSimulations;
	public ArrayList<Integer> numberOfDisruptedPorts;
	public ArrayList<Integer> numberOfDisruptedRoutes;
	public ArrayList<Double> longestDelayOnVoyage;
	public ArrayList<String>longestDelayOnVoyagePosition;
	public ArrayList<Double> totalDelay;
	private ArrayList<Integer> numOfRecoveriesList;
	private ArrayList<Double> totalTimeSavedByRSList;
	private String instanceName;
	private static ArrayList<Double> plannedObjectiveValue;
	private static ArrayList<Double> plannedOperatingCosts;
	private static ArrayList<Integer> plannedDummies;
	public ArrayList<Integer> changesToOriginalPlan; 
	public ArrayList<Integer> timeChangesToTheOriginalPlan;
	public ArrayList<Integer> swapsComparedToOrignalPlan;
	
	
	public SimulationStatistics(int currentIntInstance){
		this.objectiveValue = new ArrayList<Double>();
		numberOfSimulations = 0;
		numberOfDisruptedPorts = new ArrayList<Integer>();
		numberOfDisruptedRoutes = new ArrayList<Integer>();
		longestDelayOnVoyage = new ArrayList<Double>();
		longestDelayOnVoyagePosition = new ArrayList<String>();
		totalDelay = new ArrayList<Double>();
		numOfRecoveriesList = new ArrayList<Integer>();
		totalTimeSavedByRSList = new ArrayList<Double>();
		this.instanceName = TestInstances.testInstances.get(currentIntInstance);
		plannedObjectiveValue = new ArrayList<Double>();
		plannedOperatingCosts = new ArrayList<Double>();
		plannedDummies = new ArrayList<Integer>();
		changesToOriginalPlan = new ArrayList<Integer>();
		timeChangesToTheOriginalPlan = new ArrayList<Integer>();
		swapsComparedToOrignalPlan = new ArrayList<Integer>();
	}


		

	public void addSimulationResults(double objVal, int numDisPorts, int numDisRoutes, double maxDelay, String maxDelayVoyage, double totDelay, int numOfRecoveries, double totalTimeSavedByRS, int numberOfChanges, int timeChanges, int swaps){
		this.objectiveValue.add(objVal);
		numberOfSimulations = 	numberOfSimulations + 1;
		numberOfDisruptedPorts.add(numDisPorts);
		numberOfDisruptedRoutes.add(numDisRoutes);
		longestDelayOnVoyage.add(maxDelay);
		longestDelayOnVoyagePosition.add(maxDelayVoyage);
		totalDelay.add(totDelay);
		numOfRecoveriesList.add(numOfRecoveries);
		totalTimeSavedByRSList.add(totalTimeSavedByRS);
		changesToOriginalPlan.add(numberOfChanges);
		timeChangesToTheOriginalPlan.add(timeChanges);
		swapsComparedToOrignalPlan.add(swaps);
	}
	
	public static void addPlannedObjectiveValue(Mip mip){
		
		// Set the plannedOperatingCosts:
		try {
			plannedObjectiveValue.add(mip.model.get(GRB.DoubleAttr.ObjVal));
		} catch (GRBException e) {
			e.printStackTrace();
			System.exit(1);
		}
		
		// Set the plannedOperatingCosts:
		int dummies = 0;
		for (int i = 0; i < mip.problem.positions.size(); i++){
			try {
				if ((mip.model.getVarByName("dummy[" + mip.problem.positions.get(i).id +"]").get(GRB.DoubleAttr.X)) > 0.5){
				dummies++;
				}
			} catch (GRBException e) {
				e.printStackTrace();
			}
		}
		plannedDummies.add(dummies);
		
		try {
			plannedOperatingCosts.add(mip.model.get(GRB.DoubleAttr.ObjVal) - dummies*GlobalConstants.costOfVesselCharter);
		} catch (GRBException e) {
			e.printStackTrace();
		}
	}

	public void writeSimulationStatisticsToExcel(int currentIntInstance, int robustMethod, int recoveryMethod, HSSFWorkbook workbookout){
		
		int lastSimulationCell = GlobalConstants.numberOfSimulations +1;
		int rowAverage = GlobalConstants.numberOfSimulations + 2;
		int rowStandardDeviation = GlobalConstants.numberOfSimulations + 6;
		int rowNumVariance = GlobalConstants.numberOfSimulations + 7;
		int rowLowerConfidenceInterval = GlobalConstants.numberOfSimulations + 8;
		int rowUpperConfidenceInterval = GlobalConstants.numberOfSimulations + 9;
		
		try {
			
			FileInputStream fileIn = new FileInputStream(GlobalConstants.DirectoryOutput+"SimulationOutput_" + TestInstances.testInstances.get(currentIntInstance));
			HSSFSheet sheet = workbookout.createSheet(""+ GlobalConstants.robustMethods.get(robustMethod)+"_"+ GlobalConstants.recoveryMethods.get(recoveryMethod));
	
		HSSFRow headRow = sheet.createRow((int) 0);

		HSSFCell cellA1 = headRow.createCell((int) 0);
		cellA1.setCellValue("Simulation number");
		HSSFCell cellB1 = headRow.createCell((int) 1);
		cellB1.setCellValue("Planned operating costs");
		HSSFCell cellC1 = headRow.createCell((int) 2);
		cellC1.setCellValue("Planned Unserviced Voyages");
		HSSFCell cellD1 = headRow.createCell((int) 3);
		cellD1.setCellValue("Costs caused by disruptions");
		HSSFCell cellE1 = headRow.createCell((int) 4);
		cellE1.setCellValue("Total costs");
		HSSFCell cellF1 = headRow.createCell((int) 5);
		cellF1.setCellValue("# Disrupted ports");
		HSSFCell cellG1 = headRow.createCell((int) 6);
		cellG1.setCellValue("# Disrupted routes");
		HSSFCell cellH1 = headRow.createCell((int) 7);
		cellH1.setCellValue("Longest delay");
		HSSFCell cellI1 = headRow.createCell((int) 8);
		cellI1.setCellValue("Voyage with longest delay");
		HSSFCell cellJ1 = headRow.createCell((int) 9);
		cellJ1.setCellValue("Total delay");
		HSSFCell cellK1 = headRow.createCell((int) 10);
		cellK1.setCellValue("Number of RS");
		HSSFCell cellL1 = headRow.createCell((int) 11);
		cellL1.setCellValue("Time saved on recovery strategies");
		HSSFCell cellM1 = headRow.createCell((int) 12);
		cellM1.setCellValue("Changes made to the original plan");
		HSSFCell cellN1 = headRow.createCell((int) 13);
		cellN1.setCellValue("Swaps");
		HSSFCell cellO1 = headRow.createCell((int) 14);
		cellO1.setCellValue("TimeChanges");

		
		for (int simulation = 0; simulation < GlobalConstants.numberOfSimulations; simulation ++ ){
			HSSFRow rowsimulation = sheet.createRow((int) simulation + 1);
			
			HSSFCell cellAsimulation = rowsimulation.createCell((int) 0);
			cellAsimulation.setCellValue(simulation + 1);
			HSSFCell cellBsimulation = rowsimulation.createCell((int) 1);
			cellBsimulation.setCellValue(plannedOperatingCosts.get(simulation));
			HSSFCell cellCsimulation = rowsimulation.createCell((int) 2);
			cellCsimulation.setCellValue(plannedDummies.get(simulation));
			HSSFCell cellDsimulation = rowsimulation.createCell((int) 3);
			cellDsimulation.setCellValue(objectiveValue.get(simulation)-plannedObjectiveValue.get(simulation));
			HSSFCell cellEsimulation = rowsimulation.createCell((int) 4);
			cellEsimulation.setCellValue(plannedOperatingCosts.get(simulation) + objectiveValue.get(simulation) - plannedObjectiveValue.get(simulation));
			HSSFCell cellFsimulation = rowsimulation.createCell((int) 5);
			cellFsimulation.setCellValue(numberOfDisruptedPorts.get(simulation));
			HSSFCell cellGsimulation = rowsimulation.createCell((int) 6);
			cellGsimulation.setCellValue(numberOfDisruptedRoutes.get(simulation));
			HSSFCell cellHsimulation = rowsimulation.createCell((int) 7);
			cellHsimulation.setCellValue(longestDelayOnVoyage.get(simulation)); 
			HSSFCell cellIsimulation = rowsimulation.createCell((int) 8);
			cellIsimulation.setCellValue(longestDelayOnVoyagePosition.get(simulation));
			HSSFCell cellJsimulation = rowsimulation.createCell((int) 9);
			cellJsimulation.setCellValue(totalDelay.get(simulation));
			HSSFCell cellKsimulation = rowsimulation.createCell((int) 10);
			cellKsimulation.setCellValue(numOfRecoveriesList.get(simulation));
			HSSFCell cellLsimulation = rowsimulation.createCell((int) 11);
			cellLsimulation.setCellValue(totalTimeSavedByRSList.get(simulation));
			HSSFCell cellMsimulation = rowsimulation.createCell((int) 12);
			cellMsimulation.setCellValue(changesToOriginalPlan.get(simulation));
			HSSFCell cellNsimulation = rowsimulation.createCell((int) 13);
			cellNsimulation.setCellValue(timeChangesToTheOriginalPlan.get(simulation));
			HSSFCell cellOsimulation = rowsimulation.createCell((int) 14);
			cellOsimulation.setCellValue(swapsComparedToOrignalPlan.get(simulation));
		}
		
		// Average row
		
		HSSFRow rowaverage = sheet.createRow((int) rowAverage);
		
		HSSFCell cellArowaverage = rowaverage.createCell((int) 0);
		cellArowaverage.setCellValue("Average");
		HSSFCell cellBrowaverage = rowaverage.createCell((int) 1);
		cellBrowaverage.setCellFormula("AVERAGE(B2:B"+lastSimulationCell+")");
		HSSFCell cellCrowaverage = rowaverage.createCell((int) 2);
		cellCrowaverage.setCellFormula("AVERAGE(C2:C"+lastSimulationCell+")");
		HSSFCell cellDrowaverage = rowaverage.createCell((int) 3);
		cellDrowaverage.setCellFormula("AVERAGE(D2:D"+lastSimulationCell+")");
		HSSFCell cellErowaverage = rowaverage.createCell((int) 4);
		cellErowaverage.setCellFormula("AVERAGE(E2:E"+lastSimulationCell+")");
		HSSFCell cellFrowaverage = rowaverage.createCell((int) 5);
		cellFrowaverage.setCellFormula("AVERAGE(F2:F"+lastSimulationCell+")");
		HSSFCell cellGrowaverage = rowaverage.createCell((int) 6);
		cellGrowaverage.setCellFormula("AVERAGE(G2:G"+lastSimulationCell+")");
		HSSFCell cellHrowaverage = rowaverage.createCell((int) 7);
		cellHrowaverage.setCellFormula("AVERAGE(H2:H"+lastSimulationCell+")");
		HSSFCell cellIrowaverage = rowaverage.createCell((int) 8);
		cellIrowaverage.setCellValue("-");
		HSSFCell cellJrowaverage = rowaverage.createCell((int) 9);
		cellJrowaverage.setCellFormula("AVERAGE(J2:J"+lastSimulationCell+")");
		HSSFCell cellKrowaverage = rowaverage.createCell((int) 10);
		cellKrowaverage.setCellFormula("AVERAGE(K2:K"+lastSimulationCell+")");
		HSSFCell cellLrowaverage = rowaverage.createCell((int) 11);
		cellLrowaverage.setCellFormula("AVERAGE(L2:L"+lastSimulationCell+")");
		HSSFCell cellMrowaverage = rowaverage.createCell((int) 12);
		cellMrowaverage.setCellFormula("AVERAGE(M2:M"+lastSimulationCell+")");
		HSSFCell cellNrowaverage = rowaverage.createCell((int) 13);
		cellNrowaverage.setCellFormula("AVERAGE(N2:N"+lastSimulationCell+")");
		HSSFCell cellOrowaverage = rowaverage.createCell((int) 14);
		cellOrowaverage.setCellFormula("AVERAGE(O2:O"+lastSimulationCell+")");
		
		// Standard Deviation-row
		
		HSSFRow rowStdDev = sheet.createRow((int) rowStandardDeviation);
		
		HSSFCell cellArowStdDev = rowStdDev.createCell((int) 0);
		cellArowStdDev.setCellValue("STDEV");
		HSSFCell cellBrowStdDev = rowStdDev.createCell((int) 1);
		cellBrowStdDev.setCellFormula("STDEV(B2:B"+lastSimulationCell+")");
		HSSFCell cellCrowStdDev = rowStdDev.createCell((int) 2);
		cellCrowStdDev.setCellFormula("STDEV(C2:C"+lastSimulationCell+")");
		HSSFCell cellDrowStdDev = rowStdDev.createCell((int) 3);
		cellDrowStdDev.setCellFormula("STDEV(D2:D"+lastSimulationCell+")");
		HSSFCell cellErowStdDev = rowStdDev.createCell((int) 4);
		cellErowStdDev.setCellFormula("STDEV(E2:E"+lastSimulationCell+")");
		HSSFCell cellFrowStdDev = rowStdDev.createCell((int) 5);
		cellFrowStdDev.setCellFormula("STDEV(F2:F"+lastSimulationCell+")");
		HSSFCell cellGrowStdDev = rowStdDev.createCell((int) 6);
		cellGrowStdDev.setCellFormula("STDEV(G2:G"+lastSimulationCell+")");
		HSSFCell cellHrowStdDev = rowStdDev.createCell((int) 7);
		cellHrowStdDev.setCellFormula("STDEV(H2:H"+lastSimulationCell+")");
		HSSFCell cellIrowStdDev = rowStdDev.createCell((int) 8);
		cellIrowStdDev.setCellValue("-");
		HSSFCell cellJrowStdDev = rowStdDev.createCell((int) 9);
		cellJrowStdDev.setCellFormula("STDEV(J2:J"+lastSimulationCell+")");
		HSSFCell cellKrowStdDev = rowStdDev.createCell((int) 10);
		cellKrowStdDev.setCellFormula("STDEV(K2:K"+lastSimulationCell+")");
		HSSFCell cellLrowStdDev = rowStdDev.createCell((int) 11);
		cellLrowStdDev.setCellFormula("STDEV(L2:L"+lastSimulationCell+")");
		
		
		HSSFRow rowLowerConfInt = sheet.createRow((int) rowLowerConfidenceInterval);
		
		HSSFCell cellArowLowerConfInt = rowLowerConfInt.createCell((int) 0);
		cellArowLowerConfInt.setCellValue("LB Conf.Int.");
		HSSFCell cellBrowLowerConfInt = rowLowerConfInt.createCell((int) 1);
		cellBrowLowerConfInt.setCellFormula("AVERAGE(B2:B"+lastSimulationCell+")-"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(B2:B"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellCrowLowerConfInt = rowLowerConfInt.createCell((int) 2);
		cellCrowLowerConfInt.setCellFormula("AVERAGE(C2:C"+lastSimulationCell+")-"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(C2:C"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellDrowLowerConfInt = rowLowerConfInt.createCell((int) 3);
		cellDrowLowerConfInt.setCellFormula("AVERAGE(D2:D"+lastSimulationCell+")-"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(D2:D"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellErowLowerConfInt = rowLowerConfInt.createCell((int) 4);
		cellErowLowerConfInt.setCellFormula("AVERAGE(E2:E"+lastSimulationCell+")-"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(E2:E"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellFrowLowerConfInt = rowLowerConfInt.createCell((int) 5);
		cellFrowLowerConfInt.setCellFormula("AVERAGE(F2:F"+lastSimulationCell+")-"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(F2:F"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellGrowLowerConfInt = rowLowerConfInt.createCell((int) 6);
		cellGrowLowerConfInt.setCellFormula("AVERAGE(G2:G"+lastSimulationCell+")-"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(G2:G"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellHrowLowerConfInt = rowLowerConfInt.createCell((int) 7);
		cellHrowLowerConfInt.setCellFormula("AVERAGE(H2:H"+lastSimulationCell+")-"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(H2:H"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellIrowLowerConfInt = rowLowerConfInt.createCell((int) 8);
		cellIrowLowerConfInt.setCellValue("-");
		HSSFCell cellJrowLowerConfInt = rowLowerConfInt.createCell((int) 9);
		cellJrowLowerConfInt.setCellFormula("AVERAGE(J2:J"+lastSimulationCell+")-"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(J2:J"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellKrowLowerConfInt = rowLowerConfInt.createCell((int) 10);
		cellKrowLowerConfInt.setCellFormula("AVERAGE(K2:K"+lastSimulationCell+")-"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(K2:K"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellLrowLowerConfInt = rowLowerConfInt.createCell((int) 11);
		cellLrowLowerConfInt.setCellFormula("AVERAGE(L2:L"+lastSimulationCell+")-"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(L2:L"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		
		
		HSSFRow rowUpperConfInt = sheet.createRow((int) rowUpperConfidenceInterval);
		
		HSSFCell cellArowUpperConfInt = rowUpperConfInt.createCell((int) 0);
		cellArowUpperConfInt.setCellValue("UB Conf.Int.");
		HSSFCell cellBrowUpperConfInt = rowUpperConfInt.createCell((int) 1);
		cellBrowUpperConfInt.setCellFormula("AVERAGE(B2:B"+lastSimulationCell+")+"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(B2:B"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellCrowUpperConfInt = rowUpperConfInt.createCell((int) 2);
		cellCrowUpperConfInt.setCellFormula("AVERAGE(C2:C"+lastSimulationCell+")+"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(C2:C"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellDrowUpperConfInt = rowUpperConfInt.createCell((int) 3);
		cellDrowUpperConfInt.setCellFormula("AVERAGE(D2:D"+lastSimulationCell+")+"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(D2:D"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellErowUpperConfInt = rowUpperConfInt.createCell((int) 4);
		cellErowUpperConfInt.setCellFormula("AVERAGE(E2:E"+lastSimulationCell+")+"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(E2:E"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellFrowUpperConfInt = rowUpperConfInt.createCell((int) 5);
		cellFrowUpperConfInt.setCellFormula("AVERAGE(F2:F"+lastSimulationCell+")+"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(F2:F"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellGrowUpperConfInt = rowUpperConfInt.createCell((int) 6);
		cellGrowUpperConfInt.setCellFormula("AVERAGE(G2:G"+lastSimulationCell+")+"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(G2:G"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellHrowUpperConfInt = rowUpperConfInt.createCell((int) 7);
		cellHrowUpperConfInt.setCellFormula("AVERAGE(H2:H"+lastSimulationCell+")+"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(H2:H"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellIrowUpperConfInt = rowUpperConfInt.createCell((int) 8);
		cellIrowUpperConfInt.setCellValue("-");
		HSSFCell cellJrowUpperConfInt = rowUpperConfInt.createCell((int) 9);
		cellJrowUpperConfInt.setCellFormula("AVERAGE(J2:J"+lastSimulationCell+")+"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(J2:J"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellKrowUpperConfInt = rowUpperConfInt.createCell((int) 10);
		cellKrowUpperConfInt.setCellFormula("AVERAGE(K2:K"+lastSimulationCell+")+"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(K2:K"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		HSSFCell cellLrowUpperConfInt = rowUpperConfInt.createCell((int) 11);
		cellLrowUpperConfInt.setCellFormula("AVERAGE(L2:L"+lastSimulationCell+")+"+GlobalConstants.ConfidenceIntervalLevel+"*STDEV(L2:L"+lastSimulationCell+")/SQRT("+numberOfSimulations+")");
		
		HSSFRow rowVariance = sheet.createRow((int) rowNumVariance);
		
		HSSFCell cellArowVariance = rowVariance.createCell((int) 0);
		cellArowVariance.setCellValue("VAR");
		HSSFCell cellBrowVariance = rowVariance.createCell((int) 1);
		cellBrowVariance.setCellFormula("VAR(B2:B"+lastSimulationCell+")");
		HSSFCell cellCrowVariance = rowVariance.createCell((int) 2);
		cellCrowVariance.setCellFormula("VAR(C2:C"+lastSimulationCell+")");
		HSSFCell cellDrowVariance = rowVariance.createCell((int) 3);
		cellDrowVariance.setCellFormula("VAR(D2:D"+lastSimulationCell+")");
		HSSFCell cellErowVariance = rowVariance.createCell((int) 4);
		cellErowVariance.setCellFormula("VAR(E2:E"+lastSimulationCell+")");
		HSSFCell cellFrowVariance = rowVariance.createCell((int) 5);
		cellFrowVariance.setCellFormula("VAR(F2:F"+lastSimulationCell+")");
		HSSFCell cellGrowVariance = rowVariance.createCell((int) 6);
		cellGrowVariance.setCellFormula("VAR(G2:G"+lastSimulationCell+")");
		HSSFCell cellHrowVariance = rowVariance.createCell((int) 7);
		cellHrowVariance.setCellFormula("VAR(H2:H"+lastSimulationCell+")");
		HSSFCell cellIrowVariance = rowVariance.createCell((int) 8);
		cellIrowVariance.setCellValue("-");
		HSSFCell cellJrowVariance = rowVariance.createCell((int) 9);
		cellJrowVariance.setCellFormula("VAR(J2:J"+lastSimulationCell+")");
		HSSFCell cellKrowVariance = rowVariance.createCell((int) 10);
		cellKrowVariance.setCellFormula("VAR(K2:K"+lastSimulationCell+")");
		HSSFCell cellLrowVariance = rowVariance.createCell((int) 11);
		cellLrowVariance.setCellFormula("VAR(L2:L"+lastSimulationCell+")");
		
		FileOutputStream fileOut = new FileOutputStream(GlobalConstants.DirectoryOutput+"SimulationOutput_" + TestInstances.testInstances.get(currentIntInstance));
		workbookout.write(fileOut);
		fileOut.flush();
		fileOut.close();
		
		 // } finally {
		 fileIn.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} 
	}
	
public static void writeSummarySheetInExcel(int currentIntInstance, HSSFWorkbook workbookout){
		
		int lastSimulationCell = GlobalConstants.numberOfSimulations +1;
		int rowAverage = GlobalConstants.numberOfSimulations + 3;
		int rowStandardDeviation = GlobalConstants.numberOfSimulations + 6;
		int rowNumVariance = GlobalConstants.numberOfSimulations + 8;
		int rowLowerConfidenceInterval = GlobalConstants.numberOfSimulations + 9;
		int rowUpperConfidenceInterval = GlobalConstants.numberOfSimulations + 10;
		int degreesOfFreedom = GlobalConstants.numberOfSimulations*2-2;
		

			
			try {
				FileInputStream fileIn = new FileInputStream(GlobalConstants.DirectoryOutput+"SimulationOutput_" + TestInstances.testInstances.get(currentIntInstance));
		
			HSSFSheet summarysheet = workbookout.createSheet("Summary");
			workbookout.setSheetOrder("Summary", 0);
			
			HSSFRow rowHeader = summarysheet.createRow((int) 0);
			
			HSSFCell cellA1 = rowHeader.createCell((int) 0);
			cellA1.setCellValue("Method");
			HSSFCell cellB1 = rowHeader.createCell((int) 1);
			cellB1.setCellValue("Planned operating costs");
			HSSFCell cellC1 = rowHeader.createCell((int) 2);
			cellC1.setCellValue("Planned Unserviced Voyages");
			HSSFCell cellD1 = rowHeader.createCell((int) 3);
			cellD1.setCellValue("Costs caused by disruptions");
			HSSFCell cellE1 = rowHeader.createCell((int) 4);
			cellE1.setCellValue("Total costs");
			HSSFCell cellF1 = rowHeader.createCell((int) 5);
			cellF1.setCellValue("# Disrupted ports");
			HSSFCell cellG1 = rowHeader.createCell((int) 6);
			cellG1.setCellValue("# Disrupted routes");
			HSSFCell cellH1 = rowHeader.createCell((int) 7);
			cellH1.setCellValue("Longest delay");
			HSSFCell cellI1 = rowHeader.createCell((int) 8);
			cellI1.setCellValue("Voyage with longest delay");
			HSSFCell cellJ1 = rowHeader.createCell((int) 9);
			cellJ1.setCellValue("Total delay");
			HSSFCell cellK1 = rowHeader.createCell((int) 10);
			cellK1.setCellValue("Number of RS");
			HSSFCell cellL1 = rowHeader.createCell((int) 11);
			cellL1.setCellValue("Time saved on recovery strategies");
			HSSFCell cellM1 = rowHeader.createCell((int) 12);
			cellM1.setCellValue("Changes made to the original plan");
			HSSFCell cellN1 = rowHeader.createCell((int) 13);
			cellN1.setCellValue("Swaps");
			HSSFCell cellO1 = rowHeader.createCell((int) 14);
			cellO1.setCellValue("TimeChanges");
			
			
			for (int sheet = 1; sheet < workbookout.getNumberOfSheets(); sheet++){
				HSSFRow rowBasic = summarysheet.createRow((int) sheet);
				
				HSSFCell cellMethodName = rowBasic.createCell((int) 0);
				cellMethodName.setCellValue(""+workbookout.getSheetName(sheet));
				HSSFCell cellAverageOCsheet = rowBasic.createCell((int) 1);
				cellAverageOCsheet.setCellFormula(""+workbookout.getSheetName(sheet)+"!B"+rowAverage);
				HSSFCell cellPlannedUV = rowBasic.createCell((int) 2);
				cellPlannedUV.setCellFormula(""+workbookout.getSheetName(sheet)+"!C"+rowAverage);
				HSSFCell cellDisruptionCost = rowBasic.createCell((int) 3);
				cellDisruptionCost.setCellFormula(""+workbookout.getSheetName(sheet)+"!D"+rowAverage);
				HSSFCell TotalCost = rowBasic.createCell((int) 4);
				TotalCost.setCellFormula(""+workbookout.getSheetName(sheet)+"!E"+rowAverage);
				HSSFCell numDisrPorts = rowBasic.createCell((int) 5);
				numDisrPorts.setCellFormula(""+workbookout.getSheetName(sheet)+"!F"+rowAverage);
				HSSFCell numDisrVoyages = rowBasic.createCell((int) 6);
				numDisrVoyages.setCellFormula(""+workbookout.getSheetName(sheet)+"!G"+rowAverage);
				HSSFCell longestDelay = rowBasic.createCell((int) 8);
				longestDelay.setCellFormula(""+workbookout.getSheetName(sheet)+"!H"+rowAverage);
				HSSFCell routeWithLongestDelay = rowBasic.createCell((int) 9);
				routeWithLongestDelay.setCellValue("-");
				HSSFCell TotalDelay = rowBasic.createCell((int) 9);
				TotalDelay.setCellFormula(""+workbookout.getSheetName(sheet)+"!J"+rowAverage);
				HSSFCell numRS = rowBasic.createCell((int) 10);
				numRS.setCellFormula(""+workbookout.getSheetName(sheet)+"!K"+rowAverage);
				HSSFCell timeSavedOnRS = rowBasic.createCell((int) 11);
				timeSavedOnRS.setCellFormula(""+workbookout.getSheetName(sheet)+"!L"+rowAverage);
				HSSFCell changes = rowBasic.createCell((int) 12);
				changes.setCellFormula(""+workbookout.getSheetName(sheet)+"!M"+rowAverage);
				HSSFCell swaps = rowBasic.createCell((int) 13);
				swaps.setCellFormula(""+workbookout.getSheetName(sheet)+"!N"+rowAverage);
				HSSFCell timeChanges = rowBasic.createCell((int) 14);
				timeChanges.setCellFormula(""+workbookout.getSheetName(sheet)+"!O"+rowAverage);
				
			}
			
			/*
			HSSFRow rowTstats = summarysheet.createRow((int) workbookout.getNumberOfSheets()+1);
			
			HSSFCell celltvalue = rowTstats.createCell((int) 0);
			celltvalue.setCellValue("t-value");
			HSSFCell tcalc = rowTstats.createCell((int) 1);
			//tcalc.setCellFormula("T.INV("+GlobalConstants.Probability+";"+degreesOfFreedom+")");
			*/
			
			HSSFRow tHeader = summarysheet.createRow((int) workbookout.getNumberOfSheets()+2);
						
						HSSFCell cellAtHeader = tHeader.createCell((int) 0);
						cellAtHeader.setCellValue("Method");
						HSSFCell cellBtHeader = tHeader.createCell((int) 1);
						cellBtHeader.setCellValue("Compared to");
						HSSFCell cellDtHeader = tHeader.createCell((int) 3);
						cellDtHeader.setCellValue("t-value");
			
			// Printing the t-value for the robust methods compared to the basic model
			for (int robustSheet = 3; robustSheet < workbookout.getNumberOfSheets(); robustSheet++){
				HSSFRow rowStatSig = summarysheet.createRow((int) workbookout.getNumberOfSheets()+robustSheet);
				
				HSSFCell cellMethodName = rowStatSig.createCell((int) 0);
				cellMethodName.setCellValue(""+workbookout.getSheetName(robustSheet));
				
				HSSFCell cellComparedTo = rowStatSig.createCell((int) 1);
				cellComparedTo.setCellValue(""+workbookout.getSheetName(1));
				
				HSSFCell tTest = rowStatSig.createCell((int) 3);
				tTest.setCellFormula("(D"+(robustSheet+1)+"-D2)/(SQRT(("+workbookout.getSheetName(robustSheet)+"!D"+rowNumVariance+"/("+GlobalConstants.numberOfSimulations+"-1))+("+workbookout.getSheetName(1)+"!D"+rowNumVariance+"/"+GlobalConstants.numberOfSimulations+"-1)))");
			robustSheet++;
			}
			
			// Printing the t-value for the models with integrated recovery strategies compared to the same robust (or basic) methods without recovery
			for (int recoverySheet = 4; recoverySheet < workbookout.getNumberOfSheets(); recoverySheet++){
				HSSFRow rowStatSigRecovery = summarysheet.createRow((int) workbookout.getNumberOfSheets()+recoverySheet);
				
				HSSFCell cellMethodName = rowStatSigRecovery.createCell((int) 0);
				cellMethodName.setCellValue(""+workbookout.getSheetName(recoverySheet));
				
				HSSFCell cellComparedTo = rowStatSigRecovery.createCell((int) 1);
				cellComparedTo.setCellValue(""+workbookout.getSheetName(recoverySheet-1));
				
				HSSFCell tTest = rowStatSigRecovery.createCell((int) 3);
				tTest.setCellFormula("(D"+(recoverySheet+1)+"-D2)/(SQRT(("+workbookout.getSheetName(recoverySheet)+"!D"+rowNumVariance+"/("+GlobalConstants.numberOfSimulations+"-1))+("+workbookout.getSheetName(recoverySheet-1)+"!D"+rowNumVariance+"/"+GlobalConstants.numberOfSimulations+"-1)))");
			recoverySheet++;
			}
			
			
			FileOutputStream fileOut = new FileOutputStream(GlobalConstants.DirectoryOutput+"SimulationOutput_" + TestInstances.testInstances.get(currentIntInstance));
			//HSSFSheet sheet1 = workbookout.createSheet("RobustMethod_"+ GlobalConstants.robustMethods.get(robustMethod));
			workbookout.write(fileOut);
			fileOut.flush();
			fileOut.close();
			
			 fileIn.close();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} 
	}
	
}