
import gurobi.GRB;
import gurobi.GRBException;
import gurobi.GRBVar;

import java.util.ArrayList;

import gurobi.GRB.DoubleAttr;
import gurobi.GRBEnv;
import gurobi.GRBLinExpr;
import gurobi.GRBModel;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Date;


public class SimulationObjectState {
	public double[][]y_i;
	public double[][][]y;
	public double[][]y_o;
	public double[][]y_d;
	public double[][][]x_Bo;
	public double[][][]x;
	public double[][][][]x_B;
	public double[]dummy;
	public double[][][][]l;
	public double[][][]z;
	public double[]t;
	public double[]t_o;
	public boolean[]fixedStartTimeVoyage;
	public boolean[]fixedStartTimeVessel;
	public boolean[]fixedOriginSpeed;
	public boolean[]fixedVoyageSpeed;
	public boolean[]fixedBallastSpeed;
	Problem problem; 
	Mip mip;
	
	public SimulationObjectState(Problem problem, Mip mip){
		this.y_i 	= new double[problem.vessels.size()][problem.positions.size()];
		this.y 		= new double[problem.vessels.size()][problem.positions.size()][problem.positions.size()];
		this.y_o 	= new double[problem.vessels.size()][problem.positions.size()];
		this.y_d 	= new double[problem.vessels.size()][problem.positions.size()];
		this.x_Bo 	= new double[problem.vessels.size()][problem.positions.size()][problem.speedProfiles.size()];
		this.x		= new double[problem.vessels.size()][problem.positions.size()][problem.speedProfiles.size()];
		this.x_B	= new double[problem.vessels.size()][problem.positions.size()][problem.positions.size()][problem.speedProfiles.size()];
		this.dummy	= new double[problem.positions.size()];
		this.l		= new double[problem.positions.size()][problem.balanceCategories.size()][problem.deckCategories.size()][problem.deckCategories.size()];
		this.z		= new double[problem.balanceCategories.size()][problem.deckCategories.size()][problem.months.size()];
		this.t 		= new double[problem.positions.size()];
		this.t_o	= new double[problem.vessels.size()];
		this.problem = problem;
		this.mip = mip;
		this.fixedStartTimeVoyage = new boolean[problem.positions.size()];
		for (int i = 0; i < problem.positions.size(); i++){
			fixedStartTimeVoyage[i] = false;
		}
		this.fixedStartTimeVessel = new boolean[problem.vessels.size()];
		for(int v = 0; v < problem.vessels.size(); v ++ ){
			fixedStartTimeVessel[v] = false;
		}
		this.fixedOriginSpeed = new boolean[problem.vessels.size()];
		for(int v = 0; v < problem.vessels.size(); v ++ ){
			fixedOriginSpeed[v] = false;
		}
		this.fixedVoyageSpeed = new boolean[problem.positions.size()];
		for (int i = 0; i < problem.positions.size(); i++){
			fixedVoyageSpeed[i] = false;
		}
		this.fixedBallastSpeed = new boolean[problem.positions.size()];
		for (int i = 0; i < problem.positions.size(); i++){
			fixedBallastSpeed[i] = false;
		}
	}

	
	// This method gets stores the values from the MIP model in SimulationObjectState-object. (Except the delay-variable)
	public void getVariablesFromModel(){
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (problem.vesselCanSailPosition(problem.vessels.get(v), problem.positions.get(i))){
					try {
						y_i[v][i]= this.mip.model.getVarByName("y_i[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X);
					} catch (GRBException e4) {
						e4.printStackTrace();
						System.exit(1);
					}
				}
			}
		}
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (problem.vesselCanSailInitialPosition(problem.vessels.get(v),problem.positions.get(i))){
					try {
						y_o[v][i] = mip.model.getVarByName("y_o[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X);
					} catch (GRBException e3) {
					e3.printStackTrace();
					System.exit(1);
					}
				}
			}
		}
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (problem.vesselCanSailPosition(problem.vessels.get(v), problem.positions.get(i))){
					try {
						y_d[v][i] = mip.model.getVarByName("y_d[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X);
					} catch (GRBException e2) {
						e2.printStackTrace();
						System.exit(1);
					}
				}
			}
		}
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int s = 0; s < problem.speedProfiles.size(); s++){
					if (problem.vesselCanSailInitialPosition(problem.vessels.get(v), problem.positions.get(i))){
						try {
							x_Bo[v][i][s] 	= mip.model.getVarByName("x_Bo[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id  + "][" + problem.speedProfiles.get(s).code + "]").get(GRB.DoubleAttr.X);
						} catch (GRBException e1) {
							e1.printStackTrace();
							System.exit(1);
						}
					}
				}
			}
		}
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int s = 0; s < problem.speedProfiles.size(); s++){
					if (problem.vesselCanSailPosition(problem.vessels.get(v),problem.positions.get(i))){
						try {
							x[v][i][s] 		= mip.model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").get(GRB.DoubleAttr.X);
						} catch (GRBException e) {
						e.printStackTrace();
						System.exit(1);
						}
					}
				}
			}
		}
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int j = 0; j < problem.positions.size(); j++){
					if (problem.vesselCanSailPositionToPosition(problem.vessels.get(v), problem.positions.get(i), problem.positions.get(j))){
						try {
							y[v][i][j] = mip.model.getVarByName("y[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id + "]").get(GRB.DoubleAttr.X);
						} catch (GRBException e) {
							e.printStackTrace();
							System.out.println(e.getErrorCode());
							System.out.println(e.getMessage());
							System.exit(1);
						}
					}
				}
			}
		}
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int j = 0; j < problem.positions.size(); j++){			
					for (int s = 0; s < problem.speedProfiles.size(); s++){
						if (problem.vesselCanSailPositionToPosition(problem.vessels.get(v), problem.positions.get(i), problem.positions.get(j))){
							try {
								x_B[v][i][j][s] = mip.model.getVarByName("x_B[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id +  "]["  + problem.speedProfiles.get(s).code + "]").get(GRB.DoubleAttr.X);
							} catch (GRBException e) {
								e.printStackTrace();
								e.getErrorCode();
								e.getMessage();
								System.exit(1);
							}
						}
					}
				}
			}		
		}
	
		for (int i = 0; i < problem.positions.size(); i++){
			try {
				dummy[i] 	=  mip.model.getVarByName("dummy[" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X);
			} catch (GRBException e) {
				e.printStackTrace();
				System.exit(1);
			}
		}
		for (int i = 0; i < problem.positions.size(); i++){
			for (int b = 0; b < problem.balanceCategories.size(); b++){
	      		for (int p = 0; p < problem.deckCategories.size(); p++){
	      			for (int k = 0; k < problem.deckCategories.size(); k++){
	      				if (BalanceCategoryStandardRoute.isBalanceCategoryInRoute(problem.balanceCategories.get(b).code, problem.positions.get(i).route, problem.balanceCategoryStandardRoutes)){
	      					if (DeckCategoryToDeckCategory.ifPossibleToPut(problem.deckCategories.get(p).code, problem.deckCategories.get(k).code, problem.deckCategoryToDeckCategories)){
		      					try {
									l[i][b][p][k] = mip.model.getVarByName("l[" + problem.positions.get(i).id + "][" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.deckCategories.get(k).code + "]").get(GRB.DoubleAttr.X);
								} catch (GRBException e) {
									e.printStackTrace();
									e.getErrorCode();
									e.getMessage();
									System.exit(1);
								}
	      					}
	      				}
	      			}
	      		}	
			}
		}	
		for (int b = 0; b < problem.balanceCategories.size(); b++) {
	      	for (int p = 0; p < problem.deckCategories.size(); p++){	
	      		for (int m = 0; m < problem.months.size(); m++){
	      			if (Demand.isDeckCategoryInBalanceCategory(problem.deckCategories.get(p).code, problem.balanceCategories.get(b).code, problem.demands)){
	      				try {
							z[b][p][m] = mip.model.getVarByName("z[" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.months.get(m).code + "]").get(GRB.DoubleAttr.X);
						} catch (GRBException e) {
							e.printStackTrace();
							System.exit(1);
						}
	      			}	
	      		}
	   		}
   		}
		for (int i = 0; i < problem.positions.size(); i++){
			try {
				t[i] = mip.model.getVarByName("t[" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X);
			} catch (GRBException e) {
				e.printStackTrace();
				System.exit(1);
			}
		}
		
		
		for(int v = 0; v < problem.vessels.size(); v++ ){
			try {
				t_o[v] = mip.model.getVarByName("t_o[" + problem.vessels.get(v).code + "]").get(GRB.DoubleAttr.X);
			} catch (GRBException e) {
				e.printStackTrace();
				System.exit(1);
			}
		}
	}
	
	
	
	/*
	 * This method restricts the variables in MIP model to only have the values of the corresponding variables stored in SimulationObjectState-object.
	 */
	public void updateState(){
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (problem.vesselCanSailPosition(problem.vessels.get(v), problem.positions.get(i))){
					try {
						mip.model.getVarByName("y_i[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB,this.y_i[v][i]);
						mip.model.getVarByName("y_i[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB,this.y_i[v][i]);
					} catch (GRBException e4) {

						e4.printStackTrace();
					}
				}
			}
		}
		
		
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (problem.vesselCanSailInitialPosition(problem.vessels.get(v),problem.positions.get(i))){	
					try {
						mip.model.getVarByName("y_o[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB,this.y_o[v][i]);
						mip.model.getVarByName("y_o[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB,this.y_o[v][i]);
					} catch (GRBException e3) {
						e3.printStackTrace();
					}
				}
			}
		}
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (problem.vesselCanSailPosition(problem.vessels.get(v), problem.positions.get(i))){
					try {
						mip.model.getVarByName("y_d[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB,this.y_d[v][i]);
						mip.model.getVarByName("y_d[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB,this.y_d[v][i]);
					} catch (GRBException e2) {
						e2.printStackTrace();
					}
				}
			}
		}
		
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int j = 0; j < problem.positions.size(); j++){
					if (problem.vesselCanSailPositionToPosition(problem.vessels.get(v), problem.positions.get(i), problem.positions.get(j))){
						try {
							mip.model.getVarByName("y[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id + "]").set(GRB.DoubleAttr.LB,this.y[v][i][j]);
							mip.model.getVarByName("y[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id + "]").set(GRB.DoubleAttr.UB,this.y[v][i][j]);
						} catch (GRBException e) {
							e.printStackTrace();
						}
					}
				}
			}
		}
		
		if(GlobalConstants.openSpeedVariables == false){
			for (int v = 0; v < problem.vessels.size(); v++ ){
				for (int i = 0; i < problem.positions.size(); i++){
					for (int s = 0; s < problem.speedProfiles.size(); s++){
						if (problem.vesselCanSailPosition(problem.vessels.get(v),problem.positions.get(i))){
							try {
							mip.model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").set(GRB.DoubleAttr.LB,this.x[v][i][s]);
							mip.model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").set(GRB.DoubleAttr.UB,this.x[v][i][s]);
							} catch (GRBException e) {
								e.printStackTrace();
							}
						}
					}
				}
			}
		}
		
		if (GlobalConstants.openSpeedVariables == false){
			for (int v = 0; v < problem.vessels.size(); v++ ){
				for (int i = 0; i < problem.positions.size(); i++){
					for (int s = 0; s < problem.speedProfiles.size(); s++){
						if (problem.vesselCanSailInitialPosition(problem.vessels.get(v), problem.positions.get(i))){
							try {
								mip.model.getVarByName("x_Bo[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id  + "][" + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.LB,this.x_Bo[v][i][s]);
								mip.model.getVarByName("x_Bo[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id  + "][" + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.UB,this.x_Bo[v][i][s]);
							} catch (GRBException e1) {
								e1.printStackTrace();
							}
						}
					}
				}
			}
		}
		
		if (GlobalConstants.openSpeedVariables == false){
			for (int v = 0; v < problem.vessels.size(); v++ ){
				for (int i = 0; i < problem.positions.size(); i++){
					for (int j = 0; j < problem.positions.size(); j++){
						for (int s = 0; s < problem.speedProfiles.size(); s++){
							if (problem.vesselCanSailPositionToPosition(problem.vessels.get(v), problem.positions.get(i), problem.positions.get(j))){
								try {
									mip.model.getVarByName("x_B[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id +  "]["  + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.LB,this.x_B[v][i][j][s]);
									mip.model.getVarByName("x_B[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id +  "]["  + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.UB,this.x_B[v][i][j][s]);
								} catch (GRBException e) {
									e.printStackTrace();
								}
							}
						}
					}
				}		
			}
		}
		
	for (int i = 0; i < problem.positions.size(); i++){
		try {
			mip.model.getVarByName("dummy[" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB,this.dummy[i]);
			mip.model.getVarByName("dummy[" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB,this.dummy[i]);
		} catch (GRBException e) {
			e.printStackTrace();
		}
	}
	
	
	for (int i = 0; i < problem.positions.size(); i++){
		for (int b = 0; b < problem.balanceCategories.size(); b++){
      		for (int p = 0; p < problem.deckCategories.size(); p++){
      			for (int k = 0; k < problem.deckCategories.size(); k++){
      				if (BalanceCategoryStandardRoute.isBalanceCategoryInRoute(problem.balanceCategories.get(b).code, problem.positions.get(i).route, problem.balanceCategoryStandardRoutes)){
      					if (DeckCategoryToDeckCategory.ifPossibleToPut(problem.deckCategories.get(p).code, problem.deckCategories.get(k).code, problem.deckCategoryToDeckCategories)){
	      					try {
								mip.model.getVarByName("l[" + problem.positions.get(i).id + "][" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.deckCategories.get(k).code + "]").set(GRB.DoubleAttr.LB, this.l[i][b][p][k]);
								mip.model.getVarByName("l[" + problem.positions.get(i).id + "][" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.deckCategories.get(k).code + "]").set(GRB.DoubleAttr.UB, this.l[i][b][p][k]);
	      					} catch (GRBException e) {
								e.printStackTrace();
							}
      					}
      				}
      			}
      		}
		}
	}
	
	for (int b = 0; b < problem.balanceCategories.size(); b++) {
      	for (int p = 0; p < problem.deckCategories.size(); p++){ 			
  			for (int m = 0; m < problem.months.size(); m++){
  				if (Demand.isDeckCategoryInBalanceCategory(problem.deckCategories.get(p).code, problem.balanceCategories.get(b).code, problem.demands)){
      				try {
						mip.model.getVarByName("z[" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.months.get(m).code + "]").set(GRB.DoubleAttr.LB, this.z[b][p][m]);
						mip.model.getVarByName("z[" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.months.get(m).code + "]").set(GRB.DoubleAttr.UB, this.z[b][p][m]);
      				} catch (GRBException e) {
						e.printStackTrace();
					}
  				}
      		}
      	}
	}
	
	try {
		this.mip.model.update();
	} catch (GRBException e) {
		e.printStackTrace();
	}
	try {
		mip.model.optimize();
	} catch (GRBException e) {
		e.printStackTrace();
	}
}
	
	
	
	public void updateInitialState(){
		
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (problem.vesselCanSailPosition(problem.vessels.get(v), problem.positions.get(i))){
					try {
						mip.model.getVarByName("y_i[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB,this.y_i[v][i]);
						mip.model.getVarByName("y_i[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB,this.y_i[v][i]);
					} catch (GRBException e4) {

						e4.printStackTrace();
					}
				}
			}
		}
		
		
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (problem.vesselCanSailInitialPosition(problem.vessels.get(v),problem.positions.get(i))){	
					try {
						mip.model.getVarByName("y_o[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB,this.y_o[v][i]);
						mip.model.getVarByName("y_o[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB,this.y_o[v][i]);
					} catch (GRBException e3) {
						e3.printStackTrace();
					}
				}
			}
		}
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (problem.vesselCanSailPosition(problem.vessels.get(v), problem.positions.get(i))){
					try {
						mip.model.getVarByName("y_d[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB,this.y_d[v][i]);
						mip.model.getVarByName("y_d[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB,this.y_d[v][i]);
					} catch (GRBException e2) {
						e2.printStackTrace();
					}
				}
			}
		}
		
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int j = 0; j < problem.positions.size(); j++){
					if (problem.vesselCanSailPositionToPosition(problem.vessels.get(v), problem.positions.get(i), problem.positions.get(j))){
						try {
							mip.model.getVarByName("y[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id + "]").set(GRB.DoubleAttr.LB,this.y[v][i][j]);
							mip.model.getVarByName("y[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id + "]").set(GRB.DoubleAttr.UB,this.y[v][i][j]);
						} catch (GRBException e) {
							e.printStackTrace();
						}
					}
				}
			}
		}
		
		if(GlobalConstants.openSpeedVariables == false){
			for (int v = 0; v < problem.vessels.size(); v++ ){
				for (int i = 0; i < problem.positions.size(); i++){
					for (int s = 0; s < problem.speedProfiles.size(); s++){
						if (problem.vesselCanSailPosition(problem.vessels.get(v),problem.positions.get(i))){
							try {
							mip.model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").set(GRB.DoubleAttr.LB,this.x[v][i][s]);
							mip.model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").set(GRB.DoubleAttr.UB,this.x[v][i][s]);
							} catch (GRBException e) {
								e.printStackTrace();
							}
						}
					}
				}
			}
		}
		
		if (GlobalConstants.openSpeedVariables == false){
			for (int v = 0; v < problem.vessels.size(); v++ ){
				for (int i = 0; i < problem.positions.size(); i++){
					for (int s = 0; s < problem.speedProfiles.size(); s++){
						if (problem.vesselCanSailInitialPosition(problem.vessels.get(v), problem.positions.get(i))){
							try {
								mip.model.getVarByName("x_Bo[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id  + "][" + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.LB,this.x_Bo[v][i][s]);
								mip.model.getVarByName("x_Bo[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id  + "][" + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.UB,this.x_Bo[v][i][s]);
							} catch (GRBException e1) {
								e1.printStackTrace();
							}
						}
					}
				}
			}
		}
		
		if (GlobalConstants.openSpeedVariables == false){
			for (int v = 0; v < problem.vessels.size(); v++ ){
				for (int i = 0; i < problem.positions.size(); i++){
					for (int j = 0; j < problem.positions.size(); j++){
						for (int s = 0; s < problem.speedProfiles.size(); s++){
							if (problem.vesselCanSailPositionToPosition(problem.vessels.get(v), problem.positions.get(i), problem.positions.get(j))){
								try {
									mip.model.getVarByName("x_B[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id +  "]["  + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.LB,this.x_B[v][i][j][s]);
									mip.model.getVarByName("x_B[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id +  "]["  + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.UB,this.x_B[v][i][j][s]);
								} catch (GRBException e) {
									e.printStackTrace();
								}
							}
						}
					}
				}		
			}
		}
		
	for (int i = 0; i < problem.positions.size(); i++){
		try {
			mip.model.getVarByName("dummy[" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB,this.dummy[i]);
			mip.model.getVarByName("dummy[" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB,this.dummy[i]);
		} catch (GRBException e) {
			e.printStackTrace();
		}
	}
	
	
	for (int i = 0; i < problem.positions.size(); i++){
		for (int b = 0; b < problem.balanceCategories.size(); b++){
      		for (int p = 0; p < problem.deckCategories.size(); p++){
      			for (int k = 0; k < problem.deckCategories.size(); k++){
      				if (BalanceCategoryStandardRoute.isBalanceCategoryInRoute(problem.balanceCategories.get(b).code, problem.positions.get(i).route, problem.balanceCategoryStandardRoutes)){
      					if (DeckCategoryToDeckCategory.ifPossibleToPut(problem.deckCategories.get(p).code, problem.deckCategories.get(k).code, problem.deckCategoryToDeckCategories)){
	      					try {
								mip.model.getVarByName("l[" + problem.positions.get(i).id + "][" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.deckCategories.get(k).code + "]").set(GRB.DoubleAttr.LB, this.l[i][b][p][k]);
								mip.model.getVarByName("l[" + problem.positions.get(i).id + "][" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.deckCategories.get(k).code + "]").set(GRB.DoubleAttr.UB, this.l[i][b][p][k]);
	      					} catch (GRBException e) {
								e.printStackTrace();
							}
      					}
      				}
      			}
      		}
		}
	}
	
	for (int b = 0; b < problem.balanceCategories.size(); b++) {
      	for (int p = 0; p < problem.deckCategories.size(); p++){ 			
  			for (int m = 0; m < problem.months.size(); m++){
  				if (Demand.isDeckCategoryInBalanceCategory(problem.deckCategories.get(p).code, problem.balanceCategories.get(b).code, problem.demands)){
      				try {
						mip.model.getVarByName("z[" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.months.get(m).code + "]").set(GRB.DoubleAttr.LB, this.z[b][p][m]);
						mip.model.getVarByName("z[" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.months.get(m).code + "]").set(GRB.DoubleAttr.UB, this.z[b][p][m]);
      				} catch (GRBException e) {
						e.printStackTrace();
					}
  				}
      		}
      	}
	}
	
	try {
		this.mip.model.update();
	} catch (GRBException e) {
		e.printStackTrace();
	}
	
		
		
		
	}
	
	
	
	
	
	
	
	
	/*
	 * Method that updates the upper bound for the time variables and delay according to the new accepted delay when simulating disruptive events
	 */
	public void updateUpperBoundForTimeAndDelayVariables(){	
	for (int i = 0; i < problem.positions.size(); i++  ){
			try {
				mip.model.getVarByName("t[" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB, Position.getLatestVoyageStart(problem.positions) + GlobalConstants.maxDaysDelayedSimulation);
			} catch (GRBException e) {
				e.printStackTrace();
			}
		}
		for (int v = 0; v < problem.vessels.size(); v++) {
			try {
				mip.model.getVarByName("t_o[" + problem.vessels.get(v).code + "]").set(GRB.DoubleAttr.UB, Position.getLatestVoyageStart(problem.positions) + GlobalConstants.maxDaysDelayedSimulation);
			} catch (GRBException e) {
				e.printStackTrace();
			}
		}
		
		
		for (int i = 0; i < problem.positions.size(); i++  ){
			try {
				mip.model.getVarByName("d[" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB, GlobalConstants.maxDaysDelayedSimulation);
			} catch (GRBException e) {
				e.printStackTrace();
			}
		}
	
	try {
		this.mip.model.update();
	} catch (GRBException e) {
		e.printStackTrace();
	}
	try {
		mip.model.optimize();
	} catch (GRBException e) {
		e.printStackTrace();
	}
}

	
// ----------------------- Method to allow changes to be made to the variables after day N ----------------------------------

	public void openVariablesAfterDayN(int n){
		
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (problem.vesselCanSailPosition(problem.vessels.get(v), problem.positions.get(i))){
					try {
						mip.model.getVarByName("y_i[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.CharAttr.VType,GRB.BINARY);
					} catch (GRBException e) {
						e.printStackTrace();
					}
					try {
						mip.model.getVarByName("y_i[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB,0);
					} catch (GRBException e4) {
						e4.printStackTrace();
					}
					try {
						mip.model.getVarByName("y_i[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB,1);
					} catch (GRBException e4) {

						e4.printStackTrace();
					}
				}
			}
		}
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (problem.vesselCanSailInitialPosition(problem.vessels.get(v),problem.positions.get(i))){
					try {
						mip.model.getVarByName("y_o[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.CharAttr.VType,GRB.BINARY);
						mip.model.getVarByName("y_o[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB,0);
						mip.model.getVarByName("y_o[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB,1);
					} catch (GRBException e3) {
						e3.printStackTrace();
					}
				}
			}
		}
		
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (problem.vesselCanSailPosition(problem.vessels.get(v), problem.positions.get(i))){
				try {
						mip.model.getVarByName("y_d[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.CharAttr.VType,GRB.BINARY);
						mip.model.getVarByName("y_d[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB, 0);
						mip.model.getVarByName("y_d[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB,1);
					} catch (GRBException e2) {
						e2.printStackTrace();
					}
				}
			}
		}
		
		
		
		
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int j = 0; j < problem.positions.size(); j++){
					if (problem.vesselCanSailPositionToPosition(problem.vessels.get(v), problem.positions.get(i), problem.positions.get(j))){
						try {
							mip.model.getVarByName("y[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id + "]").set(GRB.CharAttr.VType,GRB.BINARY);
							mip.model.getVarByName("y[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id + "]").set(GRB.DoubleAttr.LB,0);
							mip.model.getVarByName("y[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id + "]").set(GRB.DoubleAttr.UB,1);
						} catch (GRBException e) {
							e.printStackTrace();
						}
					
					}
				}
			}
		}
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int s = 0; s < problem.speedProfiles.size(); s++){
					if (problem.vesselCanSailInitialPosition(problem.vessels.get(v), problem.positions.get(i))){
						 if (t[i]> n){
							try {
								mip.model.getVarByName("x_Bo[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id  + "][" + problem.speedProfiles.get(s).code + "]").set(GRB.CharAttr.VType,GRB.CONTINUOUS);
								mip.model.getVarByName("x_Bo[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id  + "][" + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.LB,0);
								mip.model.getVarByName("x_Bo[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id  + "][" + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.UB,1);
							} catch (GRBException e1) {
								e1.printStackTrace();
							}
						}
					}
				}
			}
		}
		

		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int s = 0; s < problem.speedProfiles.size(); s++){
					if (problem.vesselCanSailPosition(problem.vessels.get(v),problem.positions.get(i))){
						if (t[i]> n){
							try {
							mip.model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").set(GRB.CharAttr.VType,GRB.CONTINUOUS);
							mip.model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").set(GRB.DoubleAttr.LB,0);
							mip.model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").set(GRB.DoubleAttr.UB,1);
							} catch (GRBException e) {
								e.printStackTrace();
							}
						}
					}
				}
			}
		}
		
		if (GlobalConstants.openSpeedVariables== false){
			for (int v = 0; v < problem.vessels.size(); v++ ){
				for (int i = 0; i < problem.positions.size(); i++){
					for (int j = 0; j < problem.positions.size(); j++){
						for (int s = 0; s < problem.speedProfiles.size(); s++){
							if (problem.vesselCanSailPositionToPosition(problem.vessels.get(v), problem.positions.get(i), problem.positions.get(j))){
								if ((t[j]> n)){
									try {
										mip.model.getVarByName("x_B[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id +  "]["  + problem.speedProfiles.get(s).code + "]").set(GRB.CharAttr.VType,GRB.CONTINUOUS);
										mip.model.getVarByName("x_B[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id +  "]["  + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.LB,0);
										mip.model.getVarByName("x_B[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id +  "]["  + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.UB,1);
									} catch (GRBException e) {
										e.printStackTrace();
									}
								}
							}
						}
					}
				}		
			}
		}
	
	
		for (int i = 0; i < problem.positions.size(); i++){
			for (int b = 0; b < problem.balanceCategories.size(); b++){
	      		for (int p = 0; p < problem.deckCategories.size(); p++){
	      			for (int k = 0; k < problem.deckCategories.size(); k++){
	      				if (BalanceCategoryStandardRoute.isBalanceCategoryInRoute(problem.balanceCategories.get(b).code, problem.positions.get(i).route, problem.balanceCategoryStandardRoutes)){
	      					if (DeckCategoryToDeckCategory.ifPossibleToPut(problem.deckCategories.get(p).code, problem.deckCategories.get(k).code, problem.deckCategoryToDeckCategories)){
	      						if (t[i]> n){
		      						try {
			      						mip.model.getVarByName("l[" + problem.positions.get(i).id + "][" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.deckCategories.get(k).code + "]").set(GRB.CharAttr.VType,GRB.CONTINUOUS);
										mip.model.getVarByName("l[" + problem.positions.get(i).id + "][" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.deckCategories.get(k).code + "]").set(GRB.DoubleAttr.LB, 0);
										mip.model.getVarByName("l[" + problem.positions.get(i).id + "][" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.deckCategories.get(k).code + "]").set(GRB.DoubleAttr.UB, VesselSetup.getMaxCapacity(problem.vesselSetups, problem.deckCategories.get(k).code));
			      					} catch (GRBException e) {
										e.printStackTrace();
									}
	      						}
	      					}
	      				}
	      			}
	      		}
			}
		}
		for (int b = 0; b < problem.balanceCategories.size(); b++) {
	      	for (int p = 0; p < problem.deckCategories.size(); p++){ 			
	  			for (int m = 0; m < problem.months.size(); m++){
	  				if (Demand.isDeckCategoryInBalanceCategory(problem.deckCategories.get(p).code, problem.balanceCategories.get(b).code, problem.demands)){
	  					try {
	      					mip.model.getVarByName("z[" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.months.get(m).code + "]").set(GRB.CharAttr.VType, GRB.CONTINUOUS);
							mip.model.getVarByName("z[" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.months.get(m).code + "]").set(GRB.DoubleAttr.LB, 0);
							mip.model.getVarByName("z[" + problem.balanceCategories.get(b).code + "][" + problem.deckCategories.get(p).code + "][" + problem.months.get(m).code + "]").set(GRB.DoubleAttr.UB,  Demand.getSpaceCharterCap(problem.balanceCategories.get(b).code, problem.months.get(m).code, problem.deckCategories.get(p).code, problem.demands));
	      				} catch (GRBException e) {
							e.printStackTrace();
						}
	  				}
	      		}
	      	}
		}		
	
	
	for (int i = 0; i < problem.positions.size(); i++){
		
		try {
			mip.model.getVarByName("dummy[" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB,0);
			mip.model.getVarByName("dummy[" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB,1);
			mip.model.getVarByName("dummy[" + problem.positions.get(i).id + "]").set(GRB.CharAttr.VType, GRB.BINARY);
		} catch (GRBException e) {
			e.printStackTrace();
		}
		
	}
}
	//  ---------------------- fixStartTimeForVoyages ---------------------------------
	public void fixStartTimeForVoyages(int n){
		for (int i = 0; i < problem.positions.size(); i++){
			if ((t[i] <= n) && (fixedStartTimeVoyage[i] == false)){
				try {
					mip.model.getVarByName("t[" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB, t[i]);
					mip.model.getVarByName("t[" + problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB, t[i]);
					} catch (GRBException e) {
					e.printStackTrace();
					System.out.println("HEr er det noe galt");
					System.exit(1);
				}
				fixedStartTimeVoyage[i] = true;
			}
		}	
	}
	
	// ---------------- Fix start time variables before day N -------------------------
	public void fixStartTimeForVessels(int n){
		for(int v = 0; v < problem.vessels.size(); v++ ){
			if ((this.t_o[v] <= n) && (fixedStartTimeVessel[v] == false)){
				try {
					mip.model.getVarByName("t_o[" + problem.vessels.get(v).code + "]").set(GRB.DoubleAttr.LB, this.t_o[v]);
					mip.model.getVarByName("t_o[" + problem.vessels.get(v).code + "]").set(GRB.DoubleAttr.UB, this.t_o[v]);
				} catch (GRBException e) {
					e.printStackTrace();
					System.exit(1);
				}
				fixedStartTimeVessel[v] = true;
			}
		}
	}
	

	// ---------------- Fix Speed Variables for first ballast sailing before day N -------------------------
	public void fixBallastSpeedFromOrigin(int n) {
		for (int v = 0 ; v < problem.vessels.size(); v ++){
			if ((this.t_o[v] <= n) && (fixedOriginSpeed[v] == false)){
				for (int i = 0; i < problem.positions.size(); i++){
					for (int s = 0; s < problem.speedProfiles.size(); s++){
						if (problem.vesselCanSailInitialPosition(problem.vessels.get(v), problem.positions.get(i))){
							try {
								mip.model.getVarByName("x_Bo[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id  + "][" + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.LB,this.x_Bo[v][i][s]);
								mip.model.getVarByName("x_Bo[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id  + "][" + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.UB,this.x_Bo[v][i][s]);
							} catch (GRBException e1) {
								e1.printStackTrace();
							}
						}
					}
				}
				fixedOriginSpeed[v] = true;		
			}
		}
	}
	
	// ---------------- Fix Speed Variables before day N -------------------------
	public void fixVoyageSpeed(int n){
		for (int i = 0; i < problem.positions.size(); i++){
			if ((this.t[i] <= n) && (fixedVoyageSpeed[i] == false)){
				for (int v = 0; v < problem.vessels.size(); v++){
					for (int s = 0; s < problem.speedProfiles.size(); s++){
						if (problem.vesselCanSailPosition(problem.vessels.get(v),problem.positions.get(i))){
							try {
							mip.model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").set(GRB.DoubleAttr.LB,this.x[v][i][s]);
							mip.model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").set(GRB.DoubleAttr.UB,this.x[v][i][s]);
							
							} catch (GRBException e) {
								e.printStackTrace();
							}
						}
					}
				}
				fixedVoyageSpeed[i] = true;
			}
		}
	}
	
	// --------------- Fix Speed Variables Before Simulation -------------------------
	
	public void fixSpeedBeforeSimulationSOS(){
		getVariablesFromModel();
		
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int s = 0; s < problem.speedProfiles.size(); s++){
					if (problem.vesselCanSailPosition(problem.vessels.get(v),problem.positions.get(i))){
						try {
						mip.model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").set(GRB.DoubleAttr.LB,this.x[v][i][s]);
						mip.model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").set(GRB.DoubleAttr.UB,this.x[v][i][s]);
						} catch (GRBException e) {
							e.printStackTrace();
						}
					}
				}
			}
		}
	
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int s = 0; s < problem.speedProfiles.size(); s++){
					if (problem.vesselCanSailInitialPosition(problem.vessels.get(v), problem.positions.get(i))){
						try {
							mip.model.getVarByName("x_Bo[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id  + "][" + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.LB,this.x_Bo[v][i][s]);
							mip.model.getVarByName("x_Bo[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id  + "][" + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.UB,this.x_Bo[v][i][s]);
						} catch (GRBException e1) {
							e1.printStackTrace();
						}
					}
				}
			}
		}
	
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int j = 0; j < problem.positions.size(); j++){
					for (int s = 0; s < problem.speedProfiles.size(); s++){
						if (problem.vesselCanSailPositionToPosition(problem.vessels.get(v), problem.positions.get(i), problem.positions.get(j))){
							try {
								mip.model.getVarByName("x_B[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id +  "]["  + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.LB,this.x_B[v][i][j][s]);
								mip.model.getVarByName("x_B[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id +  "]["  + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.UB,this.x_B[v][i][j][s]);
							} catch (GRBException e) {
								e.printStackTrace();
							}
						}
					}
				}
			}		
		}
		
		try {
			mip.model.update();
		} catch (GRBException e) {
			e.printStackTrace();
		}
		
	}
	
	// -------------- Open Speed Variables Before Simulation ------------------------
	
	public void openSpeedBeforeSimulationSOS(){
		
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int s = 0; s < problem.speedProfiles.size(); s++){
					if (problem.vesselCanSailPosition(problem.vessels.get(v),problem.positions.get(i))){
						try {
						mip.model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").set(GRB.DoubleAttr.LB,0);
						mip.model.getVarByName("x[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.speedProfiles.get(s).code  + "]").set(GRB.DoubleAttr.UB,1);
						} catch (GRBException e) {
							e.printStackTrace();
						}
					}
				}
			}
		}
	
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int s = 0; s < problem.speedProfiles.size(); s++){
					if (problem.vesselCanSailInitialPosition(problem.vessels.get(v), problem.positions.get(i))){
						try {
							mip.model.getVarByName("x_Bo[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id  + "][" + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.LB,0);
							mip.model.getVarByName("x_Bo[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id  + "][" + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.UB,1);
						} catch (GRBException e1) {
							e1.printStackTrace();
						}
					}
				}
			}
		}
	
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int j = 0; j < problem.positions.size(); j++){
					for (int s = 0; s < problem.speedProfiles.size(); s++){
						if (problem.vesselCanSailPositionToPosition(problem.vessels.get(v), problem.positions.get(i), problem.positions.get(j))){
							try {
								mip.model.getVarByName("x_B[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id +  "]["  + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.LB,0);
								mip.model.getVarByName("x_B[" + problem.vessels.get(v).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id +  "]["  + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.UB,1);
							} catch (GRBException e) {
								e.printStackTrace();
							}
						}
					}
				}
			}		
		}
		
		try {
			mip.model.update();
		} catch (GRBException e) {
			e.printStackTrace();
		}
		
	}
}

