

import gurobi.GRB;
import gurobi.GRBException;

import java.util.ArrayList;

/*
 * ------- This class contains the Horizon object, and is used to store information when solving each horizon in the RHH
 */
public class Horizon {
	public int primaryPeriod;
	public int forecastingPeriod;
	public int numberOfDummies;
	public double iterationTime;
	//public double startTime;
	public double itrStartTime;
	public double itrEndTime;
	double	objectiveValue;
	Mip mip;
	
	/*
	 * ------------------- Horizon object constructor -----------------------------------
	 */
	
	public Horizon(int counter, Mip mip, double itrStartTime){
		this.mip = mip;
		this.primaryPeriod = counter+1;
		this.forecastingPeriod = getCurEndForecasting(counter, mip.problem.months)+1;
		this.numberOfDummies = 0;
		this.iterationTime = 0;
		//this.startTime = 0;
		this.itrStartTime = itrStartTime;
		this.itrEndTime = 0;
		this.objectiveValue = 0;
		
	}
	
	/*
	 * --------------------- Method to update Horizon object variables from Solution object----------------
	 */
	
	public void addRHHSolution(Mip mip){
		
		int numdummies = 0;
		
		// Update arrays of dummy positions and vessel schedules from Solution object
		for (int i = 0; i < this.mip.problem.positions.size(); i++){
				try {
					if ((this.mip.model.getVarByName("dummy[" + mip.problem.positions.get(i).id +"]").get(GRB.DoubleAttr.X)) > 0.5){
						numdummies++;
					}
				} catch (GRBException e) {
					e.printStackTrace();
				}
			}
		this.numberOfDummies = numdummies;
	}
	
	public void time(double itrStartTime, double itrEndTime){
		this.itrStartTime = itrStartTime;
		this.itrEndTime = itrEndTime;
		this.iterationTime = (itrEndTime - itrStartTime);
	}
	/*
	 * ------------------------- Method to determine end of current forecasting period -------------------------
	 */
	public int getCurEndForecasting(int counter, ArrayList<Month> months){
		if (counter + 2 < months.size()){
			return counter+2;
		}
		else if (counter + 1 < months.size()){
			return counter+1;
		}
		else{
			return(months.size()-1);
		}
	}
}
