import gurobi.*;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

/*
 * This class contains the method for selection of destroy operators, and the destroy operators themselves
 */

public class DestroyOperator {

	
	
	/*
	 *  ---------------------------- Method for choosing destroy operator -------------------------------------
	 */
	
	public static void destroy(Solution solution, Iteration iteration){
		//System.out.println("----------------------------------");
		
		
		// Before destroying, sort vessel schedules and add randomness to search
		for (int v = 0; v < solution.problem.vessels.size(); v++){
			Collections.sort(solution.vessels.get(v).assignedPositions, Position.PositionAssignedStartTimeComparator);
		}
		//Collections.shuffle(solution.vessels); FIKK IKKE DENNE TIL Å FUNGERE
		Collections.shuffle(solution.dummyPositions);
	
		
		// Choose operator
		if (iteration.destroyOperator == 1){
			destroyTradeRoute(solution);
			
		}else if (iteration.destroyOperator == 2){
			destroyPort(solution);
			
		}else if (iteration.destroyOperator == 3){
			destroyRandom(solution);
			
		}else if (iteration.destroyOperator == 4){
			destroyThreeVessels(solution);
		
		}else if (iteration.destroyOperator == 5){
			destroyTwoMonths(solution);
		}
		
		//TEST
		//System.out.println("DESTROYED: " + solution.unassignedPositions.size() + " POSITIONS");
		
		// Before repair, shuffle list of unassigned positions for search diversification
		Collections.shuffle(solution.unassignedPositions);
		
		
		
		//solution.printVariablesDestroyed();
	}

	
	
	
	
	
	/*
	 *  -------------------------- Destroy Trade Route Operator -------------------------------
	 */

	public static void destroyTradeRoute(Solution solution){
		// Choose one trade route to destroy belonging solutions from
		int random = (int )(Math.random() * solution.problem.standardRoutes.size());
		int destroyedPositions = 0;
		
		
		//TEST
		//System.out.println("REMOVED POSITIONS ON TRADE ROUTE [" + solution.problem.standardRoutes.get(random).code + "]");
			
		
		// Attempt to destroy positions covered by dummies and add to the list of unassigned positions, up to a maximum of half the amount of allowed destructions
		for (int i = 0; i < solution.dummyPositions.size(); i++){
			if (
					destroyedPositions < (GlobalConstants.maximumDestructions / 2)
					&& 
					destroyedPositions < GlobalConstants.maximumPercentageDestructions * (solution.problem.positions.size() / 2)
					){
				solution.unassignedPositions.add(solution.dummyPositions.get(i));
				solution.dummyPositions.remove(i);
				i = i - 1;
				destroyedPositions++;
			}
		}
		
		
		// Destroy positions on trade route covered by vessels and add to the list of unassigned positions
		for (int v = 0; v < solution.vessels.size(); v++){
			int i = 0;
			
			while ( i < solution.vessels.get(v).assignedPositions.size()
					&&
					destroyedPositions < GlobalConstants.maximumDestructions 
					&& 
					destroyedPositions < GlobalConstants.maximumPercentageDestructions * solution.problem.positions.size()
					){
				
				// Check if position i matches the criteria
				if (solution.vessels.get(v).assignedPositions.get(i).route.equals(solution.problem.standardRoutes.get(random).code)){
						
					int positionsDestroyed = solution.vessels.get(v).destroyPosition(i, solution);
					destroyedPositions += positionsDestroyed;
												
				}else{
					i++;
				}
			}
		} // End for
		
		
		// Destroy positions on dummies that matches the criteria
		for (int i = 0; i < solution.dummyPositions.size(); i++){
			if (
					destroyedPositions < (GlobalConstants.maximumDestructions / 2)
					&& 
					destroyedPositions < GlobalConstants.maximumPercentageDestructions * (solution.problem.positions.size() / 2)
					&&
					solution.dummyPositions.get(i).route.equals(solution.problem.standardRoutes.get(random).code)
					){
				solution.unassignedPositions.add(solution.dummyPositions.get(i));
				solution.dummyPositions.remove(i);
				i = i - 1;
				destroyedPositions++;
			}
		}

	}
	
	
	
	/*
	 * ------------------------------ Destroy Month Operator -------------------------------------------
	 */
	
	public static void destroyTwoMonths(Solution solution){
		
		// Choose one month to destroy belonging solutions from
		int randomMonth1 = (int )(Math.random() * (solution.problem.months.size()-1));
		int randomMonth2 = randomMonth1 + 1;
		int destroyedPositions = 0;

		
		// Destroy positions covered by dummies and add to the list of unassigned positions
		for (int i = 0; i < solution.dummyPositions.size(); i++){
			if (destroyedPositions < (GlobalConstants.maximumDestructions / 2)
					&& 
					destroyedPositions < (GlobalConstants.maximumPercentageDestructions * solution.problem.positions.size() / 2)
					){
				
				solution.unassignedPositions.add(solution.dummyPositions.get(i));
				solution.dummyPositions.remove(i);
				i = i - 1;
				destroyedPositions++;
				
			}
		}
		
		
		// Destroy positions on trade route covered by vessels and add to the list of unassigned positions
		for (int v = 0; v < solution.vessels.size(); v++){
			int i = 0;
				
			while ( i < solution.vessels.get(v).assignedPositions.size()
					&&
					destroyedPositions < GlobalConstants.maximumDestructions 
					&& 
					destroyedPositions < GlobalConstants.maximumPercentageDestructions * solution.problem.positions.size()
					){
					
				// Check if position i matches the criteria
				if (solution.vessels.get(v).assignedPositions.get(i).month.equals(solution.problem.months.get(randomMonth1).code) || solution.vessels.get(v).assignedPositions.get(i).month.equals(solution.problem.months.get(randomMonth2).code)){
							
					int positionsDestroyed = solution.vessels.get(v).destroyPosition(i, solution);
					destroyedPositions += positionsDestroyed;
														
				}else{
					i++;
				}
			}
		} // End for all v
		
		
		// Destroy positions on dummies that matches the criteria
		for (int i = 0; i < solution.dummyPositions.size(); i++){
			if (
					destroyedPositions < (GlobalConstants.maximumDestructions / 2)
					&& 
					destroyedPositions < GlobalConstants.maximumPercentageDestructions * (solution.problem.positions.size() / 2)
					&&
					(solution.dummyPositions.get(i).month.equals(solution.problem.months.get(randomMonth1).code) || solution.dummyPositions.get(i).month.equals(solution.problem.months.get(randomMonth2).code))
					){
				solution.unassignedPositions.add(solution.dummyPositions.get(i));
				solution.dummyPositions.remove(i);
				i = i - 1;
				destroyedPositions++;
			}
		}
		 
	}
	
	
	
	/*
	 * ------------------------------ Destroy Random Positions Operator ----------------------------------
	 */
	
	
	public static void destroyRandom(Solution solution){
		
		// Destroy a set of random positions
		int destroyedPositions = 0;
		
		
		// Destroy positions covered by dummies
		for (int i = 0; i < solution.dummyPositions.size(); i++){
			if (destroyedPositions < (GlobalConstants.maximumDestructions / 2)
					&& 
					destroyedPositions < (GlobalConstants.maximumPercentageDestructions * solution.problem.positions.size() / 2)
					){
				
				solution.unassignedPositions.add(solution.dummyPositions.get(i));
				solution.dummyPositions.remove(i);
				i = i - 1;
				destroyedPositions++;
				
			}
		}
		
		
		// Destroy random positions and add to the list of unassigned positions
		boolean voyageFound = true;
		int v = 0;
		int i = 0;
		
		while (
				destroyedPositions < GlobalConstants.maximumDestructions 
				&& 
				destroyedPositions < GlobalConstants.maximumPercentageDestructions * solution.problem.positions.size()
				){
			
			
			// Choose one random voyage to destroy
			int random = (int)(Math.random() * solution.problem.positions.size());
			String randomId = solution.problem.positions.get(random).id;
			voyageFound = false;
			v = 0;
			
			while(!voyageFound && v < solution.vessels.size()){
				
				i = 0;
				
				while(!voyageFound && i < solution.vessels.get(v).assignedPositions.size()){
					
					if (randomId.equals(solution.vessels.get(v).assignedPositions.get(i).id)){
						
						int positionsDestroyed = solution.vessels.get(v).destroyPosition(i, solution);				
						destroyedPositions += positionsDestroyed;
						voyageFound = true;
						
					}else{
						i++;
					}
				}
				
				v++;
			}
			
			
			// Else, the random voyage may be covered by a dummy vessel
			i = 0;
			while (!voyageFound && i < solution.dummyPositions.size()){
				if (randomId.equals(solution.dummyPositions.get(i).id)){
					solution.unassignedPositions.add(solution.dummyPositions.get(i));
					solution.dummyPositions.remove(i);
					voyageFound = true;
				}else{
					i++;
				}
			}
									
		} // End while
		
	}
	
	
	
	/*
	 * ------------------------------ Destroy Three Vessels Operator ----------------------------------
	 */
	
	public static void destroyThreeVessels(Solution solution){
		// Destroy positions assigned to three random vessels
		int destroyedPositions = 0;

		
		// First, destroy positions covered by dummies and add to the list of unassigned positions
		for (int i = 0; i < solution.dummyPositions.size(); i++){
			if (destroyedPositions < (GlobalConstants.maximumDestructions / 2)
					&& 
					destroyedPositions < (GlobalConstants.maximumPercentageDestructions * solution.problem.positions.size() / 2)
					){
				
				solution.unassignedPositions.add(solution.dummyPositions.get(i));
				solution.dummyPositions.remove(i);
				i = i - 1;
				destroyedPositions++;
				
			}
		} 
		
		
		int numberOfVessels = solution.vessels.size();
		int vessel1 = (int)(Math.random() * numberOfVessels);
		int vessel2 = (int)(Math.random() * numberOfVessels);
		int vessel3 = (int)(Math.random() * numberOfVessels);

		// If the number of vessels is greater than one, and the second vessel is the same as the first, find another
		if (numberOfVessels > 1){
			while (vessel1 == vessel2){
				vessel2 = (int)(Math.random() * numberOfVessels);
			}
		}
		
		
		// If the number of vessels is greater than two and the third vessel is the same as the first or the second, find another
		if (numberOfVessels > 2){
			while (vessel2 == vessel3 || vessel1 == vessel3){
				vessel3 = (int)(Math.random() * numberOfVessels);
			}
		}

		
		// Destroy positions covered by these vessels, and add them to the list of unassigned positions
		for (int v = 0; v < solution.vessels.size(); v++){
			if ((v == vessel1 || v == vessel2 || v == vessel3) && (solution.vessels.get(v).assignedPositions.size() <= GlobalConstants.maximumDestructions - destroyedPositions)){
				int removedPositions = solution.vessels.get(v).destroyVessel(solution);
				destroyedPositions += removedPositions;
			}
		}
		
	
	}	
	
	
	

	/*
	 * ------------------------------ Destroy Port Operator -------------------------------------------
	 */
	
	public static void destroyPort(Solution solution){
		
		// Choose months to destroy belonging solutions from
		int randomMonth1 = (int )(Math.random() * (solution.problem.months.size()-2));
		int randomMonth2 = randomMonth1 + 1;
		int randomMonth3 = randomMonth1 + 2;
		int destroyedPositions = 0;

		
		// Choose port to destroy
		String port = solution.problem.ports.get((int) (Math.random() * (solution.problem.ports.size()))).code;
		
		
		
		// Destroy positions covered by dummies and add to the list of unassigned positions
		for (int i = 0; i < solution.dummyPositions.size(); i++){
			if (destroyedPositions < (GlobalConstants.maximumDestructions / 2)
					&& 
					destroyedPositions < (GlobalConstants.maximumPercentageDestructions * solution.problem.positions.size() / 2)
					){
				
				solution.unassignedPositions.add(solution.dummyPositions.get(i));
				solution.dummyPositions.remove(i);
				i = i - 1;
				destroyedPositions++;
				
			}
		}
		
		

		
		// Destroy positions meeting the criteria covered by vessels and add to the list of unassigned positions
		for (int v = 0; v < solution.vessels.size(); v++){
			int i = 0;
			
			while ( i < solution.vessels.get(v).assignedPositions.size()
					&&
					destroyedPositions < GlobalConstants.maximumDestructions 
					&& 
					destroyedPositions < GlobalConstants.maximumPercentageDestructions * solution.problem.positions.size()
					){
				
				// Check if position i matches the criteria
				if ((solution.vessels.get(v).assignedPositions.get(i).month.equals(solution.problem.months.get(randomMonth1).code) 
						|| 
						solution.vessels.get(v).assignedPositions.get(i).month.equals(solution.problem.months.get(randomMonth2).code) 
						|| 
						solution.vessels.get(v).assignedPositions.get(i).month.equals(solution.problem.months.get(randomMonth3).code))
						&&
						(port.equals(StandardRoute.getStandardRouteStartPort(solution.problem.standardRoutes, solution.vessels.get(v).assignedPositions.get(i).route))
						||
						port.equals(StandardRoute.getStandardRouteEndPort(solution.problem.standardRoutes, solution.vessels.get(v).assignedPositions.get(i).route)))
						){					
					
					int positionsDestroyed = solution.vessels.get(v).destroyPosition(i, solution);
					destroyedPositions += positionsDestroyed;
									
				}else{
					i++;
				}
			}
		}
		
		
		// Destroy positions on dummies that matches the criteria
		for (int i = 0; i < solution.dummyPositions.size(); i++){
			if (
					destroyedPositions < (GlobalConstants.maximumDestructions / 2)
					&& 
					destroyedPositions < GlobalConstants.maximumPercentageDestructions * (solution.problem.positions.size() / 2)
					&&
					(solution.dummyPositions.get(i).month.equals(solution.problem.months.get(randomMonth1).code) || solution.dummyPositions.get(i).month.equals(solution.problem.months.get(randomMonth2).code))
					){
				solution.unassignedPositions.add(solution.dummyPositions.get(i));
				solution.dummyPositions.remove(i);
				i = i - 1;
				destroyedPositions++;
			}
		}
			 
	}
	

	
	
	/*
	 * ------------------- Method to check if a position can be removed from a vessel's schedule ----------------
	 */
	
	public static boolean PositionCanBeRemovedFromVessel(Vessel v, int position, Problem problem){
		int numberOfPositions = v.assignedPositions.size();
		boolean removalIsPossible = false;
		
		
		if (numberOfPositions == 1){
			removalIsPossible = true;
		}
		
		else if (
				numberOfPositions >= 2 
				&& 
				position == 0 
				&& 
				problem.vesselCanSailInitialPosition(v, v.assignedPositions.get(1))
				){
			removalIsPossible = true;
		}
		
		else if (
				numberOfPositions >= 3
				&&
				position < (numberOfPositions-1)
				&&
				position > 0
				&&
				problem.vesselCanSailPositionToPosition(v, v.assignedPositions.get(position-1), v.assignedPositions.get(position+1))
				){
			removalIsPossible = true;
		}
		
		else if (
				numberOfPositions >= 2
				&&
				position == (numberOfPositions-1)
				){
			removalIsPossible = true;
		}		
		
		return removalIsPossible;
	}
	
	
	
	/*
	 * ------------------- Method to check if two positions in succession can be removed from a vessel's schedule ----------------
	 */
	
	public static boolean ConnectedPositionsCanBeRemovedFromVessel(Vessel v, int position, Problem problem){
		int numberOfPositions = v.assignedPositions.size();
		boolean removalIsPossible = false;
		
		
		if (
				numberOfPositions == 2 
				&&
				position == 0
				){
			removalIsPossible = true;
		}
		
		else if(
				numberOfPositions >= 3
				&&
				position == 0
				&&
				problem.vesselCanSailInitialPosition(v, v.assignedPositions.get(2))
				){
			removalIsPossible = true;
		}
		
		else if(
				numberOfPositions >= 3
				&&
				position > 0
				&&
				position < (numberOfPositions - 2)
				&&
				problem.vesselCanSailPositionToPosition(v, v.assignedPositions.get(position-1), v.assignedPositions.get(position+2))
				){
			removalIsPossible = true;
		}
		
		else if(
				numberOfPositions >= 3
				&&
				position == (numberOfPositions - 2)
				){
			removalIsPossible = true;
		}
		
		return removalIsPossible;
	}
	
	
	

}
