import java.util.ArrayList;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;


public class Sailing {
	public String code;
  public String route;
  public String vessel;
  public String speedProfileCode;
  public double duration;
  public double costs;

  public Sailing(String aa, String bb, String cc, String dd, double ee, double ff) {
    code = aa;
    route = bb;
    vessel = cc;
    speedProfileCode = dd;
    duration = ee;
    costs = ff;
  }
  
  
  
  /*
   * ------------------------- Method to retrieve costs of a vessel sailing a given position at a given speed ---------------------------
   */
  
  public static double getCosts(String vessel, String route, String speedProfileCode, ArrayList<Sailing> sailings){
  	double costs = -1.00;
  	
  	for (int n = 0; n < sailings.size(); n++){
  		if (vessel.equals(sailings.get(n).vessel) && route.equals(sailings.get(n).route) && speedProfileCode.equals(sailings.get(n).speedProfileCode)){
  			costs = sailings.get(n).costs;
  		}
  	}
  	
  	/*
  	if (costs < 0){
  		System.out.println("ERROR: COSTS OF SAILING NOT FOUND: [" + vessel + "][" + route + "][" + speedProfileCode + "]");
  	}
  	*/
  	
  	return costs;
  }
  
  
  
  public static double getHighestCosts(String vessel, String route, ArrayList<Sailing> sailings){
  	double costs = 999999999999999999.9;
  	int length = sailings.size();
  	
  	for (int n = 0; n < length; n++){
  		if (vessel == sailings.get(n).vessel && route == sailings.get(n).route && sailings.get(n).costs < costs){
  			costs = sailings.get(n).costs;
  		}
  	}
  	
  	return costs;
  }
  
  
  
  /*
   * -------------------------------- Method to retrieve sailing duration of a voyage for a vessel at a given speed -------------------------------
   */
  
  public static double getSailingDuration(String vessel, String route, String speedProfileCode, ArrayList<Sailing> sailings){
  	double duration = -1.00;
  	
  	for (int n = 0; n < sailings.size(); n++){
  		if (vessel.equals(sailings.get(n).vessel) && route.equals(sailings.get(n).route) && speedProfileCode.equals(sailings.get(n).speedProfileCode)){
  			duration = sailings.get(n).duration;
  		}
  	}
  	
  	if (duration < 0){
  		System.out.println("ERROR: SAILING DURATION NOT FOUND");
  		System.exit(1);
  	}
  	
  	return duration;
  }
  
  
  
  /*
   *  -------------------------------- Method to retrieve shortest sailing duration of a voyage for a vessel -------------------------------
   */
  
  public static double getShortestSailingDuration(String vessel, String route, ArrayList<Sailing> sailings){
  	double duration = -1.00;
  	
  	for (int n = 0; n < sailings.size(); n++){
  		if (vessel.equals(sailings.get(n).vessel) && route.equals(sailings.get(n).route) && duration < 0){
  			duration = sailings.get(n).duration;
  		}
  		else if (vessel.equals(sailings.get(n).vessel) && route.equals(sailings.get(n).route) && sailings.get(n).duration < duration){
  			duration = sailings.get(n).duration;
  		}
  	}
  	if (duration > 0){
    	return duration;
  	}else{
  		//System.out.println("ERROR: SAILING OF VOYAGE NOT POSSIBLE (GETSHORTESTSAILINGDURATION)");
  		return duration;
  	}
  }
  
  
  
  /*
   * ---------------------------- Method to retrieve longest sailing duration of all sailings -------------------------
   */
  
  public static double getLongestSailingDuration(ArrayList<Sailing> sailings){
  	double longestDuration = -1.00;
  	
  	for (int n = 0 ; n < sailings.size() ; n++){
  		if (sailings.get(n).duration > longestDuration){
  			longestDuration = sailings.get(n).duration;
  		}
  	}
  	return longestDuration;
  }
  
  
  
	public static ArrayList<Sailing> importSailings(HSSFWorkbook workbook){
  	HSSFSheet sailingSheet = workbook.getSheet("Sailing");
  	int numberOfRowsInSailingSheet = sailingSheet.getLastRowNum();
		
		ArrayList<Sailing> sailings = new ArrayList<Sailing>();
				
		for (int rowNum = 1; rowNum <= numberOfRowsInSailingSheet; rowNum++){
			Row row = sailingSheet.getRow(rowNum);
			Cell c = row.getCell(0);
			
			if (c != null){
			Cell sailingCode = row.getCell(0);
		  sailingCode.setCellType(Cell.CELL_TYPE_STRING);
		  Cell sailingRoute = row.getCell(1);
		  sailingRoute.setCellType(Cell.CELL_TYPE_STRING);
		  Cell sailingVessel = row.getCell(2);
		  sailingVessel.setCellType(Cell.CELL_TYPE_STRING);
		  Cell sailingSpeedProfileCode = row.getCell(3);
		  sailingSpeedProfileCode.setCellType(Cell.CELL_TYPE_STRING);
		  Cell sailingDuration = row.getCell(4);			  
		  Cell sailingCosts = row.getCell(5);
			
			sailings.add(new Sailing(
					(String)sailingCode.getStringCellValue(),
					(String)sailingRoute.getStringCellValue(),
					(String)sailingVessel.getStringCellValue(),
					(String)sailingSpeedProfileCode.getStringCellValue(),
					(double)sailingDuration.getNumericCellValue()/100,
					(double)sailingCosts.getNumericCellValue()/100));
				}
		 	}
		return sailings;
  }
}
	
