
import gurobi.GRB;
import gurobi.GRBException;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
 
public class Model {
    static List<String> testInstances;
         
        public static void main(String[] args){
            testInstances = TestInstances.testInstances;
   
    		try {
         
        /* 
         * ------------------ Import data file -----------------------------------
         */
    	
    			for (int currentIntInstance = 0; currentIntInstance < testInstances.size(); currentIntInstance++){
    				String problemFile =  GlobalConstants.DirectoryTestInstancesInput + testInstances.get(currentIntInstance);
         
         
        		FileInputStream fileInputStream = new FileInputStream(problemFile);
        		HSSFWorkbook workbook = new HSSFWorkbook(fileInputStream);
        		
		/* 
         * ------------------ Create excel file -----------------------------------
         */		
        		FileOutputStream fileOut = new FileOutputStream(GlobalConstants.DirectoryOutput+"SimulationOutput_" + TestInstances.testInstances.get(currentIntInstance));
        		HSSFWorkbook workbookout = new HSSFWorkbook();
         
 
        /* 
         * ------------------ Build problem and mixed-integer program from data file ----------------------- 
         */
             
            // Problem contains the information from the test instances is solved
            Problem problem = new Problem(workbook, problemFile);
            
            
 
             
        /*
         * ------------------ Send the problem to a chosen solver -----------------------------
         */
            // MIP     
            if (GlobalConstants.mip == true){
            	Mip mip = new Mip(problem);
            	String robustnessMethod = "";
         
                	// 1 - Adjust sailing times
          				if (GlobalConstants.ast){
          					mip.adjustSailingTimes();
          					robustnessMethod = "ast";
          				}
          				
          				// 2 - Reward early arrivals
          				if(GlobalConstants.rea){
          					mip.rewardEarlyArrivals();
          					robustnessMethod = "rea";

          				}
          				
          				// 3 - Penalize risky start times
          				if(GlobalConstants.prst){
          					mip.penalizeRiskyStartTimes();
          					robustnessMethod = "prst";

          				}
          				if(GlobalConstants.ast && GlobalConstants.rea && GlobalConstants.prst){
          					robustnessMethod = "combined";
          				}
          			
                // SOLVE TO OPTIMALITY
                mip.solveExact(currentIntInstance);
            	
            	// INITIAL SOLUTION
                //mip.findInitialFeasibleSolution();
                
                // SOLVE FIXED SOLUTION READ FROM FILE
                //mip.solveFixedLP(variablesFile);
                
                mip.writeMIPtoFile(currentIntInstance);
              //SolutionPrinter.printSolution(mip.model, problem);
                
             // Print solution to file
	            try {
					mip.printAllVariables(currentIntInstance, robustnessMethod);
				} catch (GRBException e) {
					e.printStackTrace();
				}
            }
            
            // ALNS
            if (GlobalConstants.alns == true){
            	Mip mipALNS = new Mip(problem);
            	AdaptiveLargeNeighborhoodSearch alns = new AdaptiveLargeNeighborhoodSearch(mipALNS, problem, currentIntInstance);
            	alns.solveALNS(); 
            }
             
            //RHH
            if (GlobalConstants.rhh == true){
            	Mip mipRHH = new Mip(problem);
            	String robustnessMethod = "";
     
            	// 1 - Adjust sailing times
      				if (GlobalConstants.ast){
      					mipRHH.adjustSailingTimes();
      					robustnessMethod = "ast";
      				}
      				
      				// 2 - Reward early arrivals
      				if(GlobalConstants.rea){
      					mipRHH.rewardEarlyArrivals();
      					robustnessMethod = "rea";

      				}
      				
      				// 3 - Penalize risky start times
      				if(GlobalConstants.prst){
      					
      					mipRHH.penalizeRiskyStartTimes();
      					robustnessMethod = "prst";

      				}
      				if(GlobalConstants.ast && GlobalConstants.rea && GlobalConstants.prst){
      					robustnessMethod = "combined";
      				}
	            
      			RollingHorizonHeuristic rhh = new RollingHorizonHeuristic(mipRHH, currentIntInstance, robustnessMethod);
	            
	            try {
	            	rhh.solveRHH();
	            	
	            	// Print solution to file
		            mipRHH.printAllVariables(currentIntInstance, robustnessMethod);
		            
	            }catch (GRBException e) {
	            	e.printStackTrace();
	            }
	                   
            }
            
           // Solve with RHH and simulate with disruptive events
            if (GlobalConstants.simulation == true){
            	int robustMethodLower = 0;
            	int robustMethodUpper = 1;
            	int recoveryMethodLower = 0;
            	int recoveryMethodUpper = 1;
            	
            	// When running the metods in parallell, this is not enabled. If the methods are runned after each other, the 
            	/*
            	if(GlobalConstants.basic == true){robustMethodLower = 0; robustMethodUpper = 1;}
            	else if(GlobalConstants.ast == true){robustMethodLower = 1; robustMethodUpper = 2;}
            	else if(GlobalConstants.rea == true){robustMethodLower = 2; robustMethodUpper = 3;}
            	else if(GlobalConstants.prst == true){robustMethodLower = 3; robustMethodUpper = 4;}
            	
            	if(GlobalConstants.ast == true && GlobalConstants.rea == true && GlobalConstants.prst == true){robustMethodUpper = 4;}
            	else if(GlobalConstants.rea == true && GlobalConstants.prst == true){robustMethodUpper = 4;}
            	else if(GlobalConstants.ast == true && GlobalConstants.rea == true){robustMethodUpper = 3;}
            	else if(GlobalConstants.basic == true && GlobalConstants.ast == true){robustMethodUpper = 2;}
            	*/
            	
            	
            	if(GlobalConstants.noRecovery == false){recoveryMethodLower = 1;}
            	if(GlobalConstants.RecoveryRHH == true){recoveryMethodUpper = 2;}
            	
	           for(int robustMethod = robustMethodLower ; robustMethod < robustMethodUpper; robustMethod ++){
	            	for(int recoveryMethod = recoveryMethodLower; recoveryMethod < recoveryMethodUpper; recoveryMethod ++){
	            		Simulator sim = new Simulator(problem, currentIntInstance, robustMethod, recoveryMethod, workbookout); 
						sim.simulate();
	            	}
	            }
	            SimulationStatistics.writeSummarySheetInExcel(currentIntInstance, workbookout);
            }
            
            // SequentialRoutingAndSpeedOptimization
            
            if (GlobalConstants.SRASO == true){
            	for(int currentSpeedProfile = 0; currentSpeedProfile < problem.speedProfiles.size(); currentSpeedProfile ++){
            		SequentialRoutingAndSpeedOptimization sraso = new SequentialRoutingAndSpeedOptimization(currentIntInstance, currentSpeedProfile, problem, problem);
            		sraso.solveSRASO();
            	}
            }
            
			// SequentialRHHRoutingAndMIPSpeedOptimization
            if (GlobalConstants.RHHrMIPs == true){
            	for(int currentSpeedProfile = 0; currentSpeedProfile < problem.speedProfiles.size(); currentSpeedProfile ++){
            		RHHroutingMIPspeed rhhrmips = new RHHroutingMIPspeed(currentIntInstance, currentSpeedProfile, problem, problem);
            		rhhrmips.solveRHHrMIPs();
            	}
            }
            
             

        }} catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
			e.printStackTrace();
		}
    }
}





