
import gurobi.GRB;
import gurobi.GRBException;

import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Date;

public class SequentialRoutingAndSpeedOptimization {
	Mip mip;
	Problem problemRouting;
	Problem problemSO;
	int counter;
	double startTime;
	double endTime;
	double endTimeRouting;
	int currentInstance;
	int currentSpeedProfile;
	
	// Constructor of RHH object
			public SequentialRoutingAndSpeedOptimization(int currentInstance, int currentSpeedProfile, Problem problemRouting, Problem problemSO){
				this.mip = mip;
				this.problemRouting = problemRouting;
				this.problemSO = problemSO;
				this.counter = 0;
				this.startTime = 0;
				this.endTime = 0;
				this.currentInstance = currentInstance;
				this.currentSpeedProfile = currentSpeedProfile;
				this.endTimeRouting = 0;
			}
			
			/*
			 * ----------------------- Method find a solution having only one speed profile available, then solving the fixed LP and optimizing the speed profiles ----------------------------
			 */
			public void solveSRASO(){
				
				// Recording the starting time for the model
				this.startTime = System.currentTimeMillis();
				
				// The speed optimization problem is created
				Mip mipSO = new Mip(problemSO);
				
				// Save the current speed profile
				ArrayList<String> CurrentSpeedProfile = new ArrayList<>();
				CurrentSpeedProfile.add(problemRouting.speedProfiles.get(currentSpeedProfile).code);
				
				// Save the speed profiles that are to be removed
				ArrayList<String> RemovedSpeedProfiles = new ArrayList<>();
				for (int sp = 0; sp < problemRouting.speedProfiles.size(); sp ++){
					if (sp != currentSpeedProfile){
						RemovedSpeedProfiles.add(problemRouting.speedProfiles.get(sp).code);
					}
				}
				
				
				//Removing all of the speed profiles from the current problem
				problemRouting.speedProfiles.clear();
			
				//Adding the current speedprofile
				problemRouting.speedProfiles.add(new SpeedProfile((String)CurrentSpeedProfile.get(0)));
				
				
					
				// Creating a MIP based on the problem with only one speedprofile available
				Mip mipRouting = new Mip(problemRouting);
			    
								
				// Solve the exact model and print solution to file
				mipRouting.solveExact(this.currentInstance);
		
				
				// Print solution to a seperate text file
				try {
					mipRouting.printVariables(currentInstance);
				} catch (GRBException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
				// Record ending time for the routing-process
				this.endTimeRouting = System.currentTimeMillis();
				
				mipRouting.writeSRASOtoFile(currentInstance, CurrentSpeedProfile.get(0), "MIP-Routing", startTime, endTimeRouting);
				
		
				
				// Add the removed speed profiles
				for (int sp = 0; sp < RemovedSpeedProfiles.size(); sp++){
					problemSO.speedProfiles.add(new SpeedProfile((String)RemovedSpeedProfiles.get(sp)));
				}
				
				
				// Solve the problem with speed optimization
				mipSO.solveFixedLP("SolutionVariables_" + TestInstances.testInstances.get(currentInstance)+".txt");
				
				// Record ending time for the SRASO
				this.endTime = System.currentTimeMillis();
				
				// Write the speed optimization results to file
				mipSO.writeSRASOtoFile(currentInstance, CurrentSpeedProfile.get(0), "MIP-Routing and MIP-Speed Optimization", startTime, endTime);
				
			}

		
			
		
			
}


