
import gurobi.GRB;
import gurobi.GRBException;
import gurobi.GRBVar;

import java.util.ArrayList;

import gurobi.GRB.DoubleAttr;
import gurobi.GRBEnv;
import gurobi.GRBLinExpr;
import gurobi.GRBModel;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Date;


public class SimulationEvents {

	public ArrayList<StandardRoute> routeDisruptions;
	public ArrayList<Integer> daysRouteDisruptionsOccur;
	public ArrayList <Port> portDisruptions;
	public ArrayList <Integer> daysPortDisruptionsOccur;
	public ArrayList<StandardRoute> preDeterminedRouteDisruptions;
	public ArrayList<Integer> preDeterminedDaysRouteDisruptionsOccur;
	public ArrayList <Port> preDeterminedPortDisruptions;
	public ArrayList <Integer> preDeterminedDaysPortDisruptionsOccur;
	
	public ArrayList<String> disruptedRoutes;
	public ArrayList<Integer> routeDisruptionDays;
	public ArrayList<Integer> portDisruptionDays;
	public ArrayList<String> disruptedPorts;
	
	
	public Mip mip;
	public Problem problem;
	public int currentIntInstance;
	public int simulationNumber;
	SimulationEvents(Problem problem, int currentIntInstance, int simulationNumber){
		this.mip = null;
		this.problem = problem;
		this.currentIntInstance = currentIntInstance;
		routeDisruptions = new ArrayList<StandardRoute>();
		daysRouteDisruptionsOccur =  new ArrayList<Integer>();
		portDisruptions = new ArrayList<Port>();
		daysPortDisruptionsOccur = new ArrayList<Integer>();
		preDeterminedRouteDisruptions = new ArrayList<StandardRoute>();
		preDeterminedDaysRouteDisruptionsOccur =  new ArrayList<Integer>();
		preDeterminedPortDisruptions = new ArrayList<Port>();
		preDeterminedDaysPortDisruptionsOccur = new ArrayList<Integer>();
		this.simulationNumber = simulationNumber;
		
		this.routeDisruptionDays = new ArrayList<Integer>();
		this.disruptedRoutes = new ArrayList<String>();
		this.portDisruptionDays = new ArrayList<Integer>();
		this.disruptedPorts = new ArrayList<String>();
	}
	
	// This method checks whether any route are disrupted on the current day. The route and the day are stored listOfDisruptedRoutes and routeDisruptionDays
	public boolean sailingDisruption(int n, int r){
		double randomNumberS = Math.random();
		if (randomNumberS < GlobalConstants.sailingDisruptionProbability){
			return true;
		}else{
			return false;
		}
		
	}
	
	// If there is a sailing disruption on the route, this route is added to the routeDisruptions-list. Also, the day the disruption occur is added to the daysRouteDisruptionOccur-liste
	public void setRoutesDisruptedBySailing(StandardRoute routeDisrupted, int routeDisruptedMoment){
		routeDisruptions.add(routeDisrupted);
		daysRouteDisruptionsOccur.add(routeDisruptedMoment);
	}
	
	// This method returns true if a port disruption occurs at port number p at day n.
	public boolean portDisruption(int n, int p){
		double randomNumberP = Math.random();
		if (randomNumberP < GlobalConstants.portDisruptionProbability){
			return true;
		}else{
			return false;
		}
	}
	
	// If there is a disruption in a port, this port is added to the portDisruptions-list. Also, the day the disruption occur is added to the daysPortDisruptionOccur-liste
	public void setPortsDisruptedByPorts(Port portDisrupted, int portDisruptedMoment){
		portDisruptions.add(portDisrupted);
		daysPortDisruptionsOccur.add(portDisruptedMoment);
	}
	
	/*
	 * This method prints the disruptive events for sailing and ports. The two sections are divided with the line "DisruptivePorts
	 */
	public void storeDisruptedEvents(){
		for (int i = 0 ; i < portDisruptions.size(); i ++){
			System.out.println(portDisruptions.get(i).code);
			System.out.println(Integer.toString(daysPortDisruptionsOccur.get(i)));
		}
		Writer writer = null;
		try {
			writer = new BufferedWriter(new OutputStreamWriter(
		    new FileOutputStream(GlobalConstants.DirectorySimEvents+"Sim_" + Integer.toString(simulationNumber) + "_ DisruptedEvents" + TestInstances.testInstances.get(currentIntInstance)+".txt"), "utf-8"));
		
			for (int i = 0; i < this.routeDisruptions.size(); i++){
			writer.write(this.routeDisruptions.get(i).code + "," + Integer.toString(this.daysRouteDisruptionsOccur.get(i)));	
		    writer.write(System.getProperty( "line.separator" ));		
		}
		writer.write("DisruptivePorts");
		writer.write(System.getProperty( "line.separator" ));

		for (int i = 0; i < this.portDisruptions.size(); i++){
			writer.write(portDisruptions.get(i).code +"," + Integer.toString(this.daysPortDisruptionsOccur.get(i)));	
			writer.write(System.getProperty( "line.separator" ));
		}		
		} catch (IOException ex) {
		} finally {
		try {writer.close();} catch (Exception ex) {}
		}
			
	}
	
	/*
	 * This method reads the printed file with the disruptive events for sailing and ports.
	 */
	public void getDisruptedEvents(){
		try{
			//File file = new File(GlobalConstants.DirectorySimEvents+"Sim_" +  Integer.toString(simulationNumber)  + "_ DisruptedEvents" + TestInstances.testInstances.get(currentIntInstance)+".txt"); //Kan det v�re en feil her med filnavnet?
			File file = new File(GlobalConstants.DirectorySimEvents+"Sim_" + GlobalConstants.SimEventFile + "_ DisruptedEvents" + TestInstances.testInstances.get(currentIntInstance)+".txt"); //Kan det v�re en feil her med filnavnet?
			FileReader fileReader = new FileReader(file);
			BufferedReader bufferedReader = new BufferedReader(fileReader);
			StringBuffer stringBuffer = new StringBuffer();
			
			String line;
			// When the line "DisruptivePorts" is read in the file, the boolean "portDisruption" will be set as true, and the port disruptions will be read
			boolean portsDisruption = false;
			while ((line = bufferedReader.readLine()) != null) {
				System.out.println(line);
				if (line.equals("DisruptivePorts")){
					portsDisruption = true;
					stringBuffer.append(line);
					stringBuffer.append("\n");
				}
				// 1. The following section reads the sailing disruptions
				else if (portsDisruption == false) {
					stringBuffer.append(line);
					stringBuffer.append("\n");
					String[] parts = line.split(",");
					String part1 = parts[0];
					String part2 = parts[1];
					preDeterminedRouteDisruptions.add(findRouteFromCode(part1));
					preDeterminedDaysRouteDisruptionsOccur.add(Integer.parseInt(part2));
				}
				// 2. The following section reads the port disruptions
				else{
					stringBuffer.append(line);
					stringBuffer.append("\n");
					String[] parts = line.split(",");
					String part1 = parts[0];
					String part2 = parts[1];
					preDeterminedPortDisruptions.add(findPortFromCode(part1));
					preDeterminedDaysPortDisruptionsOccur.add(Integer.parseInt(part2));
				}
				line = "";
			}
			fileReader.close();
			System.out.println("Contents of file:");
			System.out.println(stringBuffer.toString());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public StandardRoute findRouteFromCode(String code){
		StandardRoute route = null;
		for (int r = 0; r < this.problem.standardRoutes.size(); r ++){
			if (code.equals(problem.standardRoutes.get(r).code)){
				route = this.problem.standardRoutes.get(r);
			}
		}
		if (route == null){
			System.out.println("Something is wrong with the method findRouteFromCode in the SimulationEvents class");
			System.exit(1);	
		}
		return route;
	}
	
	public Port findPortFromCode(String code){
		Port port = null;
		for (int p = 0; p < this.problem.ports.size(); p ++){
			if (code.equals(this.problem.ports.get(p).code)){
				port = this.problem.ports.get(p);
			}
		}
		if (port == null){
			System.out.println("Something is wrong with the method findPortFromCode in the SimulationEvents class");
			System.exit(1);	
		}
		return port;
	}
	
	// Making the mip in simEvents equal to the mip created here in simulation-object
	public void setMip(Mip mip){
		this.mip = mip;
	}

}