import gurobi.*;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import java.util.*;

/*
 * This class contains the repair operators.
 */

public class RepairOperator {

	
	/*
	 * ----------------------------- Method for repairing a destroyed solution -------------------------------
	 */
	
	public static void repair(Solution solution, Iteration iteration, Mip mip){
		
		
		for (int v = 0; v < solution.problem.vessels.size(); v++){
			Collections.sort(solution.vessels.get(v).assignedPositions, Position.PositionAssignedStartTimeComparator);
		}
		
				/*
				 * --------------------- Greedy insertion operator ---------------------
				 */
			
				if (iteration.repairOperator == 1){
									
					greedyInsertion(solution);
									
				
				/*
				 * ------------------ Regret-2 insertion operator --------------------------
				 */
								
				}else if (iteration.repairOperator == 2){
									
					// Choose randomly the level of regret to include ( k = 2, 3, 4)
					int k = 2 + (int)(Math.random() * ((4 - 2) + 1)); 
					
					//TEST
					//System.out.println("k is now: " + k);
					
					regret_k_Insertion(solution, k);
									
				/*
				 * ------------------- Deep greedy insertion operator --------------------------
				 */
								
				}else if (iteration.repairOperator == 3){
									
					deepGreedyInsertion(solution);
									
									
				/*
				 * -------------------- MIP repair operator --------------------------
				 */
								
				}else if (iteration.repairOperator == 4){
								
					boolean possibleSolution = true;
					possibleSolution = callMipSolver(solution, mip);
					solution.updateSolutionObject(mip);
				
					//TEST
					if (!possibleSolution){
						solution.printToTextFile();
						solution.printVariables();
						System.exit(1);
					}
				}
				
			
	}
	
	
	
	
	
	/*
	 * ---------------------------- Greedy insertion repair operator ----------------------------
	 */
	
	public static void greedyInsertion(Solution solution){
			
		// Create object that contains information regarding cheapest insertion possibility and declare search parameters
		Assignment assignment = new Assignment();
		boolean insertionFound = true;
		
		
		// Iterate over unassigned positions and attempt to reassign them to vessels 
		while (insertionFound){
			
			// Reset insertion indicator and counter
			insertionFound = false;
			int i = 0;
			
			while (i < solution.unassignedPositions.size()){

				// Reset assignment for each unassigned position and 
				assignment.reset();
					
				for (int v = 0; v < solution.vessels.size(); v++){
				
					// If vessel schedule is non-empty, check where the position may be inserted
					if (!(solution.vessels.get(v).assignedPositions.isEmpty())){
						
					
						// Check if the voyage can be inserted in the first position
						if (solution.unassignedPositions.get(i).canBeAddedInSequence(solution.vessels.get(v), 0, solution)){
										
							double costOfAssignment = SailingBallastStart.getHighestStartCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailingBallastStarts)+
																				Sailing.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings) +
																				SailingBallast.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.vessels.get(v).assignedPositions.get(0).route, solution.problem.sailingBallasts) +
																				getSailingBallastStartPenaltyCost(solution.vessels.get(v), solution.unassignedPositions.get(i), solution);
						
							if (costOfAssignment < assignment.cost){
								assignment.cost = costOfAssignment;
								assignment.vessel = solution.vessels.get(v);
								assignment.position = 0;
							}
						}	
						
					
							
						// Check if the voyage can be inserted in between existing voyages in the list
						for (int j = 1; j < solution.vessels.get(v).assignedPositions.size(); j++){
							if (solution.unassignedPositions.get(i).canBeAddedInSequence(solution.vessels.get(v), j, solution)){
							
								double costOfAssignment = SailingBallast.getHighestCosts(solution.vessels.get(v).code, solution.vessels.get(v).assignedPositions.get(j-1).route, solution.unassignedPositions.get(i).route, solution.problem.sailingBallasts) +
																					Sailing.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings) +
																					SailingBallast.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.vessels.get(v).assignedPositions.get(j).route, solution.problem.sailingBallasts) +
																					getSailingBallastPenaltyCost(solution.vessels.get(v), solution.vessels.get(v).assignedPositions.get(j-1), solution.unassignedPositions.get(i), solution);
																					// Penalty cost for i-->j mangler
								
								if (costOfAssignment < assignment.cost){
									assignment.cost = costOfAssignment;
									assignment.vessel = solution.vessels.get(v);
									assignment.position = j;
								}
							}
						}
							
					
					
						// Check if the voyage can be inserted at the end of the assigned voyages list
						if (solution.unassignedPositions.get(i).canBeAddedInSequence(solution.vessels.get(v), solution.vessels.get(v).assignedPositions.size(), solution)){
								
							double costOfAssignment = SailingBallast.getHighestCosts(solution.vessels.get(v).code, solution.vessels.get(v).assignedPositions.get(solution.vessels.get(v).assignedPositions.size() - 1).route, solution.unassignedPositions.get(i).route, solution.problem.sailingBallasts)+
																				Sailing.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings)+
																				getSailingBallastPenaltyCost(solution.vessels.get(v), solution.vessels.get(v).assignedPositions.get(solution.vessels.get(v).assignedPositions.size() - 1), solution.unassignedPositions.get(i), solution);

							if (costOfAssignment < assignment.cost){
								assignment.cost = costOfAssignment;
								assignment.vessel = solution.vessels.get(v);
								assignment.position = solution.vessels.get(v).assignedPositions.size();
							}
						}	
				
				
							
						// If a vessel's schedule is empty, check if the voyage can be served
						}else{
							if (
									solution.problem.vesselCanSailInitialPosition(solution.vessels.get(v), solution.unassignedPositions.get(i))
									){
							
								double costOfAssignment = Sailing.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings) +
																					SailingBallastStart.getHighestStartCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailingBallastStarts) +
																					getSailingBallastStartPenaltyCost(solution.vessels.get(v), solution.unassignedPositions.get(i), solution);

								if (costOfAssignment < assignment.cost){
									assignment.cost = costOfAssignment;
									assignment.vessel = solution.vessels.get(v);
									assignment.position = 0;
								}
							}
						}
					} // End for all v

			
					// Insert unassigned position at the best insertion		
					if (assignment.vessel != null){	
			  	
						assignment.vessel.assignedPositions.add(assignment.position, solution.unassignedPositions.get(i));
						assignment.vessel.assignStartTimes(solution);
						solution.unassignedPositions.remove(i);
						insertionFound = true;
					
						//TEST
						/*
						System.out.println("INSERTED POSITION " + solution.unassignedPositions.get(i).id + " TO VESSEL " + assignment.vessel.code);
						assignment.vessel.printSchedule();
						 */
					}else{
						i++;
					}
				
				
						/*
						for (int v = 0; v < solution.vessels.size(); v++){
							if (solution.vessels.get(v).code.equals(assignment.vessel.code)){
								solution.vessels.get(v).assignedPositions.add(solution.unassignedPositions.get(i));
								solution.vessels.get(v).assignedPositions.get(solution.vessels.get(v).assignedPositions.size()-1).assignedStartTime = assignment.startTime;
						
						
								// Now, sort vessel schedule after a new voyage is included, with respect to assigned start times
								Collections.sort(solution.vessels.get(v).assignedPositions, Position.PositionAssignedStartTimeComparator);
	
							}
						}
						 */			
			
				} // End while unassigned positions have not been looped through yet
			
			
			// If no insertions were found, add all remaining positions to dummies and clear list
			if (!insertionFound){
				solution.dummyPositions.addAll(solution.unassignedPositions);
				solution.unassignedPositions.clear();
			}
			
		} // End while
		
	}
	
	
	
	
	
	/*
	 * ---------------------------- Regret-2 insertion repair operator ----------------------------
	 */
	
	public static void regret_k_Insertion(Solution solution, int k){
		

		// Initiate assignment objects
		Assignment bestAssignment = new Assignment();		
		Assignment[] cheapestAssignments = new Assignment[k];
		for (int q = 0; q < k ; q++){
			cheapestAssignments[q] = new Assignment();
		}
		
		
		// Set boolean variable which indicates that an insertion was found
		boolean insertionFound = true;
		
		// While there still are unassigned positions
		while (!(solution.unassignedPositions.isEmpty()) && (insertionFound)){

			// Reset best assignment and boolean indicator
			bestAssignment.reset();
			bestAssignment.cost = - GlobalConstants.costOfVesselCharter;
			insertionFound = false;

			
			// Iterate over unassigned positions and all vessels and find the k cheapest possible assignments
			for (int i = 0; i < solution.unassignedPositions.size(); i++){

				// Reset cheapest assignments of position i
				for (int q = 0; q < k ; q++){
					cheapestAssignments[q].reset();
				}
				
				
				for (int v = 0; v < solution.vessels.size(); v++){
				
					//TEST
					//System.out.println("COST OF CHEAPEST ASSIGNMENT FOR POSITION " + solution.unassignedPositions.get(i).id + " = " + cheapestAssignments[0].cost);
					
					// If vessel schedule is non-empty, check where the position may be inserted
					if (!(solution.vessels.get(v).assignedPositions.isEmpty())){
						
						// Check if the voyage can be inserted in the first position
						if (solution.unassignedPositions.get(i).canBeAddedInSequence(solution.vessels.get(v), 0, solution)){
								
							double costOfAssignment = SailingBallastStart.getHighestStartCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailingBallastStarts) +
																				Sailing.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings) +
																				SailingBallast.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.vessels.get(v).assignedPositions.get(0).route, solution.problem.sailingBallasts) +
																				getSailingBallastStartPenaltyCost(solution.vessels.get(v), solution.unassignedPositions.get(i), solution);
							
										
							// Check if this possible assignment is cheap enough to make the cheapest or second cheapest list
							isAssignmentCheapEnough(cheapestAssignments, costOfAssignment, 0, solution.unassignedPositions.get(i), solution.vessels.get(v));
							
						
						//TEST
							//System.out.println("INSERTION FOUND FOR POSITION " + solution.unassignedPositions.get(i).id);
						}	
						
							
						// Check if the voyage can be inserted in between existing voyages in the list
						for (int j = 1; j < solution.vessels.get(v).assignedPositions.size(); j++){
							if (solution.unassignedPositions.get(i).canBeAddedInSequence(solution.vessels.get(v), j, solution)){
								
								double costOfAssignment = SailingBallast.getHighestCosts(solution.vessels.get(v).code, solution.vessels.get(v).assignedPositions.get(j-1).route, solution.unassignedPositions.get(i).route, solution.problem.sailingBallasts) +
																					Sailing.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings) +
																					SailingBallast.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.vessels.get(v).assignedPositions.get(j).route, solution.problem.sailingBallasts) +
																					getSailingBallastPenaltyCost(solution.vessels.get(v), solution.vessels.get(v).assignedPositions.get(j-1), solution.unassignedPositions.get(i), solution);
										
								// Check if this assignment is cheap enough to make the cheapest or second cheapest
								isAssignmentCheapEnough(cheapestAssignments, costOfAssignment, j, solution.unassignedPositions.get(i), solution.vessels.get(v));
								
							//TEST
								//System.out.println("INSERTION FOUND FOR POSITION " + solution.unassignedPositions.get(i).id);
							}
						}
							
					
					
						// Check if the voyage can be inserted at the end of the assigned voyages list
						if (solution.unassignedPositions.get(i).canBeAddedInSequence(solution.vessels.get(v), solution.vessels.get(v).assignedPositions.size(), solution)){
							
							double costOfAssignment = SailingBallast.getHighestCosts(solution.vessels.get(v).code, solution.vessels.get(v).assignedPositions.get(solution.vessels.get(v).assignedPositions.size() - 1).route, solution.unassignedPositions.get(i).route, solution.problem.sailingBallasts)+
																				Sailing.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings)+
																				getSailingBallastPenaltyCost(solution.vessels.get(v), solution.vessels.get(v).assignedPositions.get(solution.vessels.get(v).assignedPositions.size() - 1), solution.unassignedPositions.get(i), solution);
										
							// Check if assignment is cheap enough to make the cheapest or second cheapest
							isAssignmentCheapEnough(cheapestAssignments, costOfAssignment, solution.vessels.get(v).assignedPositions.size(), solution.unassignedPositions.get(i), solution.vessels.get(v));
							
						//TEST
							//System.out.println("INSERTION FOUND FOR POSITION " + solution.unassignedPositions.get(i).id);
						}
				
							
					// If a vessel's schedule is empty, check if the voyage can be served
					}else{
						if (
								solution.problem.vesselCanSailInitialPosition(solution.vessels.get(v), solution.unassignedPositions.get(i))
								){
						
							double costOfAssignment = Sailing.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings) +
																				SailingBallastStart.getHighestStartCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailingBallastStarts) +
																				getSailingBallastStartPenaltyCost(solution.vessels.get(v), solution.unassignedPositions.get(i), solution);

							// Check if assignment is cheap enough to make the cheapest or second cheapest
							isAssignmentCheapEnough(cheapestAssignments, costOfAssignment, 0, solution.unassignedPositions.get(i), solution.vessels.get(v));
							
							
						//TEST
							//System.out.println("INSERTION FOUND FOR POSITION " + solution.unassignedPositions.get(i).id);
						}
					}
				} // End for all v
				
				
				// If one or more insertions are found for position i
				if (cheapestAssignments[0].vessel != null){

					// Evaluate if this position's alternative cost of not inserting in the best position is greater than any previous position
					bestAssignment.evaluateAgainst(cheapestAssignments);
					insertionFound = true;
					
					
					//TEST
					//System.out.println("INSERTION FOUND FOR POSITION: " + solution.unassignedPositions.get(i).id + " AT VESSEL " + cheapestAssignments[0].vessel.code);
					//System.out.println("BEST INSERTION YET: " + bestAssignment.positionAssigned.id + " AT VESSEL " + bestAssignment.vessel.code);
					
					// Move to the next unassigned position
					
					
									
				}
					
					//TEST
					//System.out.println("NO POSSIBLE ASSIGNMENTS FOUND FOR POSITION " + solution.unassignedPositions.get(i).id);				
					// No possible assignment was found for position i, so it can be assigned to a dummy variable and removed from the list of unassigned positions		
					
					//solution.dummyPositions.add(solution.unassignedPositions.get(i));
					//solution.unassignedPositions.remove(i);	
				
				
				
				/*
				if (cheapestAssignment.vessel != null){
					if (secondCheapestAssignment.cost - cheapestAssignment.cost > bestAssignment.cost){
							
						bestAssignment.cost = secondCheapestAssignment.cost - cheapestAssignment.cost;
						bestAssignment.positionAssigned = cheapestAssignment.positionAssigned;
						bestAssignment.position = cheapestAssignment.position;
						bestAssignment.vessel = cheapestAssignment.vessel;
							
					}		
				}else{
					
					//TEST
					//System.out.println("NO POSSIBLE ASSIGNMENTS FOUND FOR POSITION " + solution.unassignedPositions.get(i).id);
					
					// No possible assignment was found for position i, so it can be assigned to a dummy variable and removed from the list of unassigned positions		
					solution.dummyPositions.add(solution.unassignedPositions.get(i));
					solution.unassignedPositions.remove(i);
					i -= 1;
				}
						*/	
			} // End while !unassigned.isEmpty()
				
			//TEST	
			//System.out.println("INSERTION FOUND? = " + insertionFound);
				
			
			// After all positions have been iterated through, perform best assignment found
			if (insertionFound){
				bestAssignment.vessel.assignedPositions.add(bestAssignment.position, bestAssignment.positionAssigned);
				bestAssignment.vessel.assignStartTimes(solution);
				
				
				//TEST	
				//System.out.println("POSITION INSERTED: " + bestAssignment.positionAssigned.id);
				//System.out.println("REMAINING UNASSIGNED: " + solution.unassignedPositions.size());
				
				
				solution.unassignedPositions.remove(bestAssignment.positionAssigned);
				
				//System.out.println("REMAINING UNASSIGNED: " + solution.unassignedPositions.size());

							
				//TEST
				//System.out.println("POSITION " + solution.vessels.get(v).assignedPositions.get(solution.vessels.get(v).assignedPositions.size()-1).id + " ADDED TO VESSEL " + solution.vessels.get(v).code);
							
				// Now, sort vessel schedule after a new voyage is included, with respect to assigned start times
				//Collections.sort(bestAssignment.vessel.assignedPositions, Position.PositionAssignedStartTimeComparator);
				
			}else{
				// No insertion has been found when iterating through the list of remaining unassigned voyages. Add all to dummies and clear the list.
				solution.dummyPositions.addAll(solution.unassignedPositions);
				solution.unassignedPositions.clear();
				
			}
			
			//TEST
				/*
			System.out.println("TEST");
			System.out.println("SIZE OF UNASSIGNED: " + solution.unassignedPositions.size());
			System.exit(1);
			}
			*/
				
		} // End while
		
				
			
		
    //TESTING:
		/*
			System.out.println("AFTER REPAIR: ");
			System.out.println("LIST OF UNASSIGNED POSITIONS: ");
			for (int i = 0; i < solution.unassignedPositions.size(); i++){
				System.out.println("" + solution.unassignedPositions.get(i).id);
			}
			
			for (int v = 0; v < solution.vessels.size(); v++){
				System.out.println("VESSEL " + solution.vessels.get(v).code + " HAS THE FOLLOWING SCHEDULE:");
				for (int o = 0; o < solution.vessels.get(v).assignedPositions.size(); o++){
					System.out.println(solution.vessels.get(v).assignedPositions.get(o).id + " STARTS AT " + solution.vessels.get(v).assignedPositions.get(o).assignedStartTime);
				}
			}
			System.out.println("LIST OF DUMMY POSITIONS: ");
			for (int i = 0; i < solution.dummyPositions.size(); i++){
				System.out.println("" + solution.dummyPositions.get(i).id);
			}
			*/
	}
	
	
	

	/*
	 * ---------------------------- Deep greedy insertion repair operator ----------------------------
	 */
	
	public static void deepGreedyInsertion(Solution solution){
		

		// Initialize assignment object
		Assignment bestAssignment = new Assignment();
		boolean insertionFound = true;
		
		
		// While there still are unassigned positions
		while (!(solution.unassignedPositions.isEmpty()) && insertionFound){

			// Reset best assignment and boolean indicator of found insertion
			bestAssignment.reset();
			insertionFound = false;
			
			
			// Iterate over unassigned positions and all vessels and find the cheapest possible assignment
			for (int i = 0; i < solution.unassignedPositions.size(); i++){	
				for (int v = 0; v < solution.vessels.size(); v++){
					
					
					// If vessel schedule is non-empty, check where the position may be inserted
					if (!(solution.vessels.get(v).assignedPositions.isEmpty())){
						
						// Check if the voyage can be inserted in the first position
						if (solution.unassignedPositions.get(i).canBeAddedInSequence(solution.vessels.get(v), 0, solution)){
										
							double costOfAssignment = SailingBallastStart.getHighestStartCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailingBallastStarts)+
																				Sailing.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings) +
																				SailingBallast.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.vessels.get(v).assignedPositions.get(0).route, solution.problem.sailingBallasts) +
																				getSailingBallastStartPenaltyCost(solution.vessels.get(v), solution.unassignedPositions.get(i), solution);
								
							insertionFound = true;
										
								
							// Check if this possible assignment is cheaper than the previous best assignment
							if (costOfAssignment < bestAssignment.cost){
									
								bestAssignment.cost = costOfAssignment;
								bestAssignment.positionAssigned = solution.unassignedPositions.get(i);
								bestAssignment.position = 0;
								bestAssignment.vessel = solution.vessels.get(v);
									
							}
						}	
						
							
						// Check if the voyage can be inserted in between existing voyages in the list
						for (int j = 1; j < solution.vessels.get(v).assignedPositions.size(); j++){
							if (solution.unassignedPositions.get(i).canBeAddedInSequence(solution.vessels.get(v), j, solution)){
								
								double costOfAssignment = SailingBallast.getHighestCosts(solution.vessels.get(v).code, solution.vessels.get(v).assignedPositions.get(j-1).route, solution.unassignedPositions.get(i).route, solution.problem.sailingBallasts)+
																					Sailing.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings) +
																					SailingBallast.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.vessels.get(v).assignedPositions.get(j).route, solution.problem.sailingBallasts)+
																					getSailingBallastPenaltyCost(solution.vessels.get(v), solution.vessels.get(v).assignedPositions.get(j-1), solution.unassignedPositions.get(i), solution);
										
								insertionFound = true;

										
								// Check if this assignment is cheap enough to make the cheapest or second cheapest
								if (costOfAssignment < bestAssignment.cost){
											
									bestAssignment.cost = costOfAssignment;
									bestAssignment.positionAssigned = solution.unassignedPositions.get(i);
									bestAssignment.position = j;
									bestAssignment.vessel = solution.vessels.get(v);

								}
							}
						}
							
					
						// Check if the voyage can be inserted at the end of the assigned voyages list
						if (solution.unassignedPositions.get(i).canBeAddedInSequence(solution.vessels.get(v), solution.vessels.get(v).assignedPositions.size(), solution)){
								
							double costOfAssignment = SailingBallast.getHighestCosts(solution.vessels.get(v).code, solution.vessels.get(v).assignedPositions.get(solution.vessels.get(v).assignedPositions.size() - 1).route, solution.unassignedPositions.get(i).route, solution.problem.sailingBallasts)+
																				Sailing.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings)+
																				getSailingBallastPenaltyCost(solution.vessels.get(v), solution.vessels.get(v).assignedPositions.get(solution.vessels.get(v).assignedPositions.size() - 1), solution.unassignedPositions.get(i), solution);
										
							insertionFound = true;

								
							// Check if assignment is cheap enough to make the cheapest or second cheapest
							if (costOfAssignment < bestAssignment.cost){
									
								bestAssignment.cost = costOfAssignment;
								bestAssignment.positionAssigned = solution.unassignedPositions.get(i);
								bestAssignment.position = solution.vessels.get(v).assignedPositions.size();
								bestAssignment.vessel = solution.vessels.get(v);
									
							}
						}
				
							 
					// Else, if a vessel's schedule is empty, check if the voyage can be served
					}else{
						if (
								solution.problem.vesselCanSailInitialPosition(solution.vessels.get(v), solution.unassignedPositions.get(i))
								){
						
							double costOfAssignment = Sailing.getHighestCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings) +
																				SailingBallastStart.getHighestStartCosts(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailingBallastStarts) +
																				getSailingBallastStartPenaltyCost(solution.vessels.get(v), solution.unassignedPositions.get(i), solution);
							
							insertionFound = true;

							
							// Check if assignment is cheap enough to make the cheapest or second cheapest
							if (costOfAssignment < bestAssignment.cost){
								
								bestAssignment.cost = costOfAssignment;
								bestAssignment.positionAssigned = solution.unassignedPositions.get(i);
								bestAssignment.position = 0;
								bestAssignment.vessel = solution.vessels.get(v);
								
							}
						}
					}
				} // End for all v			
			} // End for all i
				
				
			// After iterating all positions i and all vessel v, perform the cheapest found insertion
			if (insertionFound){
				
				bestAssignment.vessel.assignedPositions.add(bestAssignment.position, bestAssignment.positionAssigned);
				bestAssignment.vessel.assignStartTimes(solution);
				solution.unassignedPositions.remove(bestAssignment.positionAssigned);
				
			}else{
					
				// No possible assignment was found for any unassigned positions
				solution.dummyPositions.addAll(solution.unassignedPositions);
				solution.unassignedPositions.clear();
			}
				
		} // End while
			
		
		
    //TESTING:
		/*
			System.out.println("AFTER REPAIR: ");
			System.out.println("LIST OF UNASSIGNED POSITIONS: ");
			for (int i = 0; i < solution.unassignedPositions.size(); i++){
				System.out.println("" + solution.unassignedPositions.get(i).id);
			}
			
			for (int v = 0; v < solution.vessels.size(); v++){
				System.out.println("VESSEL " + solution.vessels.get(v).code + " HAS THE FOLLOWING SCHEDULE:");
				for (int o = 0; o < solution.vessels.get(v).assignedPositions.size(); o++){
					System.out.println(solution.vessels.get(v).assignedPositions.get(o).id + " STARTS AT " + solution.vessels.get(v).assignedPositions.get(o).assignedStartTime);
				}
			}
			System.out.println("LIST OF DUMMY POSITIONS: ");
			for (int i = 0; i < solution.dummyPositions.size(); i++){
				System.out.println("" + solution.dummyPositions.get(i).id);
			}
			*/
	}
	
	
	
	
	/*
	 * -------------------------------- MIP repair operator -----------------------------------
	 */
	
	public static boolean callMipSolver(Solution solution, Mip mip){
		try{
			
			// Repair Solution object. The Solution object argument has several destroyed voyages associated with it.
			
			
			// Reset model
			mip.model.reset();
			
			
			// First, reset all bounds
			for (int i = 0; i < mip.problem.positions.size(); i++){
				mip.model.getVarByName("dummy[" + mip.problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB, 0.0);
				mip.model.getVarByName("dummy[" + mip.problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB, 1.0);
				
				for (int v = 0; v < mip.problem.vessels.size(); v++){
					if (mip.problem.vesselCanSailPosition(mip.problem.vessels.get(v), mip.problem.positions.get(i))){
						mip.model.getVarByName("y_i[" + mip.problem.vessels.get(v).code + "][" + mip.problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB, 0.0);
						mip.model.getVarByName("y_i[" + mip.problem.vessels.get(v).code + "][" + mip.problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB, 1.0);
						
						//mip.model.getVarByName("t[" + mip.problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB, mip.problem.positions.get(i).startTime);
						//mip.model.getVarByName("t[" + mip.problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB, Position.getLatestVoyageStart(mip.problem.positions) + GlobalConstants.maxDaysDelayed);
					}
				}
			}
	
			
			// Fix variables from Solution object
			for (int i = 0; i < solution.dummyPositions.size(); i++){
				mip.model.getVarByName("dummy[" + solution.dummyPositions.get(i).id + "]").set(GRB.DoubleAttr.LB, 1.0);
				mip.model.getVarByName("dummy[" + solution.dummyPositions.get(i).id + "]").set(GRB.DoubleAttr.UB, 1.0);
			}
			
			for (int v = 0; v < solution.vessels.size(); v++){
				for (int i = 0; i < solution.vessels.get(v).assignedPositions.size(); i++){
					//if (mip.problem.vesselCanSailPosition(solution.vessels.get(v), solution.vessels.get(v).assignedPositions.get(i))){
						mip.model.getVarByName("y_i[" + solution.vessels.get(v).code + "][" + solution.vessels.get(v).assignedPositions.get(i).id + "]").set(GRB.DoubleAttr.LB, 1.0);
						mip.model.getVarByName("y_i[" + solution.vessels.get(v).code + "][" + solution.vessels.get(v).assignedPositions.get(i).id + "]").set(GRB.DoubleAttr.UB, 1.0);
						
						//mip.model.getVarByName("t[" + solution.vessels.get(v).assignedPositions.get(i).id + "]").set(GRB.DoubleAttr.LB, solution.vessels.get(v).assignedPositions.get(i).assignedStartTime);
						//mip.model.getVarByName("t[" + solution.vessels.get(v).assignedPositions.get(i).id + "]").set(GRB.DoubleAttr.UB, solution.vessels.get(v).assignedPositions.get(i).assignedStartTime);


					//}	
				}
			}
			
			
			// Now, the unassigned positions will be allocated when calling optimize.
			
			// Reset time, solution and MIP-gap limit from when the initial solution was found
			mip.model.getEnv().set(GRB.DoubleParam.TimeLimit, 30);
			mip.model.getEnv().set(GRB.DoubleParam.MIPGap, 0.001);
			mip.model.getEnv().set(GRB.IntParam.SolutionLimit, 1000);
			mip.model.getEnv().set(GRB.IntParam.OutputFlag, 0);
			
			// Call optimize on the fixed problem
			mip.model.optimize();
			
			
			// Remove unassigned positions from Solution object
			solution.unassignedPositions.clear();

			
			// For debugging purposes:
      int optimstatus = mip.model.get(GRB.IntAttr.Status);
			if (optimstatus == GRB.Status.INFEASIBLE){
				mip.model.computeIIS();
				mip.model.write("mymodel.ilp");
			}
			
			// Return possible
			if (mip.model.get(GRB.IntAttr.SolCount) > 0){
				return true;
			}else{
				return false;
			}
			
			
		}catch(GRBException e) {
      System.out.println("Error code: " + e.getErrorCode() + ". " +
          e.getMessage());
      e.printStackTrace();
		}
		
		return false;
	}
	
	
	
	/*
	 * ------------------------------- Reassign local search ---------------------------------
	 */
	
	public static void localSearch(Solution solution){
		
		//TEST
		System.out.println("NUMBER OF DUMMIES: " + solution.dummyPositions.size());
		System.out.println("NUMBER OF UNASSIGNED: " + solution.unassignedPositions.size());
		
		
		// Set all dummy positions as unassigned
		solution.unassignedPositions.addAll(solution.dummyPositions);
		solution.dummyPositions.clear();
		
		
		// Choose a local search operator
		
		// 1 first possible insertion 
		greedyInsertion(solution);

		// 2 reassign
		
		
		// 3 1-interchange
		
		
		System.out.println("NUMBER OF DUMMIES: " + solution.dummyPositions.size());
		System.out.println("NUMBER OF UNASSIGNED: " + solution.unassignedPositions.size());
		
		//TEST
		//boolean success = false;
		
		
		//boolean insertionPerformed = false;
		
		/*
		// Attempt to place any remaining unassigned positions in the vessel schedules
		while (!solution.unassignedPositions.isEmpty()){
			
			//TEST
			System.out.println("TRYING TO FIND A POSSIBLE INSERTION FOR [" + solution.unassignedPositions.get(0).id + "]");
			
			insertionPerformed = false;
			
			// Iterate all vessels
			int v = 0;
			while (v < solution.vessels.size() && !insertionPerformed){
				if (solution.unassignedPositions.get(0).canBeAssignedTo(solution.vessels.get(v), solution)){	
					
					// Position has been assigned to vessel v. Remove from dummy positions.
					insertionPerformed = true;
					solution.unassignedPositions.remove(0);
						
					// Assign new start times for the positions in vessel v's schedule
					solution.vessels.get(v).assignStartTimes(solution);
					
					
					//TEST:
					System.out.println("THE START TIMES FOR VESSEL [" + solution.vessels.get(v).code + "]'s POSITIONS ARE NOW:");
					for (int i = 0; i < solution.vessels.get(v).assignedPositions.size(); i++){
						System.out.println("[" + solution.vessels.get(v).assignedPositions.get(i).id + "] = " + solution.vessels.get(v).assignedPositions.get(i).assignedStartTime);
					}
					success = true;
					
				}
				
				v++;
			} // End while2
			
			*/
		/*
			//TEST
			System.out.println("INSERTION?" + insertionPerformed);
			
			// If no insertion was found for this unassigned position, assign it back to a dummy variable
			if (!insertionPerformed){
				solution.dummyPositions.add(solution.unassignedPositions.get(0));
				
				//TEST
				System.out.println("POSITION [" + solution.unassignedPositions.get(0).id + "] is added to a dummy");
				
				solution.unassignedPositions.remove(0);
				
				//TEST
				System.out.println("REMAINING UNASSIGNED POSITIONS: " + solution.unassignedPositions.size());
			}
						
		} // End while1
		

		System.out.println("NUMBER OF DUMMIES: " + solution.dummyPositions.size());
		System.out.println("NUMBER OF UNASSIGNED: " + solution.unassignedPositions.size());
		
		/*
		if (success){
			System.exit(1);

		}
		*/
		
		
		
	}
	
	
	/*
	 * -------------------------------- Transfer solution to model instance -----------------------------------
	 */
	
	public static boolean solutionToModel(Solution solution, Mip mip){
		try{
			// Build up model from Solution object
		
			
			// Reset model
			mip.model.reset();
			
			long startTime = System.currentTimeMillis();
			long elapsedTime = 0L;
			
			// First, reset all bounds
			
			for (int i = 0; i < mip.problem.positions.size(); i++){
				mip.model.getVarByName("dummy[" + mip.problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB, 0.0);
				mip.model.getVarByName("dummy[" + mip.problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB, 1.0);
				
				for (int v = 0; v < mip.problem.vessels.size(); v++){
					if (mip.problem.vesselCanSailPosition(mip.problem.vessels.get(v), mip.problem.positions.get(i))){
						mip.model.getVarByName("y_i[" + mip.problem.vessels.get(v).code + "][" + mip.problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB, 0.0);
						mip.model.getVarByName("y_i[" + mip.problem.vessels.get(v).code + "][" + mip.problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB, 1.0);
					}
				}
			}

			
			
			// Build up model from Solution object	
			for (int v = 0; v < solution.vessels.size(); v++){
				for (int i = 0; i < solution.vessels.get(v).assignedPositions.size(); i++){
					if (mip.problem.vesselCanSailPosition(solution.vessels.get(v), solution.vessels.get(v).assignedPositions.get(i))){
							//System.out.println("Position " + solution.vessels.get(v).assignedPositions.get(i).id + " is covered by vessel " + solution.vessels.get(v).code);
							mip.model.getVarByName("y_i[" + solution.vessels.get(v).code + "][" + solution.vessels.get(v).assignedPositions.get(i).id + "]").set(GRB.DoubleAttr.LB, 1.0);
							mip.model.getVarByName("y_i[" + solution.vessels.get(v).code + "][" + solution.vessels.get(v).assignedPositions.get(i).id + "]").set(GRB.DoubleAttr.UB, 1.0);
					}	
				}
			}

			
			for (int i = 0; i < solution.dummyPositions.size(); i++){
				mip.model.getVarByName("dummy[" + solution.dummyPositions.get(i).id + "]").set(GRB.DoubleAttr.LB, 1.0);
				mip.model.getVarByName("dummy[" + solution.dummyPositions.get(i).id + "]").set(GRB.DoubleAttr.UB, 1.0);
				//System.out.println("Position " + solution.dummyPositions.get(i).id + " is covered by dummy.");
			}
			
			elapsedTime = (new Date()).getTime() - startTime;
			System.out.println("TIME OF REBUILDING MODEL: " + (elapsedTime) + " MILLISECONDS");

			
			// Reset time, solution and MIP-gap limit from when the initial solution was found
			mip.model.getEnv().set(GRB.DoubleParam.TimeLimit, 30);
			//mip.model.getEnv().set(GRB.DoubleParam.MIPGap, 0.01);
			//mip.model.getEnv().set(GRB.IntParam.SolutionLimit, 1000);
						
			// Call optimize on the fixed problem
			mip.model.optimize();
			
			// For debugging purposes:
      int optimstatus = mip.model.get(GRB.IntAttr.Status);
			if (optimstatus == GRB.Status.INFEASIBLE){
				System.out.println("INFEASIBLE");
				mip.model.computeIIS();
				mip.model.write("mymodel.ilp");
			}
			
			elapsedTime = (new Date()).getTime() - startTime;
			System.out.println("TIME OF SOLVING MODEL: " + (elapsedTime) + " MILLISECONDS");

						
			// Return true if possible solution was found
			if (mip.model.get(GRB.IntAttr.SolCount) > 0){
				return true;
			}else{
				return false;
			}

		}catch(GRBException e) {
      System.out.println("Error code: " + e.getErrorCode() + ". " +
          e.getMessage());
      e.printStackTrace();
		}
		return false;
	}
	
	
	
	/*
	 * ---------------- Method to check if a start ballast sailing to an unassigned position causes delay ----------------
	 */
	
	public static double getSailingBallastStartPenaltyCost(Vessel v, Position unassigned, Solution solution){
		if (
				v.dateOffset +
				SailingBallastStart.getShortestSailingBallastStartDuration(v.code, unassigned.route, solution.problem.sailingBallastStarts)
				<=
				unassigned.endTime){
			return 0;
		}else{
			double daysDelayed = v.dateOffset + SailingBallastStart.getShortestSailingBallastStartDuration(v.code, unassigned.route, solution.problem.sailingBallastStarts) - unassigned.endTime;
			return (daysDelayed * GlobalConstants.penaltyCostPerDayDelayed);
		}
	}
	
	
	/*
	 * ---------------- Method to check if sailing from an assigned position to an unassigned position causes delay ----------------
	 */
	
	public static double getSailingBallastPenaltyCost(Vessel v, Position assigned, Position unassigned, Solution solution){
		if (
				assigned.assignedStartTime +
				Sailing.getShortestSailingDuration(v.code, assigned.route, solution.problem.sailings) +
				SailingBallast.getShortestSailingBallastDuration(v.code, assigned.route, unassigned.route, solution.problem.sailingBallasts)
				<=
				unassigned.endTime){
			return 0;
		}else{
			double daysDelayed = assigned.assignedStartTime + Sailing.getShortestSailingDuration(v.code, assigned.route, solution.problem.sailings) + SailingBallast.getShortestSailingBallastDuration(v.code, assigned.route, unassigned.route, solution.problem.sailingBallasts) - unassigned.endTime;
			return (daysDelayed * GlobalConstants.penaltyCostPerDayDelayed);
		}
	}
	
	
	
		/*
		 * ---------------- Method to calculate earliest possible start time for a voyage insertion as first voyage ----------------
		 */
	
	public static double getStartTimeInitialPosition(Vessel v, Position insertedPosition, Solution solution){
		if (v.dateOffset +
				SailingBallastStart.getShortestSailingBallastStartDuration(v.code, insertedPosition.route, solution.problem.sailingBallastStarts)
				<=
				insertedPosition.startTime){
			return insertedPosition.startTime;
		}else{
			return (v.dateOffset +
							SailingBallastStart.getShortestSailingBallastStartDuration(v.code, insertedPosition.route, solution.problem.sailingBallastStarts));
		}	
	}
	
	
	/*
	 * ---------------- Method to calculate earliest possible start time for a voyage insertion after a voyage ----------------
	 */
	
	public static double getStartTimeAfterPosition(Vessel v, Position existingPosition, Position insertedPosition, Solution solution){
		if (existingPosition.assignedStartTime +
				Sailing.getShortestSailingDuration(v.code, existingPosition.route, solution.problem.sailings) +
				SailingBallast.getShortestSailingBallastDuration(v.code, existingPosition.route, insertedPosition.route, solution.problem.sailingBallasts)
				<=
				insertedPosition.startTime){
			return 
				insertedPosition.startTime;
		}else{
			return 
				(existingPosition.assignedStartTime +
				Sailing.getShortestSailingDuration(v.code, existingPosition.route, solution.problem.sailings) +
				SailingBallast.getShortestSailingBallastDuration(v.code, existingPosition.route, insertedPosition.route, solution.problem.sailingBallasts));
		}	
	}
	
	
	
	/* 
	 * ------------------------- Method to evaluate if an initial assignment is cheap enough to be saved -------------------------
	 */
	
	public static void isAssignmentCheapEnough(Assignment[] cheapestAssignments, double costOfAssignment, int sequence, Position i, Vessel v){
		
		boolean assignmentsUpdated = false;
		int k = 0;
		
		
		// Iterate through list of assignments, and update if this cost of insertion is cheaper than insertions previously found
		while (k < cheapestAssignments.length && !assignmentsUpdated){
			
			if (costOfAssignment < cheapestAssignments[k].cost){
				
				// Update any other more expensive assignments previously found
				for (int q = cheapestAssignments.length - 1; q > k; q--){
					cheapestAssignments[q] = cheapestAssignments[q-1];	
				}
							
				// Update assignment
				cheapestAssignments[k].cost = costOfAssignment;
				cheapestAssignments[k].position = sequence;
				cheapestAssignments[k].vessel = v;
				cheapestAssignments[k].positionAssigned = i;
				
				// Set assignments as updated
				assignmentsUpdated = true;
			}else{
				
				k++;
			}
		} // End while
		
		
		/*
		if (costOfAssignment < secondCheapestAssignment.cost){
			if (costOfAssignment < cheapestAssignment.cost){
				
				// Update both second cheapest and cheapest assignment
				secondCheapestAssignment.cost = cheapestAssignment.cost;
				secondCheapestAssignment.vessel = cheapestAssignment.vessel;
				secondCheapestAssignment.position = cheapestAssignment.position;
				secondCheapestAssignment.positionAssigned = cheapestAssignment.positionAssigned;
				
				cheapestAssignment.cost = costOfAssignment;
				cheapestAssignment.vessel = solution.vessels.get(v);
				cheapestAssignment.position = 0;
				cheapestAssignment.positionAssigned = solution.unassignedPositions.get(i);
			}
			else{
				
				// Update only second cheapest assignment
				secondCheapestAssignment.cost = costOfAssignment;
				secondCheapestAssignment.vessel = solution.vessels.get(v);
				secondCheapestAssignment.position = 0;
				secondCheapestAssignment.positionAssigned = solution.unassignedPositions.get(i);
			}
		}
		*/
	}
	
	
	
	
	
	/* 
	 * ------------------------- Method to evaluate if an intermediate assignment is cheap enough to be saved -------------------------
	 */
	
	public static void isInBetweenAssignmentCheapEnough(Assignment cheapestAssignment, Assignment secondCheapestAssignment, double costOfAssignment, Solution solution, int v, int i, int j){
		
		if (costOfAssignment < secondCheapestAssignment.cost){
			if (costOfAssignment < cheapestAssignment.cost){
				
				// Update both second cheapest and cheapest assignment
				secondCheapestAssignment.cost = cheapestAssignment.cost;
				secondCheapestAssignment.vessel = cheapestAssignment.vessel;
				secondCheapestAssignment.position = cheapestAssignment.position;
				secondCheapestAssignment.positionAssigned = cheapestAssignment.positionAssigned;
				
				cheapestAssignment.cost = costOfAssignment;
				cheapestAssignment.vessel = solution.vessels.get(v);
				cheapestAssignment.position = j;
				cheapestAssignment.positionAssigned = solution.unassignedPositions.get(i);
			}
			else{
				
				// Update only second cheapest assignment
				secondCheapestAssignment.cost = costOfAssignment;
				secondCheapestAssignment.vessel = solution.vessels.get(v);
				secondCheapestAssignment.position = j;
				secondCheapestAssignment.positionAssigned = solution.unassignedPositions.get(i);
			}
		}
	}
	
	
	
	/*
	 * ------------------------- Method to evaluate if inserting a position as the first position in a schedule is possible -----------
	 */
	
	public static boolean initialInsertionPossible(Solution solution, int v, int i){
		
		if (
				solution.problem.vesselCanSailInitialPosition(solution.vessels.get(v), solution.unassignedPositions.get(i))
				
				&&
				
				solution.problem.vesselCanSailPositionToPosition(solution.vessels.get(v), solution.unassignedPositions.get(i), solution.vessels.get(v).assignedPositions.get(0))
		
				&&
		
				solution.vessels.get(v).dateOffset +
				SailingBallastStart.getShortestSailingBallastStartDuration(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailingBallastStarts) +
				Sailing.getShortestSailingDuration(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings) +
				SailingBallast.getShortestSailingBallastDuration(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.vessels.get(v).assignedPositions.get(0).route, solution.problem.sailingBallasts)
				<=
				solution.vessels.get(v).assignedPositions.get(0).assignedStartTime
				
				&&
				
				solution.unassignedPositions.get(i).startTime +
				Sailing.getShortestSailingDuration(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings) +
				SailingBallast.getShortestSailingBallastDuration(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.vessels.get(v).assignedPositions.get(0).route, solution.problem.sailingBallasts)
				<=
				solution.vessels.get(v).assignedPositions.get(0).assignedStartTime
				){
			return true;
		}else{
			return false;
		}
	}
	
	
	
	/*
	 * ------------------------- Method to evaluate if inserting a position in between positions in a schedule is possible -----------
	 */
	
	public static boolean inBetweenInsertionPossible(Solution solution, int v, int j, int i){
		
		if (
				solution.problem.vesselCanSailPositionToPosition(solution.vessels.get(v), solution.vessels.get(v).assignedPositions.get(j), solution.unassignedPositions.get(i))
				
				&&
				
				solution.problem.vesselCanSailPositionToPosition(solution.vessels.get(v), solution.unassignedPositions.get(i), solution.vessels.get(v).assignedPositions.get(j+1))
					
				&&
				
				solution.vessels.get(v).assignedPositions.get(j).assignedStartTime +
				Sailing.getShortestSailingDuration(solution.vessels.get(v).code, solution.vessels.get(v).assignedPositions.get(j).route, solution.problem.sailings) +
				SailingBallast.getShortestSailingBallastDuration(solution.vessels.get(v).code, solution.vessels.get(v).assignedPositions.get(j).route, solution.unassignedPositions.get(i).route, solution.problem.sailingBallasts)
				<= 
				solution.unassignedPositions.get(i).endTime + GlobalConstants.maxDaysDelayed

				&&

				solution.vessels.get(v).assignedPositions.get(j).assignedStartTime +
				Sailing.getShortestSailingDuration(solution.vessels.get(v).code, solution.vessels.get(v).assignedPositions.get(j).route, solution.problem.sailings) +
				SailingBallast.getShortestSailingBallastDuration(solution.vessels.get(v).code, solution.vessels.get(v).assignedPositions.get(j).route, solution.unassignedPositions.get(i).route, solution.problem.sailingBallasts) +
				Sailing.getShortestSailingDuration(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings) +
				SailingBallast.getShortestSailingBallastDuration(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.vessels.get(v).assignedPositions.get(j+1).route, solution.problem.sailingBallasts)
				<=
				solution.vessels.get(v).assignedPositions.get(j+1).assignedStartTime
				
				&&
				
				solution.unassignedPositions.get(i).startTime +
				Sailing.getShortestSailingDuration(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.problem.sailings) +
				SailingBallast.getShortestSailingBallastDuration(solution.vessels.get(v).code, solution.unassignedPositions.get(i).route, solution.vessels.get(v).assignedPositions.get(j+1).route, solution.problem.sailingBallasts)
				<=
				solution.vessels.get(v).assignedPositions.get(j+1).assignedStartTime
				){
			return true;
		}else{
			return false;
		}
	}

	
	
	/*
	 * ------------------------- Method to evaluate if inserting a position as the last position in a schedule is possible -----------
	 */
	
	public static boolean lastInsertionPossible(Solution solution, int v, int i){
		
		if (
				solution.problem.vesselCanSailPositionToPosition(solution.vessels.get(v), solution.vessels.get(v).assignedPositions.get(solution.vessels.get(v).assignedPositions.size()-1), solution.unassignedPositions.get(i))
				
				&&
				
				solution.vessels.get(v).assignedPositions.get(solution.vessels.get(v).assignedPositions.size() - 1).assignedStartTime +
				Sailing.getShortestSailingDuration(solution.vessels.get(v).code, solution.vessels.get(v).assignedPositions.get(solution.vessels.get(v).assignedPositions.size() - 1).route, solution.problem.sailings) +
				SailingBallast.getShortestSailingBallastDuration(solution.vessels.get(v).code, solution.vessels.get(v).assignedPositions.get(solution.vessels.get(v).assignedPositions.size() - 1).route, solution.unassignedPositions.get(i).route, solution.problem.sailingBallasts)
				<= 
				solution.unassignedPositions.get(i).endTime + GlobalConstants.maxDaysDelayed
				){
			return true;
		}else{
			return false;
		}
	}

	
	

}

