import gurobi.*;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

/*
 * This class contains the evaluate operator.
 */
public class EvaluateOperator {
	double bestBound;
	double bestObjectiveValue;
	GRBModel modelCopy;
	GRBModel bestSolution;
	
	public EvaluateOperator(double bestBound, double bestObjectiveValue) {
		this.bestBound = bestBound;
		this.bestObjectiveValue = bestObjectiveValue;
	}
	

	public void evaluate(Mip mip, int iter){
		try{
			
			// Initialize modelCopy
			if (iter == 0){
				this.modelCopy = mip.model;
			}
			
			// Check if a solution was found
			if (mip.model.get(GRB.IntAttr.SolCount) > 0){
				
				double bound = mip.model.get(GRB.DoubleAttr.ObjBound);
				double objectiveValue = mip.model.get(GRB.DoubleAttr.ObjVal);
				
				// Check if the solution found has a better bound
				if (bound < this.bestBound){
					this.bestBound = bound;
				}
				
				// Check if the solution found has a better objective value
				if (objectiveValue < this.bestObjectiveValue){
					System.out.println("");
					System.out.println("This iteration improved the solution by " + (bestObjectiveValue - objectiveValue) + ".");
					this.bestObjectiveValue = objectiveValue;
					this.bestSolution = mip.model;

				}
				// If not, return to previous solution
				else{
					mip.model = this.modelCopy;
				}
				
				// Copy solution
				this.modelCopy = mip.model;
				
			// If no solution was found within the time limit, relax dummies and go back to last model copy
			}else{
				for (int i = 0; i < mip.problem.positions.size(); i++){
					mip.model.getVarByName("dummy[" + mip.problem.positions.get(i).id + "]").set(GRB.DoubleAttr.LB, 0.0);
					mip.model.getVarByName("dummy[" + mip.problem.positions.get(i).id + "]").set(GRB.DoubleAttr.UB, 1.0);
					mip.model = this.modelCopy;
				}
			}
			

			}catch(GRBException e) {
				System.out.println("Error code: " + e.getErrorCode() + ". " +
          e.getMessage());
			}
	}
}
