import gurobi.GRB;
import gurobi.GRBEnv;
import gurobi.GRBException;
import gurobi.GRBLinExpr;
import gurobi.GRBModel;
import gurobi.GRBVar;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

public class LocalSearch {

	
	
	
	public static void localSearch(Solution solution, Iteration currentIteration){
			
		
		//TEST
		//System.out.println("NUMBER OF DUMMIES: " + solution.dummyPositions.size());
		//System.out.println("NUMBER OF UNASSIGNED: " + solution.unassignedPositions.size());
			
			
		// Set all dummy positions as unassigned
		solution.unassignedPositions.addAll(solution.dummyPositions);
		solution.dummyPositions.clear();
			
		
		// Choose a local search operator
		if (currentIteration.localSearch == 1){
			insertUnassigned(solution);
		}
	
		// 2 reassign
		else if (currentIteration.localSearch == 2){
			reassign(solution);
		}
			
		// 3 Swap
		else if (currentIteration.localSearch == 3){
			swap(solution);
		}
			
		
		//TEST
		//System.out.println("NUMBER OF DUMMIES: " + solution.dummyPositions.size());
		//System.out.println("NUMBER OF UNASSIGNED: " + solution.unassignedPositions.size());
		
		
	}
	
	
	
	
	/*
	 * -------------- Method that swaps voyages between vessel in order to make room for remaining positions ------------------
	 */
	
	public static void swap(Solution solution){
		
		boolean improvingSwap = false;
		int v = 0;
		int i = 0;
		ArrayList<Position> possibleSwaps = new ArrayList<Position>();
		Position j;
		
		
		while (!solution.unassignedPositions.isEmpty()){
			improvingSwap = false;
			possibleSwaps.clear();

			//TEST
			//System.out.println("-------------------------------");
			//System.out.println("SEARCHING FOR POSSIBLE SWAPS TO MAKE ROOM FOR " + solution.unassignedPositions.get(0).id);
			
			
			// Search for possible swaps of voyages between vessels. If a swap is possible, perform it and see if it makes room for another unserviced voyage.
			
			v = 0;		
			while (v < solution.vessels.size() && !improvingSwap){
				
				i = 0;
				while (i < solution.vessels.get(v).assignedPositions.size() && !improvingSwap){
					
					if (solution.vessels.get(v).assignedPositions.get(i).swapsArePossible(solution, solution.vessels.get(v), possibleSwaps, i)){
						
						j = solution.vessels.get(v).assignedPositions.get(i);
						
						// TEST
						//System.out.println("NUMBER OF SWAPS FOUND: " + possibleSwaps.size());
						
						// A set of possible swaps have been found
						while (!possibleSwaps.isEmpty() && !improvingSwap){
							
							//TEST
							//System.out.println("TESTING NEW SWAP:");
							
							// Perform swaps
							solution.swap(possibleSwaps.get(0), j);
							
							// Check if the swap makes room for another position currently not covered
							if (solution.possibleInsertionFoundFor(solution.unassignedPositions.get(0))){
								
								//TEST
								//System.out.println("SWAP MADE ROOM FOR POSITION " + solution.unassignedPositions.get(0).id);
								//System.exit(1);
								
								
								// Mark swap as improving and remove position from unassigned list
								improvingSwap = true;
								solution.unassignedPositions.remove(0);			
		
							}else{
								
								//TEST
								//System.out.println("SWAP HAD NO EFFECT ON SOLUTION QUALITY");
								
								// Revert and remove possible position to be swapped with. 
								solution.swap(possibleSwaps.get(0), j);
								//solution.swap(solution.vessels.get(v).assignedPositions.get(i), possibleSwaps.get(0));
								possibleSwaps.remove(0);
								
							}
					
						} // End while there are possible swaps left							
					} // End if					
					
					i++;
				} // End while i < assignedPositions.size() and no improving swaps have been found
				
				v++;
			} // End while v < vessels.size() and no improving swaps have been found
 
			
			
			// All vessels and schedules have been looped through
			if (!improvingSwap){
				
				//TEST
				//System.out.println("NO IMPROVING SWAP MADE ROOM FOR " + solution.unassignedPositions.get(0).id );
				
				// No improvingSwap has been found. Assign position to dummy
				solution.dummyPositions.add(solution.unassignedPositions.get(0));
				solution.unassignedPositions.remove(0);
				
			}
		
			
		} // End while there still are unassigned positions

	}
	
	
	
	
	/*
	 * -------------- Method that attempts to insert unassigned positions ----------------
	 */
	
	public static void insertUnassigned(Solution solution){
		
		while (!solution.unassignedPositions.isEmpty()){
			
			if (solution.possibleInsertionFoundFor(solution.unassignedPositions.get(0))){
								
				// Possible insertion found and executed. Remove position from unassigned list
				solution.unassignedPositions.remove(0);
				
			}else{
				
				// No possible insertion found
				solution.dummyPositions.add(solution.unassignedPositions.get(0));
				solution.unassignedPositions.remove(0);
				
			}
			
		}
		
		
	}
	
	
	/*
	 * -------------- Method that attempts to reassign positions in Solution in order to make room for unassigned positions -----------------------
	 */
	
	public static void reassign(Solution solution){
		
		boolean reassignmentFound = false;
		int v = 0;
		int i = 0;
		Position j;
		
		while (!solution.unassignedPositions.isEmpty()){
			reassignmentFound = false;
			
			//TEST
			//System.out.println("SEARCHING FOR A REASSIGNMENT TO INSERT POSITION " + solution.unassignedPositions.get(0).id);
			
			
			// Iterate over all vessels to find a possible position this unassigned voyage can be swapped with
			v = 0;		
			while (v < solution.vessels.size() && !reassignmentFound){
				
				//TEST
				//System.out.println("SCHEDULE FOR VESSEL " + solution.vessels.get(v).code + ":");
				//solution.vessels.get(v).printSchedule();

				
				i = 0;							
				while (i < solution.vessels.get(v).assignedPositions.size() && !reassignmentFound){
					
					// Check if we can remove the current position and replace it with the unassigned dummy position
					j = solution.vessels.get(v).assignedPositions.get(i);
					solution.vessels.get(v).assignedPositions.remove(i);
					
					
					if (solution.unassignedPositions.get(0).canBeAddedInSequence(solution.vessels.get(v), i, solution)){					
						
						//TEST
						//System.out.println("TRYING TO REASSIGN POSITION " + j.id);
						
						
						// Perform assignment
						solution.vessels.get(v).assignedPositions.add(i, solution.unassignedPositions.get(0));
						
						
						// Search for a possible place of inserting j
						if (!solution.possibleInsertionFoundFor(j)){
							
							// No possible placements of j was found. Revert the changes made.
							solution.vessels.get(v).assignedPositions.remove(i);
							solution.vessels.get(v).assignedPositions.add(i, j);
							i++;

						}else{
							
							//TEST
							//System.out.println("SUCCESSFUL REASSIGNMENT. POSITION " + solution.unassignedPositions.get(0).id + " INSERTED. NEW VESSEL SCHEDULE: ");
							//solution.vessels.get(v).printSchedule();

							// The reassignment was successful
							//solution.vessels.get(v).assignStartTimes(solution);
							reassignmentFound = true;
							solution.unassignedPositions.remove(0);

						}
						
						
					}else{
						
						// Add position back to the schedule
						solution.vessels.get(v).assignedPositions.add(i, j);
						i++;
					}
					
				} // End while i
				
				v++;
				
			} // End while v
			
			
			
			if (!reassignmentFound){
				
				//TEST
				//System.out.println("NO REASSIGNMENT POSSIBLE TO MAKE ROOM FOR POSITION " + solution.unassignedPositions.get(0).id);
				
				// No reassignments possible, add the unassigned position to a dummy.
				solution.dummyPositions.add(solution.unassignedPositions.get(0));
				solution.unassignedPositions.remove(0);	
			}
			
			
			
			// Find a possible voyage covered by vessels which can be swapped with the unassigned position
			
			// For each of these voyages, see if it can be inserted elsewhere
			
			// If it can, perform the reassignment. If not, search further
			
			// No reassignments can be done to make room for the unassigned position, assign it to a dummy and move on the next unassigned position
			
			
			
			
		} // End while there are unassigned positions left
		
	}
	
	
}
