import gurobi.GRB;
import gurobi.GRBException;

import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Date;
import java.util.Random;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

 

public class Simulator {
	public Problem problem;
	public SimulationStatistics SimulationStats;
	private int currentIntInstance;
	private int robustMethod;
	private int recoveryMethod;
	private HSSFWorkbook workbookout;
	public Writer writer1;
	public Writer writer2;
	
	

	// Declaring the constructor

	public Simulator( Problem string, int currentIntInstance, int robustMethod, int recoveryMethod, HSSFWorkbook workbookout) {
		this.problem = string; 
		this.SimulationStats= new SimulationStatistics(currentIntInstance);
		this.currentIntInstance = currentIntInstance;
		this.robustMethod = robustMethod;
		this.recoveryMethod = recoveryMethod;
		this.workbookout = workbookout;
		this.writer1 = null;
		try {
			this.writer1 = new BufferedWriter(new OutputStreamWriter(new FileOutputStream("ErrorTesting 1- Fixing speedVariables.txt")));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		this.writer2 = null;
		try {
			this.writer2 = new BufferedWriter(new OutputStreamWriter(new FileOutputStream("ErrorTesting 2- Opening speedVariables.txt")));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/*
	 * Defining the simulate-method. It will simulate the operation of the plan and include the possibility of disruptive events for each day in the planning period.
	 */
	
	public void simulate(){
	
		
		for (int simulation = 0; simulation < GlobalConstants.numberOfSimulations; simulation++){
			
			
			//Creating the SimulationEvents object. If GlobalConstants.predeterminedEvents is false, it will create new events. If it is true it will import the predetermined events from file
			SimulationEvents simEvents = new SimulationEvents(problem, currentIntInstance, simulation);
			
			
			if (GlobalConstants.predeterminedEvents == true){
				
				
				// Creating the mixed integer problem with variables, constraints and objective value
				Mip mip = new Mip(problem);
				
				// Making the mip in simEvents equal to the mip created here in simulation-object
				simEvents.setMip(mip);
				
				// Getting the disrupted events from file if GlobalConstants.predeterminedEvents == true
				simEvents.getDisruptedEvents();
				// Creates an object with information about the plan found when solving the FDP using the RHH. This plan will be simulated  
				SimulationObject shippingPlan = new SimulationObject(mip, this.problem, this.currentIntInstance, simEvents); 
				
				
				/*
				 * ------------------ Add robustness to model based on which robust method that is chosen for this simulation -----------------------
				 */
				
				if (GlobalConstants.predeterminedInitialSolution == false){
						
					
				    // 1 - Adjust sailing times
					if (robustMethod == 1){
			            mip.adjustSailingTimes();
					}
					
					// 2 - Reward early arrivals
					if(robustMethod == 2){
			            mip.rewardEarlyArrivals();
					}
					
					// 3 - Penalize risky start times
					if(robustMethod == 3){
			            mip.penalizeRiskyStartTimes();
					}
					
					
					// Creating a RHH-solving method and solving the problem with the RHH
					RollingHorizonHeuristic rhh = new RollingHorizonHeuristic(mip, this.currentIntInstance, GlobalConstants.robustMethods.get(robustMethod));
					try {
						rhh.solveRHH();
					} catch (GRBException e) {
						e.printStackTrace();
					}
					
					
					// Importing the initial plan from the solution of the RHH. Fixing all variables except from the time- and delay variables
					shippingPlan.setInitialState();	
					
					// Remove the new variables and constraints that was added with the robust plan
					// 1 - Adjust sailing times
					if (robustMethod == 1){
			            mip.removeAdjustSailingTimes();
					}
					
					// 2 - Remove reward early arrivals
					if(robustMethod == 2){
						mip.removeRewardEarlyArrivals();
					}
					
					// 3 - Remove Penalize risky start times
					if(robustMethod == 3){
			            mip.removePenalizeRiskyStartTimes();
					}
				
											
				    if (GlobalConstants.openSpeedVariables == true){
				    	shippingPlan.fixSpeedBeforeSimulation();
				    	
				    }
				 // Run model to update model in order to the changes made 
				      try {
						mip.model.optimize();
					} catch (GRBException e1) {
						e1.printStackTrace();
					}
				    // Adding statistics in order to record values from the model 
				    SimulationStatistics.addPlannedObjectiveValue(mip);
				    
					if (GlobalConstants.openSpeedVariables == true){
						shippingPlan.openSpeedBeforeSimulation();
					}
				}
				if (GlobalConstants.predeterminedInitialSolution == true){
					
			       	String robustnessMethod = "";
			        
	            	// 1 - Adjust sailing times
	      				if (GlobalConstants.ast){
	      					robustnessMethod = "ast";
	      				}
	      				
	      				// 2 - Reward early arrivals
	      				if(GlobalConstants.rea){
	      					robustnessMethod = "rea";

	      				}
	      				
	      				// 3 - Penalize risky start times
	      				if(GlobalConstants.prst){
	      					robustnessMethod = "prst";

	      				}
	      				 				
	      				
	      		mip.getInitialSoultion(robustnessMethod);
				
	      		// Run model to update model in order to the changes made 
			      try {
					mip.model.optimize();
				} catch (GRBException e1) {
					e1.printStackTrace();
				}
			    // Adding statistics in order to record values from the model 
			    SimulationStatistics.addPlannedObjectiveValue(mip);
			    shippingPlan.setInitialState();	
			    if (GlobalConstants.openSpeedVariables == true){
					shippingPlan.openSpeedBeforeSimulation();
			    }
					try {
						mip.model.optimize();
					} catch (GRBException e1) {
						e1.printStackTrace();
					}
                	
				
			    
				}
				
				
				/*
				 * Starting simulating the plan in the planning period period and checking for disruptive events for each day. If disruptive events occur, the start time of voyage and possible delays are updated in shippingPlan 
				 */
				

				for (int m = 0; m < this.problem.months.size(); m++){
					for (int n = this.problem.months.get(m).getStart(); n < this.problem.months.get(m).getEnd() ; n++ ){
						/*
						if (GlobalConstants.openSpeedVariables == false){
							try {
								mip.model.optimize();
							} catch (GRBException e1) {
								e1.printStackTrace();
							}
						}*/
						System.out.println("Day: "+n);
						shippingPlan.objectState.getVariablesFromModel();
						SimulationObjectState objectStatePlanned = shippingPlan.objectState;
						PlannedState plannedState = shippingPlan.saveObjectState(problem, mip, objectStatePlanned);
						
							if (simEvents.preDeterminedDaysRouteDisruptionsOccur.contains(n)){
								if (shippingPlan.updateModelDueToSailingDisruptions(n)){
									shippingPlan.updateState();	
								}
							}
							
							if (simEvents.preDeterminedDaysPortDisruptionsOccur.contains(n)){
							
								if (shippingPlan.updateModelDueToPortsDisruptions(n)){
									shippingPlan.updateState();
								}
							}
						
							/*
							 * RECOVERY. If delay caused by disruptions is high enough, a recovery method is called. 
							 */
							
							if(recoveryMethod == 1){
								
								if (shippingPlan.recoveryNeeded(n)){
								    shippingPlan.performRHHRecoveryStrategy(mip, currentIntInstance, n, m);
									shippingPlan.updateListOfTriggerPositions(n);
								}
							}
							
							SimulationObjectState objectStateAfterRecovery = shippingPlan.objectState;
							shippingPlan.countTimeChanges(plannedState, objectStateAfterRecovery);
							shippingPlan.countSwaps(plannedState, objectStateAfterRecovery);
							shippingPlan.changesMadeComparedToObjectStateBeforeRecovery =  shippingPlan.timeChangesToObjectStateBeforeRecovery + shippingPlan.swapsComparedToObjectStateBeforeRecovery;	
							
						// The following methods fixes the start time for vessels and voyages that have started before day n
						
						shippingPlan.fixStartTimeForVesselsUpToCurrentDay(n);
						shippingPlan.fixStartTimeForVoyagesUpToCurrentDay(n);
						
						if (GlobalConstants.openSpeedVariables == true){
							shippingPlan.fixSpeedVariablesUpToCurrentDay(n);
						}
						
					
					}
				} 
				
				// Save key information to statistic object 	
				SimulationStats = shippingPlan.saveInfoToSimStats(SimulationStats);
				
				//Mip.model.dispose() delets the current mip-model. This needs to be done after each simulation so we're only working with one model in each simulation.
				mip.model.dispose();
				try {
					mip.env.dispose();
				} catch (GRBException e) {
					e.printStackTrace();
				}
				
			
			}
			
			// If the predetermined events has not been created, this is done in the following section
			if (GlobalConstants.predeterminedEvents == false){	
				for (int m = 0; m < this.problem.months.size(); m++){
					for (int n = this.problem.months.get(m).getStart(); n < this.problem.months.get(m).getEnd() ; n++ ){
						for (int r = 0; r < problem.standardRoutes.size(); r++){	
							if (simEvents.sailingDisruption(n,r)){
								simEvents.setRoutesDisruptedBySailing(problem.standardRoutes.get(r), n);	
							}
						}
						for (int p = 0; p < this.problem.ports.size(); p ++){
							if (simEvents.portDisruption(n,p)){
								simEvents.setPortsDisruptedByPorts(problem.ports.get(p), n);
							}
						}
					}
				}
				simEvents.storeDisruptedEvents();
				
			}
			
			
		}
		
		if (GlobalConstants.predeterminedEvents == true){	
		// This method writes out all the information saved in SimulationStats to a file.
		SimulationStats.writeSimulationStatisticsToExcel(currentIntInstance, robustMethod, recoveryMethod, workbookout);
		}
		System.out.println("Done Simulating!");
		
	}
}