import java.util.ArrayList;

import gurobi.GRB;
import gurobi.GRBException;
import gurobi.GRBLinExpr;
import gurobi.GRBVar;

import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Date;
import java.util.Random;

public class SimulationObject {
	public Mip mip;
	public Problem problem;
	public ArrayList<String> disruptedRoutes;
	public ArrayList<String> disruptedPorts;
	private int numberOfDisruptedPorts;
	private int numberOfDisruptedRoutes;
	public SimulationObjectState objectState;
	private double SimObjectiveValue;
	private ArrayList<String> triggerPositions;
	private ArrayList<Double> delayOnTriggerPositions;
	private int currentIntInstance;
	public int numberOfRecoveries;
	public int changesMadeComparedToObjectStateBeforeRecovery;
	public int timeChangesToObjectStateBeforeRecovery;
	public int swapsComparedToObjectStateBeforeRecovery;
	public double totalTimeSavedByRS;
	public ArrayList<Position> positionsDisruptedByRoutes;
	public ArrayList<Position> positionsDisruptedByPorts;
	public ArrayList<Double> disruptedPositionsCompletionRatio;
	public ArrayList<Double> plannedYvi;
	public SimulationEvents simulationEvents;
	
	public SimulationObject(Mip mip2, Problem string, int currentIntInstance, SimulationEvents simEvents){	
		this.mip = mip2;
		this.problem = string; 
		this.disruptedRoutes = new ArrayList<String>();
		this.disruptedPorts = new ArrayList<String>();
		this.objectState= new SimulationObjectState(this.problem, this.mip);
		this.numberOfDisruptedPorts = 0;
		this.numberOfDisruptedRoutes = 0;
		this.SimObjectiveValue = 0;
		this.triggerPositions = new ArrayList<String>();
		this.delayOnTriggerPositions = new ArrayList<Double>();
		this.currentIntInstance = currentIntInstance;
		this.numberOfRecoveries = 0;
		this.changesMadeComparedToObjectStateBeforeRecovery = 0;
		this.timeChangesToObjectStateBeforeRecovery = 0;
		this.swapsComparedToObjectStateBeforeRecovery = 0;
		this.totalTimeSavedByRS = 0;
		this.positionsDisruptedByRoutes = new ArrayList<Position>();
		this.positionsDisruptedByPorts = new ArrayList<Position>();
		this.disruptedPositionsCompletionRatio = new ArrayList<Double>();
		this.simulationEvents = simEvents;
		
		
	}



	
	
// ----------------- ROUTE DISRUPTION METHODS ---------------------	
	
	
	

	// This method extends the sailingtime of the positions that are disrupted, if they are not already disrupted by routes. 
	public boolean updateModelDueToSailingDisruptions(int n){
		numberOfDisruptedRoutes = numberOfDisruptedRoutes +1;
		ArrayList<Position> disruptedPositions = new ArrayList<Position>();
		ArrayList<Position> disruptedPositionsHelpingList = new ArrayList<Position>();
		for (int i= 0; i < simulationEvents.preDeterminedDaysRouteDisruptionsOccur.size(); i ++){
			if (simulationEvents.preDeterminedDaysRouteDisruptionsOccur.get(i) == n){
				StandardRoute disruptedRoute = simulationEvents.preDeterminedRouteDisruptions.get(i);
				for(int r = 0; r < this.problem.standardRoutes.size(); r ++){
					if (this.problem.standardRoutes.get(r).code == disruptedRoute.code){
						disruptedPositionsHelpingList = findPositionsByRouteAndDay(n,r);
						for (int e = 0; e <disruptedPositionsHelpingList.size(); e++ ){
							disruptedPositions.add(disruptedPositionsHelpingList.get(e));
						}
					}
				}
			}		
		}
		if(disruptedPositions.isEmpty()){
			return false;
		}
		else{
			for (int i = 0 ; i < disruptedPositions.size(); i ++){
				for (int v = 0 ; v < problem.vessels.size(); v ++ ){
					int index = problem.positions.indexOf(disruptedPositions.get(i));
					if (objectState.y_i[v][index] > 0.5){
						double positionCompletionRatio = findPositionCompletionRatio(v,index,n);
						disruptedPositionsCompletionRatio.add(positionCompletionRatio);
						this.mip.addPositionSailingTime(disruptedPositions.get(i),problem.vessels.get(v), positionsDisruptedByPorts, positionCompletionRatio);
						
						try {
							this.mip.model.update();
						} catch (GRBException e) {
							e.printStackTrace();
						}

					}
					
				}
				
			}
			
			if (disruptedPositionsCompletionRatio.size() != positionsDisruptedByRoutes.size()){
				System.out.println("The size of positionsDisruptedByRoutesRatio and positionsDisruptedByRoutes. Something is not right");
				System.exit(1);
			}
			
			return true;
		}
		
	}

	// This method finds positions that are sailing route number r at day n. 
	public ArrayList<Position> findPositionsByRouteAndDay(int n , int r){
		ArrayList<Position> positionsSailingRouteAtDayN = new ArrayList<Position> ();
		ArrayList<Position> positionsSailingRouteAtDayNHelpingList = new ArrayList<Position> ();
		double durationOfVoyage = 0;
		double sailingTime = 0;
		for (int v = 0; v < this.problem.vessels.size(); v++){
			for (int i = 0; i < this.problem.positions.size(); i ++ ){
				if(objectState.y_i[v][i]>0.5){
					for (int s = 0; s < this.problem.speedProfiles.size(); s ++){
						sailingTime = Sailing.getSailingDuration(this.problem.vessels.get(v).code, this.problem.positions.get(i).route, problem.speedProfiles.get(s).code, problem.sailings);
						durationOfVoyage = durationOfVoyage + sailingTime*objectState.x[v][i][s];
					}
					if ((this.problem.positions.get(i).route == problem.standardRoutes.get(r).getStandardRouteCode()) && ((objectState.t[i] <= n) && ((objectState.t[i] + durationOfVoyage)  > n))){
						positionsSailingRouteAtDayNHelpingList.add(problem.positions.get(i));
					}
					durationOfVoyage = 0;
				}
			}
		}		
		for (int i = 0; i < positionsSailingRouteAtDayNHelpingList.size(); i ++){
			if (positionsDisruptedByRoutes.contains(positionsSailingRouteAtDayNHelpingList.get(i)) == false){
				this.positionsDisruptedByRoutes.add(positionsSailingRouteAtDayNHelpingList.get(i));
				positionsSailingRouteAtDayN.add(positionsSailingRouteAtDayNHelpingList.get(i));
			}
			
		}
	
		
		return positionsSailingRouteAtDayN;
	}
	
	
	public double findPositionCompletionRatio(int v, int index, int n){
		double durationOfVoyage = 0;
		for (int s = 0; s < problem.speedProfiles.size(); s++){
			double sailingTime = Sailing.getSailingDuration(problem.vessels.get(v).code, problem.positions.get(index).route, problem.speedProfiles.get(s).code, problem.sailings);
			double duration = sailingTime*objectState.x[v][index][s];
			durationOfVoyage = durationOfVoyage + duration;
		}
		
		double ratio = ((int)Math.floor(objectState.t[index] - n) + durationOfVoyage)/durationOfVoyage;
		
		if (ratio > 1){
			System.out.println("Value of durationOfVoyage: "+ durationOfVoyage);
			System.out.println("Value of start time for voyage: "+ Double.toString(objectState.t[index]));
			System.out.println("Value of N: "+ Integer.toString(n));
			System.out.println("ratio for voyage number " + Integer.toString(index) + " exceeds 1. Something is wrong");
			System.exit(1);
		}
			
		return ratio;
	}

// ------------------ PORT DISRUPTION METHODS	------------------	
	
	

// This method adds delay to the shipping plan due to disruptive events in ports	
	public boolean updateModelDueToPortsDisruptions(int n){
		numberOfDisruptedPorts = numberOfDisruptedPorts +1;
		ArrayList<Position> disruptedPositions = new ArrayList<Position>();
		ArrayList<Position> disruptedPositionsHelpingList = new ArrayList<Position>();
		for (int i= 0; i < simulationEvents.preDeterminedDaysPortDisruptionsOccur.size(); i ++){
			if (simulationEvents.preDeterminedDaysPortDisruptionsOccur.get(i) == n){
				Port port = simulationEvents.preDeterminedPortDisruptions.get(i);
				for (int p = 0; p < this.problem.ports.size(); p ++){
					if (this.problem.ports.get(p).code == port.code)
					disruptedPositionsHelpingList = findPositionsSailingAtDayNToPortNumberP(n,p);
					for (int e = 0; e <disruptedPositionsHelpingList.size(); e++ ){
						disruptedPositions.add(disruptedPositionsHelpingList.get(e));
					}
					disruptedPositionsHelpingList.clear();
				}
			}
		}
		
		if(disruptedPositions.isEmpty()){
			return false;
		}
		else{
			this.mip.addPortDelays(disruptedPositions, positionsDisruptedByRoutes, disruptedPositionsCompletionRatio, objectState);		
		}
			try {
				this.mip.model.update();
			} catch (GRBException e) {
				e.printStackTrace();
			}
			
		return true;
		
	}
	
	

	// This method identifies the positions that has the disruptive ports as destination ports
	public ArrayList<Position> findPositionsSailingAtDayNToPortNumberP(int n , int p){
		ArrayList<Position> positionsUsingPortNumberPAtDayN = new ArrayList<Position> ();
		ArrayList<Position> positionsUsingPortNumberPAtDayNHelpingList = new ArrayList<Position> ();
		double durationOfVoyage = 0;
		double sailingTime = 0;
		for (int r = 0; r < this.problem.standardRoutes.size() ; r ++){
				if (this.problem.standardRoutes.get(r).portCodeEnd == this.problem.ports.get(p).code){
					for (int v = 0; v < this.problem.vessels.size(); v++){
						for (int i = 0; i < this.problem.positions.size(); i ++ ){	
							if(objectState.y_i[v][i]>0.5){
								for (int s = 0; s < this.problem.speedProfiles.size(); s ++){
									sailingTime = Sailing.getSailingDuration(this.problem.vessels.get(v).code, this.problem.positions.get(i).route, problem.speedProfiles.get(s).code, problem.sailings);
									durationOfVoyage = durationOfVoyage + sailingTime*objectState.x[v][i][s];
								}
								if ((this.problem.positions.get(i).route == problem.standardRoutes.get(r).getStandardRouteCode()) && ((objectState.t[i] <= n) && ((objectState.t[i] + durationOfVoyage) > n))){
									positionsUsingPortNumberPAtDayNHelpingList.add(problem.positions.get(i));
								}
								durationOfVoyage = 0;
							}
						}
					}
				}
					
			}
		for (int i = 0; i < positionsUsingPortNumberPAtDayNHelpingList.size(); i ++){
			if (positionsDisruptedByPorts.contains(positionsUsingPortNumberPAtDayNHelpingList.get(i)) == false){
				this.positionsDisruptedByPorts.add(positionsUsingPortNumberPAtDayNHelpingList.get(i));
				positionsUsingPortNumberPAtDayN.add(positionsUsingPortNumberPAtDayNHelpingList.get(i));
			}
		}			
		return positionsUsingPortNumberPAtDayN;
	}
	
	


    

	//  ------------------ State Methods --------------------

	//These methods fixes variables in the model to the variables values found from the optimized RHH
	

		public void setInitialState(){
			objectState.getVariablesFromModel(); 
			objectState.updateUpperBoundForTimeAndDelayVariables();
			objectState.updateInitialState();
		}

		public void updateState(){
			this.mip.solveExact(currentIntInstance);
			objectState.getVariablesFromModel(); 
			objectState.updateState();
		}
		
		
		
	//These methods fixes the time-start variables for voyages and vessels that are started before day n
		public void fixStartTimeForVoyagesUpToCurrentDay(int n){
			objectState.fixStartTimeForVoyages(n);
			
		}
		
		public void fixStartTimeForVesselsUpToCurrentDay(int n){
			objectState.fixStartTimeForVessels(n);
		}
		

		
	// ---------------- Recovery Methods ----------------------	
		
		
		public boolean recoveryNeeded(int dayNumber){
			boolean recoveryNeeded = false;
			for (int i = 0; i < problem.positions.size(); i ++){
				if (objectState.t[i] > dayNumber){	
					try {
						if (GlobalConstants.allowedDelayBeforeTriggeringRS <  mip.model.getVarByName( "d[" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X)){
							if (triggerPositions.contains(problem.positions.get(i).id)){
								if (delayOnTriggerPositions.get(triggerPositions.indexOf(problem.positions.get(i).id)) < mip.model.getVarByName( "d[" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X)){
									recoveryNeeded = true;
								}
								
							}
							else{
								recoveryNeeded = true;
							}
						}
					} catch (GRBException e) {
						e.printStackTrace();
						
					}
				}
			}
			return recoveryNeeded;
		}
	
	public void performRHHRecoveryStrategy(Mip mipA, int currentIntInstance, int n, int monthNumber){
		double delayBeforeRecoveryStrategyIsCalled = mip.getDelayBeforeRS(n);
		objectState.openVariablesAfterDayN(n);
		RecoveryStrategyRHH rhhRecovery = new RecoveryStrategyRHH(mipA, currentIntInstance, objectState, n);
		try {
			rhhRecovery.solveRHH();
		} catch (GRBException e) {
			e.printStackTrace();
		}
		objectState.getVariablesFromModel();
		objectState.updateState();
		try {
			mip.model.optimize();
		} catch (GRBException e) {
			e.printStackTrace();
		}
		objectState.getVariablesFromModel();
		objectState.updateState();
		double delayAfterRecoveryStrategyIsCalled = mip.getDelayAfterRS(n);
		double delaySaved = delayBeforeRecoveryStrategyIsCalled - delayAfterRecoveryStrategyIsCalled;
		numberOfRecoveries = numberOfRecoveries +1;
		totalTimeSavedByRS = totalTimeSavedByRS + delaySaved;

		
		}
	

	// Method that finds the positions that has a delay above the trigger-point to the recovery
	public void updateListOfTriggerPositions(int n){
		this.triggerPositions = new ArrayList<String>();
		this.delayOnTriggerPositions = new ArrayList<Double>();
		double maxDelay = 0;
		for (int i = 0; i < problem.positions.size(); i ++){
				try {
					if (GlobalConstants.allowedDelayBeforeTriggeringRS <  this.mip.model.getVarByName( "d[" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X)){
						triggerPositions.add(problem.positions.get(i).id);
						maxDelay= mip.model.getVarByName( "d[" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X);
						delayOnTriggerPositions.add(maxDelay);
					}
				} catch (GRBException e) {
					e.printStackTrace();
					System.out.println(e.getMessage());
					System.out.println(e.getErrorCode());
					System.out.println("This happens at day: " + n + " and position number: " + Integer.toString(i));
				}
		}
	}
		
		
	// ---------------- Simulation Statistics Methods ----------------
		// The following method saves key information from the simulation object to the simulation statistics object.
		public SimulationStatistics saveInfoToSimStats(SimulationStatistics simStats){
			try {
				 SimObjectiveValue =  this.mip.model.get(GRB.DoubleAttr.ObjVal);
			} catch (GRBException e1) {
				System.out.println("Something is wrong");
				e1.printStackTrace();
				System.exit(1);
			}
			double maxDelay = 0;
			double totalDelay = 0;
			String maxDelayPos = "No voyages are delayed";
			for (int i = 0; i < problem.positions.size(); i++) {
				try {
					totalDelay = totalDelay + mip.model.getVarByName( "d[" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X);
				} catch (GRBException e) {
					e.printStackTrace();
				}
				try {
					if (maxDelay <  mip.model.getVarByName( "d[" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X)){
						maxDelay = mip.model.getVarByName( "d[" + problem.positions.get(i).id + "]").get(GRB.DoubleAttr.X);
						maxDelayPos = problem.positions.get(i).id;
					}
				} catch (GRBException e) {
					e.printStackTrace();
				}
			}
			simStats.addSimulationResults(SimObjectiveValue, numberOfDisruptedPorts, numberOfDisruptedRoutes, maxDelay, maxDelayPos, totalDelay, numberOfRecoveries, totalTimeSavedByRS, changesMadeComparedToObjectStateBeforeRecovery, swapsComparedToObjectStateBeforeRecovery, timeChangesToObjectStateBeforeRecovery);
			
			return simStats;
		}
		
		/*
		 * Method to save object state to static values for comparison with how many changes that occurs in a recovery strategy
		 */

		public PlannedState	saveObjectState(Problem problem, Mip mip, SimulationObjectState objectState){
			PlannedState plannedState = new PlannedState(problem, mip);
			
			for (int v = 0; v < problem.vessels.size(); v ++){
				for (int i = 0; i < problem.positions.size(); i ++){
				plannedState.y_vi[v][i] = objectState.y_i[v][i];
				plannedState.y_d[v][i] = objectState.y_d[v][i];
				plannedState.y_o[v][i] = objectState.y_o[v][i];
				plannedState.t[i] = objectState.t[i];
				plannedState.t_o[v] = objectState.t_o[v];
				}
			}
			return plannedState;
		}
		
	// ------------------ Changes To The Original Plan Methods -------------------	
		

		public void countTimeChanges(PlannedState objectStateBeforeRS, SimulationObjectState objectStateAfterRS){
			
			int timechanges = 0; 
			
			for (int i = 0; i < problem.positions.size(); i ++){
				if (Math.round(objectStateBeforeRS.t[i]) >= Math.round(objectStateAfterRS.t[i]) + GlobalConstants.acceptedTimeDeviation || Math.round(objectStateBeforeRS.t[i]) <= Math.round(objectStateAfterRS.t[i]) - GlobalConstants.acceptedTimeDeviation){
					timechanges = timechanges +1;
				}
			}
	
			for (int v = 0;v < problem.vessels.size(); v++){
				if (Math.round(objectStateBeforeRS.t_o[v]) >= Math.round(objectStateAfterRS.t_o[v]) + GlobalConstants.acceptedTimeDeviation || Math.round(objectStateBeforeRS.t_o[v]) <= Math.round(objectStateAfterRS.t_o[v]) - GlobalConstants.acceptedTimeDeviation){
					timechanges = timechanges +1;					
				}
			}
			timeChangesToObjectStateBeforeRecovery = timeChangesToObjectStateBeforeRecovery + timechanges; 
		}
		

		public void countSwaps(PlannedState objectStateBeforeRS, SimulationObjectState objectStateAfterRS){

			int swaps = 0;
			for (int i = 0; i < problem.positions.size(); i ++){
				for (int v = 0; v < problem.vessels.size(); v ++){
					if (objectStateBeforeRS.y_vi[v][i] > 0.5 && objectStateAfterRS.y_i[v][i] < 0.5){
						swaps = swaps +1;
					}
				}
			}
			swapsComparedToObjectStateBeforeRecovery = swapsComparedToObjectStateBeforeRecovery + swaps; 
		}
		
		// ---------------- Fix Speed Variables Before Simulation -------------------
		
	public void fixSpeedBeforeSimulation(){
		objectState.fixSpeedBeforeSimulationSOS();
	}
	
		// --------------- Open Speed Variables Before Simulation ------------------
	public void openSpeedBeforeSimulation(){
		objectState.openSpeedBeforeSimulationSOS();
	}
		
		// ---------------- Fix Speed Variables Before Day N -------------------------
		
	public void fixSpeedVariablesUpToCurrentDay(int n){
			objectState.fixVoyageSpeed(n);
			fixBallastSpeed(n);
			objectState.fixBallastSpeedFromOrigin(n);
			
		}


	public void fixBallastSpeed(int n){
		
		double durationOfVoyage = 0;
		double sailingTime = 0;
		
		for (int i = 0; i < problem.positions.size(); i++){
			if ((objectState.fixedBallastSpeed[i] == false)){
				for (int v = 0; v < this.problem.vessels.size(); v ++){
					if ((this.objectState.y_i[v][i]) > 0.5){
						
						if ((positionsDisruptedByRoutes.contains(this.problem.positions.get(i))) && positionsDisruptedByPorts.contains(this.problem.positions.get(i)) ){
							int index = positionsDisruptedByRoutes.indexOf(this.problem.positions.get(i));
							for (int s = 0; s < problem.speedProfiles.size(); s++){
								sailingTime = sailingTime + (1+ GlobalConstants.disruptedSailingTimeFactor*disruptedPositionsCompletionRatio.get(index))*Sailing.getSailingDuration(this.problem.vessels.get(v).code, this.problem.positions.get(i).route, problem.speedProfiles.get(s).code, problem.sailings)*objectState.x[v][i][s];
				      		}
							durationOfVoyage=  sailingTime + GlobalConstants.delaysByPortStrikes;	
							if ((this.objectState.t[i] + durationOfVoyage)<= n){
								objectState.fixedBallastSpeed[i] = true;
							}
							sailingTime = 0;
							durationOfVoyage =0;
						}
						
						else if (positionsDisruptedByRoutes.contains(this.problem.positions.get(i))){
							int index = positionsDisruptedByRoutes.indexOf(this.problem.positions.get(i));
							for (int s = 0; s < problem.speedProfiles.size(); s++){
								sailingTime = sailingTime + (1+ GlobalConstants.disruptedSailingTimeFactor*disruptedPositionsCompletionRatio.get(index))*Sailing.getSailingDuration(this.problem.vessels.get(v).code, this.problem.positions.get(i).route, problem.speedProfiles.get(s).code, problem.sailings)*objectState.x[v][i][s];
				      		}
							durationOfVoyage = sailingTime;
							if ((this.objectState.t[i] + durationOfVoyage)<= n){
								objectState.fixedBallastSpeed[i] = true;
							}
							sailingTime = 0;
							durationOfVoyage =0;
						}
						
						else if (positionsDisruptedByPorts.contains(this.problem.positions.get(i))){
							for (int s = 0; s < problem.speedProfiles.size(); s++){
								sailingTime = sailingTime + Sailing.getSailingDuration(this.problem.vessels.get(v).code, this.problem.positions.get(i).route, problem.speedProfiles.get(s).code, problem.sailings)*objectState.x[v][i][s];
				      		}
							durationOfVoyage = sailingTime + GlobalConstants.delaysByPortStrikes; 
							if ((this.objectState.t[i] + durationOfVoyage)<= n){
								objectState.fixedBallastSpeed[i] = true;
							}
							sailingTime = 0;
							durationOfVoyage =0;
						}
						
						else{
							for (int s = 0; s < problem.speedProfiles.size(); s++){
								sailingTime = sailingTime + Sailing.getSailingDuration(this.problem.vessels.get(v).code, this.problem.positions.get(i).route, problem.speedProfiles.get(s).code, problem.sailings)*objectState.x[v][i][s];
				      		}
							durationOfVoyage = sailingTime;
							if ((this.objectState.t[i] + durationOfVoyage)<= n){
								objectState.fixedBallastSpeed[i] = true;
							}
							sailingTime = 0;
							durationOfVoyage =0;
						}
						
						
						if (objectState.fixedBallastSpeed[i] == true){
							for (int vessels = 0; vessels < problem.vessels.size(); vessels ++ ){
								for (int j = 0; j < problem.positions.size(); j ++){
									for (int s = 0; s < problem.speedProfiles.size(); s++){
										if (problem.vesselCanSailPositionToPosition(problem.vessels.get(vessels), problem.positions.get(i), problem.positions.get(j))){
											try {
												mip.model.getVarByName("x_B[" + problem.vessels.get(vessels).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id +  "]["  + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.LB,this.objectState.x_B[vessels][i][j][s]);
												mip.model.getVarByName("x_B[" + problem.vessels.get(vessels).code + "][" + problem.positions.get(i).id + "][" + problem.positions.get(j).id +  "]["  + problem.speedProfiles.get(s).code + "]").set(GRB.DoubleAttr.UB,this.objectState.x_B[vessels][i][j][s]);
											} catch (GRBException e) {
												e.printStackTrace();
												e.getMessage();
												e.getErrorCode();
											}
										}
									}
								}	
							}
						}
					}
				}
			}
		}
	}		
}
	


