
import gurobi.GRB;
import gurobi.GRBException;
import gurobi.GRBVar;

import java.util.ArrayList;

public class FixedVariables {
	public double[][]y_i;
	public double[][][]y;
	public double[][]y_o;
	public double[][]y_d;
	public double[][][]x_Bo;
	public double[][][]x;
	public double[][][][]x_B;
	public double[]dummy;
	public double[]t;
	public double[]d;
	public double[][][][]l;
	public double[][][]z;
	Problem problem; 
	
	public FixedVariables(Problem problem){
		this.y_i 	= new double[problem.vessels.size()][problem.positions.size()];
		this.y 		= new double[problem.vessels.size()][problem.positions.size()][problem.positions.size()];
		this.y_o 	= new double[problem.vessels.size()][problem.positions.size()];
		this.y_d 	= new double[problem.vessels.size()][problem.positions.size()];
		this.x_Bo 	= new double[problem.vessels.size()][problem.positions.size()][problem.speedProfiles.size()];
		this.x		= new double[problem.vessels.size()][problem.positions.size()][problem.speedProfiles.size()];
		this.x_B	= new double[problem.vessels.size()][problem.positions.size()][problem.positions.size()][problem.speedProfiles.size()];
		this.dummy	= new double[problem.positions.size()];
		this.t		= new double[problem.positions.size()];
		this.d		= new double[problem.positions.size()];
		this.l		= new double[problem.positions.size()][problem.balanceCategories.size()][problem.deckCategories.size()][problem.deckCategories.size()];
		this.z		= new double[problem.balanceCategories.size()][problem.deckCategories.size()][problem.months.size()];
		this.problem = problem;
	}
	
		
	public void reset(){
		for (int v = 0; v < problem.vessels.size(); v++ ){
				for (int i = 0; i < problem.positions.size(); i++){
					
					y_i[v][i] = -1;
					if (problem.vesselCanSailInitialPosition(problem.vessels.get(v),problem.positions.get(i))){
					y_o[v][i] = -1;
					}
					y_d[v][i] = -1;
					for (int s = 0; s < problem.speedProfiles.size(); s++){
						x_Bo[v][i][s] 	= -1;
						x[v][i][s] 		= -1;
					}
					for (int j = 0; j < problem.positions.size(); j++){
						y[v][i][j] = -1;
						for (int s = 0; s < problem.speedProfiles.size(); s++){
							x_B[v][i][j][s] = -1;
						}
					}
					dummy[i] 	= -1;
					t[i] 		= -1;
					d[i]		= -1;
					for (int b = 0; b < problem.balanceCategories.size(); b++){
			      		for (int p = 0; p < problem.deckCategories.size(); p++){
			      			for (int k = 0; k < problem.deckCategories.size(); k++){
			      					l[i][b][p][k] = -1;
			      			}
			      			for (int m = 0; m < problem.months.size(); m++){
			      				z[b][p][m] = -1;
			      			}
			      		}
					}	
				}		
			}
	}
	
	public void addPrimary(int counter, GRBVar[][] y_iPV, GRBVar[][] y_oPV, GRBVar[][][] yPV){
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (isPositionInPrimaryPeriod(counter, problem.positions.get(i), problem.months)){
					try {
						if(y_iPV[v][i] != null){
						y_i[v][i]= y_iPV[v][i].get(GRB.DoubleAttr.X);
						}
						if(y_oPV[v][i] != null){
						y_o[v][i]= y_oPV[v][i].get(GRB.DoubleAttr.X);
						}
					}catch (GRBException e){
						e.printStackTrace();
					}
				}	
				for (int j = 0; j < problem.positions.size(); j++){
					try {
						if (yPV[v][i][j] != null && isPositionInPrimaryPeriod(counter, problem.positions.get(j), problem.months)){
								y[v][i][j] = yPV[v][i][j].get(GRB.DoubleAttr.X);
						}
					} catch (GRBException e) {
						e.printStackTrace();
					}
				}	
			}
		}
	}
	


	/*
	* ------------------------- Method to evaluate if position is in primary -------------------------
	*/
	public boolean isPositionInPrimaryPeriod(int counter, Position i, ArrayList<Month> months){
		if (i.endTime <= months.get(counter).end && i.endTime >= months.get(counter).start){
			return true;
		}
		else{
			return false;
		}
	}
	
	// Method to get fixed route
	public double getRouteY(int v, int i, int j){
		return this.y[v][i][j];
	}

/*
 * Method to update the fixed variables according to the simulation when calling the RHH as a recovery strategy
 */

	public void updateAccordingToObjectState(SimulationObjectState objectState, int dayNumber) {
		// t
		for (int i = 0; i < problem.positions.size(); i++){
			if (objectState.t[i] <= dayNumber){
				this.t[i] = objectState.t[i];
			}
		}
		
		// y_vi
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (problem.vesselCanSailPosition(problem.vessels.get(v), problem.positions.get(i))){
					if (objectState.t[i] <= dayNumber){
						this.y_i[v][i] = objectState.y_i[v][i];
					}
					if (GlobalConstants.openSpeedVariables== true){
						for (int j =0; j < problem.positions.size(); j++){
							if (objectState.fixedBallastSpeed[i]==true){
								if (objectState.y[v][i][j] > 0.5){
									this.y_i[v][j] = objectState.y_i[v][j];
								}
							}
						}
					}
				}
			}
		}
		
		// y_o
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (problem.vesselCanSailInitialPosition(problem.vessels.get(v),problem.positions.get(i))){
					if (objectState.t[i] <= dayNumber){
						this.y_o[v][i] = objectState.y_o[v][i];
					}
				}
			}
		}
		
		// y_d
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				if (problem.vesselCanSailPosition(problem.vessels.get(v), problem.positions.get(i))){
					if (objectState.t[i] <= dayNumber){
						this.y_d[v][i] = objectState.y_d[v][i];
						}
				}
			}
		}
		
		
		// y_vij
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int j = 0; j < problem.positions.size(); j++){
					if (problem.vesselCanSailPositionToPosition(problem.vessels.get(v), problem.positions.get(i), problem.positions.get(j))){
						if (GlobalConstants.openSpeedVariables== false){
							if (objectState.t[j] <= dayNumber){
								this.y[v][i][j] = objectState.y[v][i][j];
							}
						}
						else if (GlobalConstants.openSpeedVariables== true){
							if (objectState.t[j] <= dayNumber){
								this.y[v][i][j] = objectState.y[v][i][j];
							}
							
						else if (objectState.fixedBallastSpeed[i] == true){
								if (objectState.y[v][i][j] > 0.5){
									this.y[v][i][j] = objectState.y[v][i][j];
								}
							}
						}
					}
				}
			}
		}
		
		// x^B_0
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int s = 0; s < problem.speedProfiles.size(); s++){
					if (problem.vesselCanSailInitialPosition(problem.vessels.get(v), problem.positions.get(i))){
						if (objectState.t[i] <= dayNumber){
							this.x_Bo[v][i][s] = objectState.x_Bo[v][i][s];
						}
					}
				}
			}
		}
		
		// x
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int s = 0; s < problem.speedProfiles.size(); s++){
					if (problem.vesselCanSailPosition(problem.vessels.get(v),problem.positions.get(i))){
						if (objectState.t[i] <= dayNumber){
							this.x[v][i][s] = objectState.x[v][i][s];
						}
					}
				}
			}
		}
		
		// x^B
		for (int v = 0; v < problem.vessels.size(); v++ ){
			for (int i = 0; i < problem.positions.size(); i++){
				for (int j = 0; j < problem.positions.size(); j++){
					for (int s = 0; s < problem.speedProfiles.size(); s++){
						if (problem.vesselCanSailPositionToPosition(problem.vessels.get(v), problem.positions.get(i), problem.positions.get(j))){
							if (objectState.t[j] <= dayNumber){
								this.x_B[v][i][j][s] = objectState.x_B[v][i][j][s];
							}
						}
					}
				}
			}
		}
		
		// Dummy
		
		for (int i = 0; i < problem.positions.size(); i++){
				if (objectState.dummy[i] > 0.5){
				 this.dummy[i] = objectState.dummy[i];
				}
		}
		
		
		// l
		/*
		for (int i = 0; i < problem.positions.size(); i++){
			for (int b = 0; b < problem.balanceCategories.size(); b++){
	      		for (int p = 0; p < problem.deckCategories.size(); p++){
	      			for (int k = 0; k < problem.deckCategories.size(); k++){
	      				if (BalanceCategoryStandardRoute.isBalanceCategoryInRoute(problem.balanceCategories.get(b).code, problem.positions.get(i).route, problem.balanceCategoryStandardRoutes)){
	      					if (DeckCategoryToDeckCategory.ifPossibleToPut(problem.deckCategories.get(p).code, problem.deckCategories.get(k).code, problem.deckCategoryToDeckCategories)){
	      						if (t[i] <= dayNumber){
	      							this.l[i][b][p][k] = objectState.l[i][b][p][k];
	      						}
	      					}
	      				}
	      			}
	      		}
			}
		}
		
	*/
	}
}