
import gurobi.GRB;
import gurobi.GRBException;

import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;

public class RollingHorizonHeuristic {
	Mip mip;
	int counter;
	long maxTimer;
	ArrayList<Horizon> horizon;
	double startTime;
	double endTime;
	int currentInstance;
	String robustnessMethod;
	
	// Constructor of RHH object
			public RollingHorizonHeuristic(Mip mip, int currentInstance, String robustnessMethod){
				this.mip = mip;
				this.counter = 0;
				this.maxTimer = GlobalConstants.maxTimer;
				this.horizon = new ArrayList<Horizon>();
				this.startTime = 0;
				this.endTime = 0;
				this.currentInstance = currentInstance;
				this.robustnessMethod = robustnessMethod;
			}
	
			public void solveRHH() throws GRBException{
				
				FixedVariables fixed = new FixedVariables(mip.problem);
				fixed.reset();
				
				//Setup of RHH
				int solutionTimeLimit = (int)(this.maxTimer/(1000*mip.problem.months.size()));
				this.mip.model.getEnv().set(GRB.DoubleParam.TimeLimit, solutionTimeLimit);
				
				System.out.println("MONTHS " + mip.problem.months.size());
				System.out.println("TIME LIMIT " + solutionTimeLimit);

				// START TIMER for whole RHH-model
				this.startTime = System.currentTimeMillis();
			    
				
				/* 
			       * ------------------ Rolling horizons -----------------------
			       */      				
			while (this.counter < this.mip.problem.months.size()){
				double itrStartTime = System.currentTimeMillis();
				Horizon currentHorizon = new Horizon(counter, mip, itrStartTime);
				this.horizon.add(currentHorizon);
			
					System.out.println("");
					System.out.println("Counter: " + counter + " Current time: " + ((System.currentTimeMillis() - startTime)/1000)); 
					int numEndForecasting = getCurEndForecasting(this.counter, this.mip.problem.months);
					
			// Reset model
					try {
						mip.model.reset();
					} catch (GRBException e) {
						e.printStackTrace();
					}
			
					
			// Remove y_vij-variables from old MIP and rebuild for the rolling horizon
					for (int v = 0; v < mip.problem.vessels.size(); v++) {
					      	for (int i = 0; i < mip.problem.positions.size(); i++){
					      		for (int j = 0; j < mip.problem.positions.size(); j++){
					      			if (mip.y[v][i][j] != null && vesselCanSailPositionToPosition(mip.problem.vessels.get(v), mip.problem.positions.get(i), mip.problem.positions.get(j), mip.problem)){
					      				if (fixed.y[v][i][j] >= 0){
					      					mip.y[v][i][j].set(GRB.DoubleAttr. LB, fixed.y[v][i][j]);
					      					mip.y[v][i][j].set(GRB.DoubleAttr. UB, fixed.y[v][i][j]);
					      				}else if (isPositionInPrimaryPeriod(counter, mip.problem.positions.get(j), mip.problem.months)){
					      					mip.y[v][i][j].set(GRB.CharAttr.VType, GRB.BINARY);
					      				}else if (isPositionInForecastingPeriod(counter, numEndForecasting, mip.problem.positions.get(j), mip.problem.months)){
					      					mip.y[v][i][j].set(GRB.CharAttr.VType, GRB.CONTINUOUS);
					      					mip.y[v][i][j].set(GRB.DoubleAttr. UB, 1.0);
					      				}else{
					      					mip.y[v][i][j].set(GRB.DoubleAttr. LB, 0.0);
					      					mip.y[v][i][j].set(GRB.DoubleAttr. UB, 0.0);
					      				}
					      			}					    
					      		}
					    	}
					     }
			
		
			// Remove y_vi-variables from old MIP and rebuild for the rolling horizon
		      for (int v = 0; v < mip.problem.vessels.size(); v++) {
			      	for (int i = 0; i < mip.problem.positions.size(); i++){
			      		if (mip.y_i[v][i] != null && vesselCanSailPosition(mip.problem.vessels.get(v),mip.problem.positions.get(i), mip.problem)){
			      			if (fixed.y_i[v][i] >= 0){
			      				mip.y_i[v][i].set(GRB.DoubleAttr. LB, fixed.y_i[v][i]);
			      				mip.y_i[v][i].set(GRB.DoubleAttr. UB, fixed.y_i[v][i]);
		      				}else if (isPositionInPrimaryPeriod(counter, mip.problem.positions.get(i), mip.problem.months)){
		      					mip.y_i[v][i].set(GRB.CharAttr.VType, GRB.BINARY);
		      				}else if (isPositionInForecastingPeriod(counter, numEndForecasting, mip.problem.positions.get(i), mip.problem.months)){
		      					mip.y_i[v][i].set(GRB.CharAttr.VType, GRB.CONTINUOUS);
			      				mip.y_i[v][i].set(GRB.DoubleAttr. UB, 1.0);
		      				}else{
		      					mip.y_i[v][i].set(GRB.DoubleAttr. LB, 0.0);
			      				mip.y_i[v][i].set(GRB.DoubleAttr. UB, 0.0);
		      				}
			      		}	
			      	}
		      }	

				      
			// Remove y_o-variables from old MIP and rebuild for the rolling horizon
			      for (int v = 0; v < mip.problem.vessels.size(); v++) {
					      	for (int i = 0; i < mip.problem.positions.size(); i++){
					      		if (vesselCanSailInitialPosition(mip.problem.vessels.get(v),mip.problem.positions.get(i))){
				      				if (fixed.y_o[v][i] >= 0){
				      					mip.y_o[v][i].set(GRB.DoubleAttr. LB, fixed.y_o[v][i]);
					      				mip.y_o[v][i].set(GRB.DoubleAttr. UB, fixed.y_o[v][i]);
				      				}else if (isPositionInPrimaryPeriod(counter, mip.problem.positions.get(i), mip.problem.months)){
				      					mip.y_o[v][i].set(GRB.CharAttr.VType, GRB.BINARY);

				      				}else if (isPositionInForecastingPeriod(counter, numEndForecasting, mip.problem.positions.get(i), mip.problem.months)){
				      					mip.y_o[v][i].set(GRB.CharAttr.VType, GRB.CONTINUOUS);
			      				}
				      		}
				      	}		
				  }
				     
		     // Update model
		     mip.model.update();
		     
		     //Solve MIP in current horizon
		     mip.solveMIPinRHH();
		
		     // Transfer the y, y_i and y_o variables to fixed variables
		     fixed.addPrimary(counter, mip.y_i, mip.y_o, mip.y);
		     
		     // Update information about number of dummies and the time it takes to solve each iteration for each horizon and the objective value
		     currentHorizon.addRHHSolution(mip);
		     double itrEndTime = System.currentTimeMillis();
		     currentHorizon.time(itrStartTime, itrEndTime);
		     currentHorizon.objectiveValue = mip.model.get(GRB.DoubleAttr.ObjVal);
		     
		     // Update value of the counter
		     this.counter++;
			}
			
			// Record ending time for the whole RHH
			this.endTime = System.currentTimeMillis();
			
			// printVariables will print the solution with the values of y_i[v][i] to an own file
			//this.printVariables();
			
			// writeRHHtoFile prints creates an OUTPUT-file with the solution in a seperate .txt-file
			this.writeRHHtoFile(mip, horizon, robustnessMethod);
			}

			/* 
			 * ------------------------------------METHODS---------------------------------------
			 */		
	
			/*
			 * ------------------------- Method to print a solution to file -------------------------
			 */

			public void printVariables() throws GRBException{
				
				Writer writer = null;
			
				try {
					writer = new BufferedWriter(new OutputStreamWriter(
			          new FileOutputStream("SolutionRHH"+".txt"), "utf-8"));
			    
			    for (int i = 0; i < this.mip.problem.positions.size(); i++){
			    		if ((this.mip.model.getVarByName("dummy[" + this.mip.problem.positions.get(i).id +"]").get(GRB.DoubleAttr.X)) > 0.5){
						    writer.write("dummy["+ this.mip.problem.positions.get(i).id +"]");		
						    writer.write(System.getProperty( "line.separator" ));
			    		}
					}
			    
			    for (int v = 0; v < this.mip.problem.vessels.size(); v++){
			    	for (int i = 0; i < this.mip.problem.positions.size(); i++){
			    		if (mip.y_i[v][i] != null && vesselCanSailPosition(mip.problem.vessels.get(v),mip.problem.positions.get(i), mip.problem) && mip.y_i[v][i].get(GRB.DoubleAttr.X) > 0.5){
			    			writer.write("y_i[" + this.mip.problem.vessels.get(v).code + "][" + this.mip.problem.positions.get(i).id + "]");	
			    			writer.write(System.getProperty( "line.separator" ));
			    		}	
			    	}
			    }
			    
			    		
				} catch (IOException ex) {
			  // report
				} finally {
			   try {writer.close();} catch (Exception ex) {}
				}
			}
			
			/*
			 * -------------------------------- Method to solve exact-----------------------------------
			 */
			public void solveExact(){
				
				try{
					
					// Set solver parameters
					int solutionTimeLimit = 10000;		// Set how long solver will run
					int solutionLimit = 0;				// Set to 1 to return first feasible solution
					
				      
					
		      // (Solution limit, 1) = Returns first feasible solution found
		      if (solutionLimit > 0){
		      	mip.model.getEnv().set(GRB.IntParam.SolutionLimit, solutionLimit);    
		      }
		      
		      
		      
		      // Set time limit when solving
		      mip.model.getEnv().set(GRB.DoubleParam.TimeLimit, solutionTimeLimit);
		      
		      
		      
		      // Set MIPFocus ( 1 = find good solutions, 2 = prove optimality, 3 = focus on the bound)
		      mip.model.getEnv().set(GRB.IntParam.MIPFocus, 3);

		      
		      
		      // Run model
		      mip.model.optimize();

		      
		      
					// For debugging purposes:
		      int optimstatus = mip.model.get(GRB.IntAttr.Status);
					if (optimstatus == GRB.Status.INFEASIBLE){
						System.out.println("INFEASIBLE");
						mip.model.computeIIS();
						mip.model.write("mymodel.ilp");
					
		      
					}

				}catch(GRBException e) {
		      System.out.println("Error code: " + e.getErrorCode() + ". " +
		          e.getMessage());
				}
			}
	
			
			
			
			/*
			 * ------------------------- Method to determine end of current forecasting period -------------------------
			 */
			public int getCurEndForecasting(int counter, ArrayList<Month> months){
				if (counter + 2 < months.size()){
					return counter+2;
				}
				else if (counter + 1 < months.size()){
					return counter+1;
				}
				else{
					return(months.size()-1);
				}
			}
			
			/*
			 * ------------------------- Method to evaluate if position is in primary -------------------------
			 */
			public boolean isPositionInPrimaryPeriod(int counter, Position i, ArrayList<Month> months){
				if (i.endTime <= months.get(counter).end && i.endTime >= months.get(counter).start){
					return true;
				}
				else{
					return false;
				}
			}
			
			/*
			 * ------------------------- Method to evaluate if position is in forecasting -------------------------
			 */
			public boolean isPositionInForecastingPeriod(int counter, int numEndForecasting, Position i, ArrayList<Month> months){
				if (i.endTime <= months.get(numEndForecasting).end){
					return true;
				}
				else{
					return false;
				}
			}
			/*
			 * ------------------------- Method to evaluate if a vessel can sail from a voyage to another voyage -------------------------
			 */

			public static boolean vesselCanSailPositionToPosition(Vessel v, Position i, Position j, Problem problem){
				
				if (
						i.endTime + GlobalConstants.maxDaysDelayed >= v.dateOffset
						&&
						i != j
						&&
						i.startTime + Sailing.getShortestSailingDuration(v.code, i.route, problem.sailings) + SailingBallast.getShortestSailingBallastDuration(v.code, i.route, j.route, problem.sailingBallasts) <= j.endTime + GlobalConstants.maxDaysDelayed
						){
					return true;
				}
				else{
					return false;
				}
			}
			  /*
			   *  -------------------- Method to evaluate if a vessel can sail a position as its first scheduled position -----------------------------
			   */
				
				public boolean vesselCanSailInitialPosition(Vessel v, Position i){		
					if (i.endTime + GlobalConstants.maxDaysDelayed >= v.dateOffset
							+ SailingBallastStart.getShortestSailingBallastStartDuration(v.code, i.route, mip.problem.sailingBallastStarts)	
							
							&&
							
							SailingBallastStart.getShortestSailingBallastStartDuration(v.code, i.route, mip.problem.sailingBallastStarts) >= 0
							
							&&
							
							Sailing.getShortestSailingDuration(v.code, i.route, mip.problem.sailings) >= 0

						){
							return true;
					}else{
							return false;
					}
				}
				
			/*
			 * ------------------------- Method to evaluate if a vessel can sail a voyage ------------------------------------------------
			 */

			public static boolean vesselCanSailPosition(Vessel v, Position i, Problem problem){
				if (i.endTime + GlobalConstants.maxDaysDelayed >= v.dateOffset){
					return true;
				}
				else{
					return false;
				}
			}
			
			/*
			 * -------------------------------- METHOD TO WRITE RHH RESULT TO FILE ----------------------------------------------
			 */
			
			public void writeRHHtoFile(Mip mip, ArrayList<Horizon> horizon, String robustnessMethod){
				
				Writer writer = null;
				DecimalFormat formatter = new DecimalFormat("#");
				try {
				    writer = new BufferedWriter(new OutputStreamWriter(
				          new FileOutputStream(GlobalConstants.rhhOutputDirectory+"RHH_OUTPUT_"+robustnessMethod+"_" + TestInstances.testInstances.get(this.currentInstance) +".txt"), "utf-8"));
				    writer.write("OUTPUT FROM SOLVING " + this.mip.problem.instanceName + " WITH RHH");
				    writer.write(System.getProperty( "line.separator" ));
				    writer.write("-------------------------------------------------------");
				    writer.write(System.getProperty( "line.separator" ));
				    writer.write("TOTAL: ");
				    writer.write(System.getProperty( "line.separator" ));
				    writer.write(String.format("%-20s", "Primary periods"));
				    writer.write(String.format("%-20s", "Forecasting period"));
				    writer.write(String.format("%-30s", "Start time of RHH"));
				    writer.write(String.format("%-20s", "Duration of RHH"));
				    writer.write(String.format("%-40s", "Number of unserviced voyages"));
				    writer.write(String.format("%-20s", "Objective value"));
		    		writer.write("\n");
		    		writer.write(String.format("%-20d", this.horizon.get(this.horizon.size()-1).primaryPeriod));
				    		writer.write(String.format("%-20s", "2 months"));
				    		writer.write(String.format("%-30s", (new Date()).toString()));
				    		writer.write(String.format("%-20f", (endTime - startTime)/1000));
				    		writer.write(String.format("%-40d", this.horizon.get(this.horizon.size()-1).numberOfDummies));
				    		try {
								writer.write(String.format("%-20s", formatter.format(this.mip.model.get(GRB.DoubleAttr.ObjVal))));
							} catch (GRBException e) {
								e.printStackTrace();
							}
				    writer.write(System.getProperty( "line.separator" ));
				    writer.write(System.getProperty( "line.separator" ));
				    writer.write("KEY INFORMATION FOR EACH HORIZON: ");
				    writer.write(System.getProperty( "line.separator" ));
				    writer.write(String.format("%-20s", "Primary Period"));
				    writer.write(String.format("%-20s", "Forecasting period"));
				    writer.write(String.format("%-30s", "Start time of iteration"));
				    writer.write(String.format("%-20s", "Iteration time"));
				    writer.write(String.format("%-40s", "Number of unserviced voyages"));
				    writer.write(String.format("%-20s", "Objective value"));
		    		writer.write("\n");	
				    	for (int h = 0; h < this.horizon.size(); h++){
				    		writer.write(String.format("%-20d", this.horizon.get(h).primaryPeriod));
				    		writer.write(String.format("%-20d", this.horizon.get(h).forecastingPeriod));
				    		writer.write(String.format("%-30f", (this.horizon.get(h).itrStartTime - startTime)/1000));
				    		writer.write(String.format("%-20f", (this.horizon.get(h).iterationTime)/1000));
				    		writer.write(String.format("%-40d", this.horizon.get(h).numberOfDummies));
				    		writer.write(String.format("%-20e", this.horizon.get(h).objectiveValue));
				    		writer.write(System.getProperty( "line.separator" ));
				    	}
				    
				    writer.write(System.getProperty( "line.separator" ));
				    writer.write("-------------------------------------------------------");
				    writer.write(System.getProperty( "line.separator" ));
				    writer.write("VOYAGES COVERED BY DUMMY VARIABLES:" );
				    writer.write(System.getProperty( "line.separator" ));
				    	int dummies = 0;
				    	for (int i = 0; i < mip.problem.positions.size(); i++){
							try {
								if ((this.mip.model.getVarByName("dummy[" + this.mip.problem.positions.get(i).id +"]").get(GRB.DoubleAttr.X)) > 0.5){
								writer.write("" + this.mip.problem.positions.get(i).id);
								writer.write(System.getProperty( "line.separator" ));
								dummies++;
								}
							} catch (GRBException e) {
								e.printStackTrace();
							}
						}
					writer.write("TOTAL NUMBER OF UNSERVICED VOYAGES: " + dummies);
					writer.write(System.getProperty( "line.separator" ));
					writer.write("-------------------------------------------------------");
				    writer.write(System.getProperty( "line.separator" ));
				    writer.write("VOYAGE ASSIGNMENTS: ");
				    writer.write(System.getProperty( "line.separator" ));
						for (int v = 0; v < mip.problem.vessels.size();v++){
					    writer.write("VESSEL " + mip.problem.vessels.get(v).code + " SAILS:");
					    writer.write(System.getProperty( "line.separator" ));
							for (int i = 0; i < mip.problem.positions.size(); i++){
								try {
									if (mip.y_i[v][i] != null && vesselCanSailPosition(mip.problem.vessels.get(v),mip.problem.positions.get(i), mip.problem) && mip.y_i[v][i].get(GRB.DoubleAttr.X) > 0.5){
										writer.write("" + this.mip.problem.positions.get(i).id);
										writer.write(System.getProperty( "line.separator" ));
									}
								} catch (GRBException e) {
									e.printStackTrace();
								}
							}
						}
				    
				} catch (IOException ex) {
				  // report
				} finally {
				   try {writer.close();} catch (Exception ex) {}
				}
				

			}

		
}