import java.util.ArrayList;
import java.util.Comparator;

import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;


public class Position{
  public String id;
  public String route;
  public String month;
  public int startTime;
  public int dateOffset;
  public int endTime;
  public boolean isMaintenance;
  
  // For use in Solution object
  public double assignedStartTime;

  
  public Position(String aa, String bb, String cc, int dd, int ee, int ff, boolean gg) {
    id = aa;
    route = bb;
    month = cc;
    startTime = dd;
    dateOffset = ee;
    endTime = ff;
    isMaintenance = gg;
  	}
  
  
  public boolean isMaintenance(Position position){
  	if (position.isMaintenance == true){
  	return true;
  	}else{
  		return false;
  	}
  }
 
 
	
  
  /*
   * ----------------- Method to find a possible assignment for a position in a vessel schedule ----------------
   */
  
  public boolean canBeAssignedTo(Vessel v, Solution solution){
  	
  	boolean possible = false;
  	
  	// If no other positions have been assigned to this vessel
  	if (v.assignedPositions.size() == 0){
  		if (solution.problem.vesselCanSailInitialPosition(v, this)){
  			v.assignedPositions.add(this);
  			return true;
  		}else{
  			return false;
  		}
  		
  		
  	// If other positions have been assigned to this vessel
  	}else{
  		for (int i = 0; i < v.assignedPositions.size() + 1; i++){ 			
  			if (i == 0){
  				
  				// Check if the position can be inserted as the first position
  				if (v.initialInsertionIsPossible(this, solution)){
  					
  					//TEST
  					System.out.println("SUCCESSFUL INITIAL INSERTION");
  					return true;
  				}
  			}
  			
  			else if ( i < v.assignedPositions.size()){
  				
    			// Check if the position can be inserted in between positions
  				if (v.inBetweenInsertionIsPossible(this, i, solution)){
  				
  					//TEST
  					System.out.println("SUCCESSFUL IN BETWEEN INSERTION");
  					return true;
  				}
  			}
  			
  			else{
  				
  				// Check if the position can be inserted as the last position
  				if (v.lastInsertionIsPossible(this, solution)){
  					
  				//TEST
  					System.out.println("SUCCESSFUL LAST INSERTION");
  					return true;
  				}
  			}
  					
  		}
  	}
  	
  	
  	
  	return possible;
  }
  
  
  
  /*
   * ------------------------------ Method to check if a position can be added to a schedule as an initial position --------------------------
   */
  
  public boolean canBeAddedAsInitialPosition(Vessel v, Solution solution){
  	
  	// Check if the arc from this voyage exists to the first voyage in the vessel's schedule
  	if (
  			solution.problem.vesselCanSailInitialPosition(v, this)
  			&&	
  			solution.problem.vesselCanSailPositionToPosition(v, this, v.assignedPositions.get(0))
  			){
  	
  		
  		// The sailing is possible. Now evaluate if the remaining schedule will be feasible.
	  	v.assignedPositions.add(0, this);
	  	int i = 0;
	  	double[] startTimes = new double[v.assignedPositions.size()];
	  	startTimes[0] = this.getStartTimeInitialPosition(v, solution);
	  	  	
	  	while (i < (v.assignedPositions.size()-1)){
	  		
	  		if (
	  				startTimes[i]
	  			+ Sailing.getShortestSailingDuration(v.code, v.assignedPositions.get(i).route, solution.problem.sailings)
	  			+ SailingBallast.getShortestSailingBallastDuration(v.code, v.assignedPositions.get(i).route, v.assignedPositions.get(i+1).route, solution.problem.sailingBallasts)
	  				<=
	  				v.assignedPositions.get(i+1).startTime){
	  			
	  			startTimes[i+1] = v.assignedPositions.get(i+1).startTime;
	  			
	  		}else if (
	  				startTimes[i]
	  			+ Sailing.getShortestSailingDuration(v.code, v.assignedPositions.get(i).route, solution.problem.sailings)
	  			+ SailingBallast.getShortestSailingBallastDuration(v.code, v.assignedPositions.get(i).route, v.assignedPositions.get(i).route, solution.problem.sailingBallasts)
	  				<=
	  				v.assignedPositions.get(i+1).endTime + GlobalConstants.maxDaysDelayed){
	  			
	  			startTimes[i+1] = startTimes[i]
	    										+ Sailing.getShortestSailingDuration(v.code, v.assignedPositions.get(i).route, solution.problem.sailings)
	    										+ SailingBallast.getShortestSailingBallastDuration(v.code, v.assignedPositions.get(i).route, v.assignedPositions.get(i+1).route, solution.problem.sailingBallasts); 
	  			
	  		}else{
	  			
	  			// The insertion is infeasible.
	  			v.assignedPositions.remove(0);
	  			return false;
	  		}
	  		
	  		i++; 		
	  	} // End while
  	 	
	  	// The arc exists and the insertion is feasible
			v.assignedPositions.remove(0);
	  	return true;
	  	
  	}else{
  		
  		// The arc does not exist.
  		return false;
  		
  	}
  }
  
  
  
  
  /*
	 * ------------------ Method to check if a position can be assigned to a vessel in a given sequence in the schedule ------------
	 */
	
	public boolean canBeAddedInSequence(Vessel v, int positionInSchedule, Solution solution){
		
		boolean arcExists = false;
		
		// Check if the arcs defined by the requested insertion exist in the network
		if (positionInSchedule == 0){
			
			// We have an initial insertion
			if (v.assignedPositions.size() > 0.5){
				if (
		  			solution.problem.vesselCanSailInitialPosition(v, this)
		  			&&	
		  			solution.problem.vesselCanSailPositionToPosition(v, this, v.assignedPositions.get(0))
		  			){
					arcExists = true;
				}else{
					return false;
				}
			}else{
				if (
						solution.problem.vesselCanSailInitialPosition(v, this)
						){
					arcExists = true;
				}else{
					return false;
				}
			}
				
		}else if (positionInSchedule > 0 && positionInSchedule < v.assignedPositions.size()){
			
			// We have an in-between insertion
			if (
					solution.problem.vesselCanSailPositionToPosition(v, v.assignedPositions.get(positionInSchedule - 1), this)
					&&
					solution.problem.vesselCanSailPositionToPosition(v, this, v.assignedPositions.get(positionInSchedule))
					){
				arcExists = true;	
			}else{
				return false;
			}
			
		}else if (positionInSchedule == v.assignedPositions.size()){
			
			// We have a final insertion
			if (
					solution.problem.vesselCanSailPositionToPosition(v, v.assignedPositions.get(positionInSchedule - 1), this)
					){
				arcExists = true;
			}else{
				return false;
			}
		}
		
		
		// If the arc exists, do the insertion and see if the schedule remains feasible. If the arc does not exist, return false.
		if (arcExists){

			// The sailing is possible. Now evaluate if the remaining schedule will be feasible.
	  	v.assignedPositions.add(positionInSchedule, this);	
	  	int i = 0;
	  	double[] startTimes = new double[v.assignedPositions.size()];
	  	startTimes[0] = v.assignedPositions.get(0).getStartTimeInitialPosition(v, solution);
	  	  	
	 
	  	while (i < (v.assignedPositions.size()-1)){
	  		
	  		if (
	  				startTimes[i]
	  			+ Sailing.getShortestSailingDuration(v.code, v.assignedPositions.get(i).route, solution.problem.sailings)
	  			+ SailingBallast.getShortestSailingBallastDuration(v.code, v.assignedPositions.get(i).route, v.assignedPositions.get(i+1).route, solution.problem.sailingBallasts)
	  				<=
	  				v.assignedPositions.get(i+1).startTime){
	  			
	  			startTimes[i+1] = v.assignedPositions.get(i+1).startTime;
	  			
	  		}else if (
	  				startTimes[i]
	  			+ Sailing.getShortestSailingDuration(v.code, v.assignedPositions.get(i).route, solution.problem.sailings)
	  			+ SailingBallast.getShortestSailingBallastDuration(v.code, v.assignedPositions.get(i).route, v.assignedPositions.get(i+1).route, solution.problem.sailingBallasts)
	  				<=
	  				v.assignedPositions.get(i+1).endTime + GlobalConstants.maxDaysDelayed){
	  			
	  			startTimes[i+1] = startTimes[i]
	    										+ Sailing.getShortestSailingDuration(v.code, v.assignedPositions.get(i).route, solution.problem.sailings)
	    										+ SailingBallast.getShortestSailingBallastDuration(v.code, v.assignedPositions.get(i).route, v.assignedPositions.get(i+1).route, solution.problem.sailingBallasts); 
	  			
	  		}else{
	  			
	  			// The insertion is infeasible.
	  			v.assignedPositions.remove(positionInSchedule);
	  			return false;
	  		}
	  		
	  		i++; 		
	  	} // End while
	  	
	  	
	  	// The arc exists and the insertion is found feasible
			v.assignedPositions.remove(positionInSchedule);
	  	return true;
	  	
  	}else{
  		
  		// The arc does not exist.
  		return false;
  		
  	}
	}
	
  

  /*
   * ------------------------------ Method to check if a position can be added to a schedule --------------------------
   */
  
  public boolean canBeAdded(Vessel v, int positionInSequence, Solution solution){
  	
  	//TEST
  	System.out.println("ATTEMPT TO ADD [" + this.id + "] TO VESSEL [" + v.code + "]'S SCHEDULE AT POSITION " + positionInSequence); 	
  	v.printSchedule();
  	
  	
  	v.assignedPositions.add(positionInSequence, this);
  	
  	//TEST
  	v.printSchedule();

  	
  	boolean schedulePossible = true;
  	int i = 0;
  	double[] startTimes = new double[v.assignedPositions.size()];
  	startTimes[0] = RepairOperator.getStartTimeInitialPosition(v, v.assignedPositions.get(0), solution);

  	
  	while (schedulePossible && i < (v.assignedPositions.size()-1)){
  		
  		if (
  				startTimes[i]
  			+ Sailing.getShortestSailingDuration(v.code, v.assignedPositions.get(i).route, solution.problem.sailings)
  			+ SailingBallast.getShortestSailingBallastDuration(v.code, v.assignedPositions.get(i).route, v.assignedPositions.get(i+1).route, solution.problem.sailingBallasts)
  				<=
  				v.assignedPositions.get(i+1).startTime){
  			
  			startTimes[i+1] = v.assignedPositions.get(i+1).startTime;
  			
  		}else if(
  				startTimes[i]
  			+ Sailing.getShortestSailingDuration(v.code, v.assignedPositions.get(i).route, solution.problem.sailings)
  			+ SailingBallast.getShortestSailingBallastDuration(v.code, v.assignedPositions.get(i).route, v.assignedPositions.get(i+1).route, solution.problem.sailingBallasts)
  				<=
  				v.assignedPositions.get(i+1).endTime + GlobalConstants.maxDaysDelayed){
  			
  			startTimes[i+1] = startTimes[i]
    												+ Sailing.getShortestSailingDuration(v.code, v.assignedPositions.get(i).route, solution.problem.sailings)
    												+ SailingBallast.getShortestSailingBallastDuration(v.code, v.assignedPositions.get(i).route, v.assignedPositions.get(i+1).route, solution.problem.sailingBallasts); 
  			
  		}else{
  			schedulePossible = false;
  			
  			//TEST
  	  	v.printSchedule();

  	  	System.out.println("ATTEMPT FAILED, REMOVED [" + v.assignedPositions.get(positionInSequence).id + "] FROM VESSEL [" + v.code + "]'S SCHEDULE");

  			v.assignedPositions.remove(positionInSequence);
  			
  			
  			//TEST
  	  	v.printSchedule();

  		}
  		
  		i++;
  	} // End while
  	 	
  	return schedulePossible;
  }
  
  
  
  /*
   * ----------------------- Method to find possible swaps for a position ------------------------
   */
  
  public boolean swapsArePossible(Solution solution, Vessel vessel, ArrayList<Position> possibleSwaps, int sequence){
  	
  	boolean swapsFound = false;
  	
  	// Iterate all vessels (except the one the position is assigned to) and find possible swaps with this position
  	for (int v = 0; v < solution.vessels.size(); v++){
  		
  		if (solution.vessels.get(v) != vessel){
  			
  			for (int i = 0; i < solution.vessels.get(v).assignedPositions.size(); i++){
  				
  				// Check if a swap between this position and the assigned position is possible
  				if (this.canBeSwappedWith(sequence, i, vessel, solution.vessels.get(v), solution)){
  					possibleSwaps.add(solution.vessels.get(v).assignedPositions.get(i));
  					swapsFound = true;
  					
  					//TEST
						//System.out.println(this.id + " CAN BE SWAPPED WITH " + solution.vessels.get(v).assignedPositions.get(i).id);
  					
  				}
  				
  			} // End for all i
  		}
  		
  	} // End for all v
  	
  	return swapsFound;
  }

  
  
  /*
   * -------------- Method to check if a swap is possible ------------------
   */
  
	public boolean canBeSwappedWith(int i1, int i2, Vessel v1, Vessel v2, Solution solution){
		
		Position j = v2.assignedPositions.get(i2);
		Position i = this;
		v2.assignedPositions.remove(i2);
		v1.assignedPositions.remove(i1);
		
		if (i.canBeAddedInSequence(v2, i2, solution) && j.canBeAddedInSequence(v1, i1, solution)){
			
			// The swap is possible
			v2.assignedPositions.add(i2, j);
			v1.assignedPositions.add(i1, i);
			return true;
		}else{
			
			// The swap is not possible
			v2.assignedPositions.add(i2, j);
			v1.assignedPositions.add(i1, i);
			return false;
		}	
	}

  
  
  /*
	 * ---------------- Method to calculate earliest possible start time for this voyage inserted as first voyage in a schedule ----------------
	 */

  public double getStartTimeInitialPosition(Vessel v, Solution solution){
  	if (v.dateOffset +
			SailingBallastStart.getShortestSailingBallastStartDuration(v.code, this.route, solution.problem.sailingBallastStarts)
			<=
			this.startTime){
  		return this.startTime;
  	}else{
  		return (v.dateOffset + SailingBallastStart.getShortestSailingBallastStartDuration(v.code, this.route, solution.problem.sailingBallastStarts));
  	}	
  }

  
  
  /*
	 * ---------------- Method to calculate earliest possible start time for a voyage insertion after a voyage ----------------
	 */
	
	public double getStartTimeAfterPosition(Vessel v, Position existingPosition, Solution solution){
		if (existingPosition.assignedStartTime +
				Sailing.getShortestSailingDuration(v.code, existingPosition.route, solution.problem.sailings) +
				SailingBallast.getShortestSailingBallastDuration(v.code, existingPosition.route, this.route, solution.problem.sailingBallasts)
				<=
				this.startTime){
			return 
				this.startTime;
		}else{
			return (existingPosition.assignedStartTime +
							Sailing.getShortestSailingDuration(v.code, existingPosition.route, solution.problem.sailings) +
							SailingBallast.getShortestSailingBallastDuration(v.code, existingPosition.route, this.route, solution.problem.sailingBallasts));
		}	
	}


  
  
  /*
   * ------------------------------- Method to retrieve latest possible start of voyage --------------------------------
   */
  
  public static double getLatestVoyageStart(ArrayList<Position> positions){
	 double latestStart = -1.00;
	 
	 for (int n = 0; n < positions.size() ; n++){
		 if (positions.get(n).endTime > latestStart){
			 latestStart = positions.get(n).endTime;
		 }
	 }
	 return latestStart;
  }

 
  
  public double getLateStart(){
  	return this.endTime;
  }
  
  public double getDateOffset(){
  	return this.dateOffset;
  }
  
  public double getAssignedStartTime(){
  	return this.assignedStartTime;
  }
  
  
  /* Comparator for sorting a list of Positions by endTime */
  public static Comparator<Position> PositionLateStartComparator = new Comparator<Position>(){
  	
  	public int compare(Position p1, Position p2){
  		
  		double lateStart1 = p1.getLateStart();
  		double lateStart2 = p2.getLateStart();
  		
  		return (int)(lateStart1-lateStart2);
  	}
  };
  
  /* Comparator for sorting a list of Positions by DateOffset */
  public static Comparator<Position> PositionDateOffsetComparator = new Comparator<Position>(){
  	
  	public int compare(Position p1, Position p2){
  		
  		double dateOffset1 = p1.getLateStart();
  		double dateOffset2 = p2.getLateStart();
  		
  		return (int)(dateOffset1-dateOffset2);
  	}
  };
  
  
 public static Comparator<Position> PositionAssignedStartTimeComparator = new Comparator<Position>(){
  	
  	public int compare(Position p1, Position p2){
  		
  		double assignedStartTime1 = p1.getAssignedStartTime();
  		double assignedStartTime2 = p2.getAssignedStartTime();
  		
  		return (int)(assignedStartTime1-assignedStartTime2);
  	}
  };
 
  
  public static ArrayList<Position> importPositions(HSSFWorkbook workbook){
  	HSSFSheet positionSheet = workbook.getSheet("Position");
  	int numberOfPositions = positionSheet.getLastRowNum();
		
		ArrayList<Position> positions = new ArrayList<Position>();
		
		//System.out.println(numberOfPositions);
		
		for (int rowNum = 1; rowNum <= numberOfPositions; rowNum++){
			Row row = positionSheet.getRow(rowNum);
			Cell c = row.getCell(0);
			
			if (c != null){
			Cell positionId = row.getCell(0);
		  positionId.setCellType(Cell.CELL_TYPE_STRING);
		  Cell positionRoute = row.getCell(1);
		  positionRoute.setCellType(Cell.CELL_TYPE_STRING);
		  Cell positionMonth = row.getCell(2);
		  positionMonth.setCellType(Cell.CELL_TYPE_STRING);
		  Cell positionStartTime = row.getCell(3);
		  Cell positionDateOffset = row.getCell(4);
		  Cell positionEndTime = row.getCell(5);
		  Cell positionIsMaintenance = row.getCell(6);
		  positionIsMaintenance.setCellType(Cell.CELL_TYPE_BOOLEAN);
		  
		  positions.add(new Position(
					(String)positionId.getStringCellValue(),
					(String)positionRoute.getStringCellValue(),
					(String)positionMonth.getStringCellValue(),
					(int)positionStartTime.getNumericCellValue(),
					(int)positionDateOffset.getNumericCellValue(),
					(int)positionEndTime.getNumericCellValue(),
					(Boolean)positionIsMaintenance.getBooleanCellValue()));
		  
				}
		 	}
		return positions;
  }
  public int getStartTime(){
	  return this.startTime;
  }
  public int getEndTime(){
	  return this.endTime;
  }
//Assums that starttime represents the day number og the position's startime. this needs to be verified by looking at excel sheets
	public boolean positionIsInMonth(Month mon){
		if ((mon.getStart() <= this.startTime) && (this.startTime  <= mon.getEnd())){
			return true;
		}
		else{
			return false;
		}
	}

}
