import java.util.ArrayList;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;


public class Vessel {
  	public String code;
  	public String name;
  	public int dateOffset;
  	
  	// For use in Solution object
  	public ArrayList<Position> assignedPositions;
  
  	
  	public Vessel(String aa, String bb, int cc) {
    code = aa;
    name = bb;
    dateOffset = cc;
  	}
  
  	
  	
  	

  	
  	/*
  	 * ------------------- Method to check if a position can be removed from a vessel's schedule ----------------
  	 */
  	
  	public boolean positionCanBeRemoved(int position, Problem problem){
  		int numberOfPositions = this.assignedPositions.size();
  		boolean removalIsPossible = false;
  		
  		
  		if (numberOfPositions == 1){
  			removalIsPossible = true;
  		}
  		
  		else if (
  				numberOfPositions >= 2 
  				&& 
  				position == 0 
  				&& 
  				problem.vesselCanSailInitialPosition(this, this.assignedPositions.get(1))
  				){
  			removalIsPossible = true;
  		}
  		
  		else if (
  				numberOfPositions >= 3
  				&&
  				position < (numberOfPositions-1)
  				&&
  				position > 0
  				&&
  				problem.vesselCanSailPositionToPosition(this, this.assignedPositions.get(position-1), this.assignedPositions.get(position+1))
  				){
  			removalIsPossible = true;
  		}
  		
  		else if (
  				numberOfPositions >= 2
  				&&
  				position == (numberOfPositions-1)
  				){
  			removalIsPossible = true;
  		}		
  		
  		return removalIsPossible;
  	}
  	
  	
  	
  	
  	/*
  	 * ------------------- Method to check if a set of positions can be removed from a vessel's schedule ----------------
  	 */
  	
  	public boolean connectedPositionsCanBeRemoved(int positionStart, int positionEnd, Problem problem){
  		int numberOfPositions = this.assignedPositions.size();
  		int numberOfPositionsToDestroy = (positionEnd-positionStart) + 1;
  		
  		if (numberOfPositions == numberOfPositionsToDestroy){
  			return true;
  		}
  		
  		else if (
  				numberOfPositions >= numberOfPositionsToDestroy + 1 
  				&& 
  				positionStart == 0 
  				&& 
  				problem.vesselCanSailInitialPosition(this, this.assignedPositions.get(numberOfPositionsToDestroy))
  				){
  			return true;
  		}
  		
  		else if (
  				numberOfPositions >= numberOfPositionsToDestroy + 2
  				&&
  				positionEnd < (numberOfPositions-1)
  				&&
  				positionStart > 0
  				&&
  				problem.vesselCanSailPositionToPosition(this, this.assignedPositions.get(positionStart-1), this.assignedPositions.get(positionEnd+1))
  				){
  			return true;
  		}
  		
  		else if (
  				numberOfPositions >= numberOfPositionsToDestroy + 1
  				&&
  				positionEnd == (numberOfPositions-1)
  				){
  			return true;
  		}		
  		
  		return false;
  	}
  	
  	
  	
  	/*
  	 * ------------------ Method to check if a position can be assigned to a vessel as the first position ------------
  	 */
  	
  	public boolean initialInsertionIsPossible(Position voyage, Solution solution){
  		
  		if (
  				solution.problem.vesselCanSailInitialPosition(this, voyage)	
  				&&	
  				solution.problem.vesselCanSailPositionToPosition(this, voyage, this.assignedPositions.get(0))
  				){
  			
  			// The arc exists in the network, now find the earliest possible start time for the voyage
  			voyage.assignedStartTime = RepairOperator.getStartTimeInitialPosition(this, voyage, solution);
  			
  			// Check if this start time makes the remaining schedule feasible
  			if (voyage.canBeAddedAsInitialPosition(this, solution)){
  				return true;
  			}else{
  				return false;
  			}
  		}else{
  			return false;
  		}
  	}
  	
  	
  	
  	/*
  	 * ------------------ Method to check if a position can be assigned to a vessel in a given sequence in the schedule ------------
  	 */
  	
  	public boolean inBetweenInsertionIsPossible(Position voyage, int positionInSchedule, Solution solution){
  		
  		if (
  				solution.problem.vesselCanSailPositionToPosition(this, this.assignedPositions.get(positionInSchedule - 1), voyage)
  				&&
  				solution.problem.vesselCanSailPositionToPosition(this, voyage, this.assignedPositions.get(positionInSchedule))
  				){
  			
  			// The arc exists in the network. Now check if this is a possible schedule
  			if (voyage.canBeAdded(this, positionInSchedule, solution)){
  				return true;
  			}else{
  				return false;
  			}
  		}else{
  			return false;
  		}
  	}
  	
  	
  	
  	/*
  	 * ------------------ Method to check if a position can be assigned to a vessel as the last voyage in a schedule ------------
  	 */
  	
  	public boolean lastInsertionIsPossible(Position voyage, Solution solution){
  		
  		if (
  				solution.problem.vesselCanSailPositionToPosition(this, this.assignedPositions.get(this.assignedPositions.size()-1), voyage)
  				){
  			
  			// The arc exists in the network. Now check if this is a possible schedule
  			if (voyage.canBeAdded(this, this.assignedPositions.size(), solution)){
  				return true;
  			}else{
  				return false;
  			}
  		}else{
  			return false;
  		}
  	}
  	
  	
  	
  	/*
  	 * ------------------ Method to assign start times in a vessel schedule ----------------
  	 */
  	
  	
		public void assignStartTimes(Solution solution){
			
			if (!this.assignedPositions.isEmpty()){
				this.assignedPositions.get(0).assignedStartTime = this.assignedPositions.get(0).getStartTimeInitialPosition(this, solution);		
				for (int i = 1; i < this.assignedPositions.size(); i++){
					this.assignedPositions.get(i).assignedStartTime = this.assignedPositions.get(i).getStartTimeAfterPosition(this, this.assignedPositions.get(i-1),  solution);
				}		
			}
		}

  	
		
		/*
		 * ----------------- Method to print a vessel schedule ------------
		 */
		
		public void printSchedule(){
			String schedule = "";
			
			schedule += "[" + this.code + "]";
			for (int i = 0; i < this.assignedPositions.size(); i++){
				schedule += "[" + this.assignedPositions.get(i).id + "]";
			}
			
			System.out.println(schedule);
		}
  	
		
		
		
		
  	public static ArrayList<Vessel> importVessels(HSSFWorkbook workbook){
  		HSSFSheet vesselSheet = workbook.getSheet("Vessel");
  		int numberOfVessels = vesselSheet.getLastRowNum();
		
  		ArrayList<Vessel> vessels = new ArrayList<Vessel>();
				
  		for (int rowNum = 1; rowNum <= numberOfVessels; rowNum++){
  			Row row = vesselSheet.getRow(rowNum);
  			Cell c = row.getCell(0);
			
  			if (c != null){
  				Cell vesselCode = row.getCell(0);
  				vesselCode.setCellType(Cell.CELL_TYPE_STRING);
  				Cell vesselName = row.getCell(1);
  				vesselName.setCellType(Cell.CELL_TYPE_STRING);
					Cell vesselDateOffset = row.getCell(2);
		  
					vessels.add(new Vessel(
							(String)vesselCode.getStringCellValue(),
							(String)vesselName.getStringCellValue(),
							(int)vesselDateOffset.getNumericCellValue()));
  			}
  		}
  		return vessels;
  	}
  
  	
  	public String getVesselCode(Vessel vessel){
  		String code = vessel.code;
  		return code;
  	}
  	
  	
  	/*
  	 * -------------- Method that attempts to destroy a given position in a vessel schedule. Returns the number of positions destroyed --------
  	 */
  	
  	public int destroyPosition(int i, Solution solution){
  		  		
  		int size = this.assignedPositions.size();
  		
  		if (this.positionCanBeRemoved(i, solution.problem)){
  			solution.unassignedPositions.add(this.assignedPositions.get(i));
				this.assignedPositions.remove(i);
				
				return 1;
				
  		}else if (i-1 >= 0 && this.connectedPositionsCanBeRemoved(i-1, i, solution.problem)){
  			solution.unassignedPositions.add(this.assignedPositions.get(i));
				this.assignedPositions.remove(i);

  			solution.unassignedPositions.add(this.assignedPositions.get(i-1));
				this.assignedPositions.remove(i-1);
				
				return 2;

  			// DESTROY AS LITTLE AS POSSIBLE ON VESSEL V. ONE POSITION AT A TIME, CHECK EACH TIME IF THE SCHEDULE REMAINS FEASIBLE. METHOD RETURNS DESTROYED POSITIONS
  			
  		}else if (i+1 < size && this.connectedPositionsCanBeRemoved(i, i+1, solution.problem)){
  			solution.unassignedPositions.add(this.assignedPositions.get(i+1));
				this.assignedPositions.remove(i+1);

  			solution.unassignedPositions.add(this.assignedPositions.get(i));
				this.assignedPositions.remove(i);
  			
				return 2;

  		}else if (i-1 >= 0 && i+1 < size && this.connectedPositionsCanBeRemoved(i-1, i+1, solution.problem)){
  			solution.unassignedPositions.add(this.assignedPositions.get(i+1));
				this.assignedPositions.remove(i+1);

  			solution.unassignedPositions.add(this.assignedPositions.get(i));
				this.assignedPositions.remove(i);
				
				solution.unassignedPositions.add(this.assignedPositions.get(i-1));
				this.assignedPositions.remove(i-1);
				
				return 3;

  		}else if (i-2 >= 0 && this.connectedPositionsCanBeRemoved(i-2, i, solution.problem)){			
  			solution.unassignedPositions.add(this.assignedPositions.get(i));
				this.assignedPositions.remove(i);

  			solution.unassignedPositions.add(this.assignedPositions.get(i-1));
				this.assignedPositions.remove(i-1);
				
				solution.unassignedPositions.add(this.assignedPositions.get(i-2));
				this.assignedPositions.remove(i-2);
				
				return 3;

  		}else if (i+2 < size && this.connectedPositionsCanBeRemoved(i, i+2, solution.problem)){
  			solution.unassignedPositions.add(this.assignedPositions.get(i+2));
				this.assignedPositions.remove(i+2);

  			solution.unassignedPositions.add(this.assignedPositions.get(i+1));
				this.assignedPositions.remove(i+1);
				
				solution.unassignedPositions.add(this.assignedPositions.get(i));
				this.assignedPositions.remove(i);
				
				return 3;

  		}else{
  			this.destroyVessel(solution);
				return size;
  			
  		}
  		
  	}
  	
  	
  	
  	/*
  	 * -------------- Method to destroy all positions assigned to a vessel --------------
  	 */
  	
  	public int destroyVessel(Solution solution){
  		  		
  		int positions = this.assignedPositions.size();
  		
  		solution.unassignedPositions.addAll(this.assignedPositions);
  		
  		this.assignedPositions.clear();
  		
  		return positions;
  	}
  	
}