import java.util.ArrayList;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;


public class SailingBallastStart {
	public String code;
  public String vessel;
  public String route;
  public String speedProfileCode;
  public double duration;
  public double costs;

  public SailingBallastStart(String aa, String bb, String cc, String dd, double ee, double ff) {
    code = aa;
    vessel = bb;
    route = cc;
    speedProfileCode = dd;
    duration = ee;
    costs = ff;
  }
  
  
  /*
   * --------------------------- Method to retrieve costs of sailing ballast start at a given speed ------------------------------
   */
  
  public static double getStartCosts(String vessel, String route, String speedProfileCode, ArrayList<SailingBallastStart> sailingBallastStarts){
  	double costs = -1.00;
  	
  	for (int n = 0; n < sailingBallastStarts.size(); n++){
  		if (vessel.equals(sailingBallastStarts.get(n).vessel) && route.equals(sailingBallastStarts.get(n).route) && speedProfileCode.equals(sailingBallastStarts.get(n).speedProfileCode)){
  			costs = sailingBallastStarts.get(n).costs;
  		}
  	}
  	
  	/*
  	if (costs < 0){
  		System.out.println("ERROR: COSTS OF SAILING BALLAST START NOT FOUND: [" + vessel + "][" + route + "]");
  	}
  	*/
  	
  	return costs;
  }
  
  
  
  public static double getHighestStartCosts(String vessel, String route, ArrayList<SailingBallastStart> sailingBallastStarts){
  	double costs = 999999999999999.9;
  	int length = sailingBallastStarts.size();
  	
  	for (int n = 0; n < length; n++){
  		if (vessel == sailingBallastStarts.get(n).vessel && route == sailingBallastStarts.get(n).route && sailingBallastStarts.get(n).costs < costs){
  			costs = sailingBallastStarts.get(n).costs;
  		}
  	}
  	
  	return costs;
  }
  
  
  
  /*
   * ----------------------- Method to get the duration of an initial ballast sailing for a vessel at a given speed ------------------------
   */
  
  public static double getStartDuration(String vessel, String route, String speedProfileCode, ArrayList<SailingBallastStart> sailingBallastStarts){
  	double duration = -1.00;
  	
  	for (int n = 0; n < sailingBallastStarts.size(); n++){
  		if (vessel.equals(sailingBallastStarts.get(n).vessel) && (route.equals(sailingBallastStarts.get(n).route)) && (speedProfileCode.equals(sailingBallastStarts.get(n).speedProfileCode))){
  			duration = sailingBallastStarts.get(n).duration;
  		}
  	}
  	
  	if (duration < 0){
  		System.out.println("ERROR: DURATION OF BALLAST SAILING START NOT FOUND");
  	}
  	
  	return duration;
  }
  
  
  
  /*
   *  ----------------------------------- Method to retrieve shortest start ballast sailing for a vessel to a voyage -----------------------------
   */
  
  public static double getShortestSailingBallastStartDuration(String vessel, String route, ArrayList<SailingBallastStart> sailingBallastStarts){
  	double duration = -1.00;
  	
  	for (int n = 0; n < sailingBallastStarts.size(); n++){
  		if (duration < 0 && vessel.equals(sailingBallastStarts.get(n).vessel) && route.equals(sailingBallastStarts.get(n).route)){
  			duration = sailingBallastStarts.get(n).duration;
  		}
  		else if (vessel.equals(sailingBallastStarts.get(n).vessel) && route.equals(sailingBallastStarts.get(n).route) && sailingBallastStarts.get(n).duration < duration){
  			duration = sailingBallastStarts.get(n).duration;
  		}
  	}
  	return duration;
  }
  
  
	public static ArrayList<SailingBallastStart> importSailingBallastStarts(HSSFWorkbook workbook){
  	HSSFSheet sailingBallastStartSheet = workbook.getSheet("SailingBallastStart");
  	int numberOfRowsInSailingBallastStartSheet = sailingBallastStartSheet.getLastRowNum();
		
		ArrayList<SailingBallastStart> sailingBallastStarts = new ArrayList<SailingBallastStart>();
				
		for (int rowNum = 1; rowNum <= numberOfRowsInSailingBallastStartSheet; rowNum++){
			Row row = sailingBallastStartSheet.getRow(rowNum);
			Cell c = row.getCell(0);
			
			if (c != null){
			Cell sailingBallastStartCode = row.getCell(0);
			sailingBallastStartCode.setCellType(Cell.CELL_TYPE_STRING);
		  Cell sailingBallastStartVessel = row.getCell(1);
		  sailingBallastStartVessel.setCellType(Cell.CELL_TYPE_STRING);
		  Cell sailingBallastStartRoute = row.getCell(2);
		  sailingBallastStartRoute.setCellType(Cell.CELL_TYPE_STRING);
		  Cell sailingBallastStartSpeedProfileCode = row.getCell(3);
		  sailingBallastStartSpeedProfileCode.setCellType(Cell.CELL_TYPE_STRING);
		  Cell sailingBallastStartDuration = row.getCell(4);			  
		  Cell sailingBallastStartCosts = row.getCell(5);
			
		  sailingBallastStarts.add(new SailingBallastStart(
					(String)sailingBallastStartCode.getStringCellValue(),
					(String)sailingBallastStartVessel.getStringCellValue(),
					(String)sailingBallastStartRoute.getStringCellValue(),
					(String)sailingBallastStartSpeedProfileCode.getStringCellValue(),
					(double)sailingBallastStartDuration.getNumericCellValue()/100,
					(double)sailingBallastStartCosts.getNumericCellValue()/100));
				}
		 	}
		return sailingBallastStarts;
  }
  
}
